/***************************************************************************** 
 *  Copyright 1998 National Instruments Corporation.  All Rights Reserved.   * 
 *****************************************************************************/
								
/*****************************************************************************
 *  Chroma 19032 Electrical Safety Analyzer Instrument Driver                               
 *  LabWindows/CVI 5.0 Instrument Driver                                     
 *  Original Release: 2003~109                                  
 *  By: ROCK, CHROMA                              
 *      PH. 03-3279999                                 
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       2011~111 - Instrument Driver Created.                  
 *                                                                           
 *****************************************************************************/

#include <userint.h>
#include <ansi_c.h>
#include <utility.h>
#include <gpib.h>
#include <string.h>
#include <stdio.h>  
#include <formatio.h>
#include "C19032P.h"

/*****************************************************************************
 *--------------------- Hidden Attribute Declarations -----------------------*
 *****************************************************************************/

#define C19032P_ATTR_OPC_TIMEOUT      (IVI_SPECIFIC_PRIVATE_ATTR_BASE + 1L)   /* ViInt32 */

/*****************************************************************************
 *---------------------------- Useful Macros --------------------------------*
 *****************************************************************************/

    /*- I/O buffer size -----------------------------------------------------*/
#define BUFFER_SIZE                             512L
#define AUTO_BUFFER_SIZE                        -1
#define C19032P_IO_SESSION_TYPE   				IVI_VAL_VISA_SESSION_TYPE

    /*- 488.2 Event Status Register (ESR) Bits ------------------------------*/
#define IEEE_488_2_QUERY_ERROR_BIT              0x04
#define IEEE_488_2_DEVICE_DEPENDENT_ERROR_BIT   0x08
#define IEEE_488_2_EXECUTION_ERROR_BIT          0x10
#define IEEE_488_2_COMMAND_ERROR_BIT            0x20

/* I/O DELAY TIME -----------------------------------------------------------*/
#define	nDelayTime								0  //second
#define	ComparePrecision						0.0000000001
#define STEP_MODE								6
#define MAX_SCAN_CH								8


    /*- List of channels passed to the Ivi_BuildChannelTable function -------*/ 
#define CHANNEL_LIST                            "1"

static IviRangeTableEntry attrDcCurrentOffset_Lower_RangeTableEntries[] =
	{
		{0, 0.00002999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDcCurrentOffset_Lower_RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDcCurrentOffset_Lower_RangeTableEntries,
	};

static IviRangeTableEntry attrDcCurrentOffset_Lowest_RangeTableEntries[] =
	{
		{0, 0.000002999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDcCurrentOffset_Lowest_RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDcCurrentOffset_Lowest_RangeTableEntries,
	};

static IviRangeTableEntry ChrLcPowerCurrLimit040506RangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.001, 10, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrLcPowerCurrLimit040506RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrLcPowerCurrLimit040506RangeTableEntries,
	};

static IviRangeTableEntry ChrLcPowerVALimit040506RangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.1, 2200, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrLcPowerVALimit040506RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrLcPowerVALimit040506RangeTableEntries,
	};

static IviRangeTableEntry attrDCModeDcLimitHighRangeTableEntries[] =
	{
		{0.000000001, 0.025, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDCModeDcLimitHighRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDCModeDcLimitHighRangeTableEntries,
	};

static IviRangeTableEntry attrDCModeDcLimitLowRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.000000001, 0.025, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDCModeDcLimitLowRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDCModeDcLimitLowRangeTableEntries,
	};

static IviRangeTableEntry ChrAcLimitLowRangeTableEntries[] =
	{
		{0.000001, 0.1, 0, "", 1},
		{0, 0, 0, "", 2},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrAcLimitLowRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrAcLimitLowRangeTableEntries,
	};

static IviRangeTableEntry attrLcPowerSourPfactorRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.001, 0.999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcPowerSourPfactorRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrLcPowerSourPfactorRangeTableEntries,
	};

static IviRangeTableEntry AttrLcPowSouVALimit040506RangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{1, 2200, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable AttrLcPowSouVALimit040506RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        AttrLcPowSouVALimit040506RangeTableEntries,
	};

static IviRangeTableEntry AttrLcPowSouVALimit0708RangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{1, 4400, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable AttrLcPowSouVALimit0708RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        AttrLcPowSouVALimit0708RangeTableEntries,
	};

static IviRangeTableEntry AttrLcPowSouCurrLimit0708RangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.001, 20, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable AttrLcPowSouCurrLimit0708RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        AttrLcPowSouCurrLimit0708RangeTableEntries,
	};

static IviRangeTableEntry AttrLcPowSouCurrLimit040506RangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.001, 10, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable AttrLcPowSouCurrLimit040506RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        AttrLcPowSouCurrLimit040506RangeTableEntries,
	};

static IviRangeTableEntry attrLcTimeDwellRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.1, 999.0, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcTimeDwellRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrLcTimeDwellRangeTableEntries,
	};

static IviRangeTableEntry attrLCDisplayRangeTableEntries[] =
	{
		{C19032P_VAL_LC_DISP_RMS, 0, 0, "RMS", 0},
		{C19032P_VAL_LC_DISP_PEAK, 0, 0, "PEAK", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLCDisplayRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrLCDisplayRangeTableEntries,
	};

static IviRangeTableEntry attrOscRangeSelectRangeTableEntries[] =
	{
		{0, 2, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrOscRangeSelectRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrOscRangeSelectRangeTableEntries,
	};

static IviRangeTableEntry ChrPresetGFIRangeTableEntries[] =
	{
		{C19032P_VAL_GFII_ON, 0, 0, "ON", 0},
		{C19032P_VAL_GFI_OFF, 0, 0, "OFF", 0},
		{C19032P_VAL_GFI_FLOAT, 0, 0, "FLOAT", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrPresetGFIRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrPresetGFIRangeTableEntries,
	};

static IviRangeTableEntry attrIrCurrHighRangeRangeTableEntries[] =
	{
		{0, 0.01, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrIrCurrHighRangeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrIrCurrHighRangeRangeTableEntries,
	};

static IviRangeTableEntry attrIrCurrentLowRangeRangeTableEntries[] =
	{
		{0, 0.01, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrIrCurrentLowRangeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrIrCurrentLowRangeRangeTableEntries,
	};

static IviRangeTableEntry attrDcCurrentOffset_HighRangeTableEntries[] =
	{
		{0, 0.025, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDcCurrentOffset_HighRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDcCurrentOffset_HighRangeTableEntries,
	};

static IviRangeTableEntry attrDcCurrentOffset_MidRangeTableEntries[] =
	{
		{0, 0.002999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDcCurrentOffset_MidRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDcCurrentOffset_MidRangeTableEntries,
	};

static IviRangeTableEntry attrDcCurrentOffset_LowRangeTableEntries[] =
	{
		{0, 0.0002999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDcCurrentOffset_LowRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDcCurrentOffset_LowRangeTableEntries,
	};

static IviRangeTableEntry attrAcCurrentOffset_HighRangeTableEntries[] =
	{
		{0, 0.1, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrAcCurrentOffset_HighRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrAcCurrentOffset_HighRangeTableEntries,
	};

static IviRangeTableEntry attrAcCurrentOffset_MidRangeTableEntries[] =
	{
		{0, 0.02999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrAcCurrentOffset_MidRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrAcCurrentOffset_MidRangeTableEntries,
	};

static IviRangeTableEntry attrAcCurrentOffset_LowRangeTableEntries[] =
	{
		{0, 0.002999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrAcCurrentOffset_LowRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrAcCurrentOffset_LowRangeTableEntries,
	};

static IviRangeTableEntry attrAcFrequencyRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{50, 600, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrAcFrequencyRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrAcFrequencyRangeTableEntries,
	};

static IviRangeTableEntry attrGbCurrentOffsetRangeTableEntries[] =
	{
		{0.000, 0.5, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrGbCurrentOffsetRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrGbCurrentOffsetRangeTableEntries,
	};

static IviRangeTableEntry attrAcLimitRealRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.000001, 0.12, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrAcLimitRealRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrAcLimitRealRangeTableEntries,
	};

static IviRangeTableEntry attrModelRangeTableEntries[] =
	{
		{C19032P_VAL_MODEL_QUADTECH, 0, 0, "", 0},
		{C19032P_VAL_MODEL_19031, 0, 0, "", 0},
		{C19032P_VAL_MODEL_19032, 0, 0, "", 0},
		{C19032P_VAL_MODEL_19032P, 0, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrModelRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrModelRangeTableEntries,
	};

static IviRangeTableEntry attrDcArcLimitRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.001, 0.01, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDcArcLimitRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDcArcLimitRangeTableEntries,
	};

static IviRangeTableEntry attrIrCurrRangeUpperRangeTableEntries[] =
	{
		{0.0, 299.9e-9, 200.0e-9, "300nA", 0},
		{299.9e-9, 2.99e-6, 2.0e-6, "3uA", 0},
		{2.99e-6, 29.99e-6, 20.00e-6, "30uA", 0},
		{29.99e-6, 299.99e-6, 200.0e-6, "300uA", 0},
		{299.99e-6, 2.99e-3, 2.0e-3, "3mA", 0},
		{2.99e-3, 10.0e-3, 9.0e-3, "10mA", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrIrCurrRangeUpperRangeTable =
	{
		IVI_VAL_COERCED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrIrCurrRangeUpperRangeTableEntries,
	};

static IviRangeTableEntry attrLcPowerSourTargetFreqRangeTableEntries[] =
	{
		{45, 65, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcPowerSourTargetFreqRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrLcPowerSourTargetFreqRangeTableEntries,
	};

static IviRangeTableEntry attrOptionScanNumRangeTableEntries[] =
	{
		{0, 8, 1, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrOptionScanNumRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrOptionScanNumRangeTableEntries,
	};

static IviRangeTableEntry attrOptionGbRateRangeTableEntries[] =
	{
		{30, 0, 0, "30:30", 0},
		{40, 0, 0, "30:40", 0},
		{45, 0, 0, "30:45", 0},
		{60, 0, 0, "30:60", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrOptionGbRateRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrOptionGbRateRangeTableEntries,
	};

static IviRangeTableEntry attrOptionScanTypeRangeTableEntries[] =
	{
		{C19032P_VAL_NULL, 0, 0, "", 0},
		{C19032P_VAL_GB_FLOATING, 0, 0, "", 0},
		{C19032P_VAL_6000_01, 0, 0, "", 0},
		{C19032P_VAL_6000_02, 0, 0, "", 0},
		{C19032P_VAL_6000_03, 0, 0, "", 0},
		{C19032P_VAL_6000_04, 0, 0, "", 0},
		{C19032P_VAL_6000_06, 0, 0, "", 0},
		{C19032P_VAL_9030A, 0, 0, "", 0},
		{C19032P_VAL_6000_04_FLOATING, 0, 0, "", 0},
		{C19032P_VAL_6000_06_FLOATING, 0, 0, "", 0},
		{C19032P_VAL_6000_11, 0, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrOptionScanTypeRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrOptionScanTypeRangeTableEntries,
	};

static IviRangeTableEntry attrLcGrSwitchStateRangeTableEntries[] =
	{
		{0, 0, 0, "OPEN", 0},
		{1, 0, 0, "CLOSE", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcGrSwitchStateRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrLcGrSwitchStateRangeTableEntries,
	};

static IviRangeTableEntry attrOscOpenCheckRangeTableEntries[] =
	{
		{0.10, 0.15, 0.1, "", 0},
		{0.15, 0.25, 0.2, "", 0},
		{0.25, 0.35, 0.3, "", 0},
		{0.35, 0.45, 0.4, "", 0},
		{0.45, 0.55, 0.5, "", 0},
		{0.55, 0.65, 0.6, "", 0},
		{0.65, 0.75, 0.7, "", 0},
		{0.75, 0.85, 0.8, "", 0},
		{0.85, 0.95, 0.9, "", 0},
		{0.95, 1.00, 1.0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrOscOpenCheckRangeTable =
	{
		IVI_VAL_COERCED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrOscOpenCheckRangeTableEntries,
	};

static IviRangeTableEntry attrOscShortCheckRangeTableEntries[] =
	{
		{0.0, 0.0, 0.0, "", 0},
		{1.0, 1.5, 1.0, "", 0},
		{1.5, 2.5, 2.0, "", 0},
		{2.5, 3.5, 3.0, "", 0},
		{3.5, 4.5, 4.0, "", 0},
		{4.5, 5.0, 5.0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrOscShortCheckRangeTable =
	{
		IVI_VAL_COERCED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrOscShortCheckRangeTableEntries,
	};

static IviRangeTableEntry attrDcDwellTimeRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.1, 999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDcDwellTimeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDcDwellTimeRangeTableEntries,
	};

static IviRangeTableEntry ChrDcLimitHighRangeTableEntries[] =
	{
		{0.0001e-3, 12e-3, 0, "", 0},
		{0.0001e-3, 20e-3, 0, "", 1},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrDcLimitHighRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrDcLimitHighRangeTableEntries,
	};

static IviRangeTableEntry attrDcVoltageRangeTableEntries[] =
	{
		{50, 6000, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDcVoltageRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDcVoltageRangeTableEntries,
	};

static IviRangeTableEntry ChrArcFilterRangeTableEntries[] =
	{
		{0, 0, 0, "23000", 0},
		{1, 0, 0, "50000", 0},
		{2, 0, 0, "100000", 0},
		{3, 0, 0, "230000", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrArcFilterRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrArcFilterRangeTableEntries,
	};

static IviRangeTableEntry attrAcArcLimitRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.001, 0.02, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrAcArcLimitRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrAcArcLimitRangeTableEntries,
	};

static IviRangeTableEntry ChrAcLimitHighRangeTableEntries[] =
	{
		{0.000001, 0.04, 0, "", 0},
		{0.000001, 0.1, 0, "", 1},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrAcLimitHighRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrAcLimitHighRangeTableEntries,
	};

static IviRangeTableEntry ChrGbCurrentRangeTableEntries[] =
	{
		{1, 30, 30, "", 30},
		{1, 40, 40, "", 40},
		{1, 45, 45, "", 45},
		{1, 60, 60, "", 60},
		{3, 40, 0, "", 1},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrGbCurrentRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrGbCurrentRangeTableEntries,
	};

static IviRangeTableEntry attrStepModeTypeRangeTableEntries[] =
	{
		{C19032P_VAL_MODE_GB, 0, 0, "GB", 0},
		{C19032P_VAL_MODE_AC, 0, 0, "AC", 0},
		{C19032P_VAL_MODE_DC, 0, 0, "DC", 0},
		{C19032P_VAL_MODE_IR, 0, 0, "IR", 0},
		{C19032P_VAL_MODE_LC, 0, 0, "LC", 0},
		{C19032P_VAL_MODE_PA, 0, 0, "PA", 0},
		{C19032P_VAL_MODE_OSC, 0, 0, "OSC", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrStepModeTypeRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrStepModeTypeRangeTableEntries,
	};

static IviRangeTableEntry attrPresetStartWaitRangeTableEntries[] =
	{
		{0, 0, 0, "OFF", 0},
		{0.1, 99.9, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrPresetStartWaitRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrPresetStartWaitRangeTableEntries,
	};

static IviRangeTableEntry ChrOnOffRangeTableEntries[] =
	{
		{0, 0, 0, "OFF", 0},
		{1, 0, 0, "ON", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrOnOffRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrOnOffRangeTableEntries,
	};

static IviRangeTableEntry attrPresetGbFrequencyRangeTableEntries[] =
	{
		{50, 0, 0, "50", 0},
		{60, 0, 0, "60", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrPresetGbFrequencyRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrPresetGbFrequencyRangeTableEntries,
	};

static IviRangeTableEntry attrPresetAcFrequencyRangeTableEntries[] =
	{
		{50, 600, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrPresetAcFrequencyRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrPresetAcFrequencyRangeTableEntries,
	};

static IviRangeTableEntry attrLcMeterMethodRangeTableEntries[] =
	{
		{C19032P_VAL_LC_METER_LG, 0, 0, "L,G", 0},
		{C19032P_VAL_LC_METER_LP, 0, 0, "L,P", 0},
		{C19032P_VAL_LC_METER_PP, 0, 0, "P,P", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcMeterMethodRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrLcMeterMethodRangeTableEntries,
	};

static IviRangeTableEntry ChrRampFallTimeRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.1, 999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrRampFallTimeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrRampFallTimeRangeTableEntries,
	};

static IviRangeTableEntry attrIrLimitLowRangeTableEntries[] =
	{
		{0.1e+6, 50000e+6, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrIrLimitLowRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrIrLimitLowRangeTableEntries,
	};

static IviRangeTableEntry attrIrVoltageRangeTableEntries[] =
	{
		{50, 1000, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrIrVoltageRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrIrVoltageRangeTableEntries,
	};

static IviRangeTableEntry attrPresetGbVoltageRangeTableEntries[] =
	{
		{6.0, 15.0, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrPresetGbVoltageRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrPresetGbVoltageRangeTableEntries
	};

static IviRangeTableEntry attrPresetStepHoldTimeRangeTableEntries[] =
	{
		{0, 0, 0, "KEY", 0},
		{0.1, 99.9, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrPresetStepHoldTimeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrPresetStepHoldTimeRangeTableEntries,
	};

static IviRangeTableEntry attrPresetPassHoldTimeRangeTableEntries[] =
	{
		{0.2, 99.9, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrPresetPassHoldTimeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrPresetPassHoldTimeRangeTableEntries,
	};

static IviRangeTableEntry attrLcPowerSimuTargetVoltRangeTableEntries[] =
	{
		{10, 300.00, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcPowerSimuTargetVoltRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrLcPowerSimuTargetVoltRangeTableEntries,
	};

static IviRangeTableEntry ChrLcLimitHighRangeTableEntries[] =
	{
		{0.0000001, 0.006, 0, "", 0},
		{0.0000001, 0.01, 0, "", 1},
		{0.0000001, 0.01, 0, "", 2},
		{0.0000001, 0.01, 0, "", 3},
		{0.0000001, 0.01, 0, "", 4},
		{0.0000001, 0.07, 0, "", 5},
		{0.0000001, 0.01, 0, "", 6},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrLcLimitHighRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrLcLimitHighRangeTableEntries,
	};

static IviRangeTableEntry ChrLcPowerVALimit0708RangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.1, 4400, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrLcPowerVALimit0708RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrLcPowerVALimit0708RangeTableEntries,
	};

static IviRangeTableEntry ChrLcPowerCurrLimit0708RangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.001, 20, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrLcPowerCurrLimit0708RangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrLcPowerCurrLimit0708RangeTableEntries,
	};

static IviRangeTableEntry ChrLcPowerVoltLimitRangeTableEntries[] =
	{
		{0, 0, 0, "", 0},
		{0.1, 300, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrLcPowerVoltLimitRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrLcPowerVoltLimitRangeTableEntries,
	};

static IviRangeTableEntry attrLcLineInputRangeTableEntries[] =
	{
		{C19032P_VAL_LC_LINE_NORMAL, 0, 0, "NOR", 0},
		{C19032P_VAL_LC_LINE_REVERSE, 0, 0, "REV", 0},
		{C19032P_VAL_LC_LINE_SFNORMAL, 0, 0, "SFN", 0},
		{C19032P_VAL_LC_LINE_SFREVERSE, 0, 0, "SFR", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcLineInputRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrLcLineInputRangeTableEntries,
	};

static IviRangeTableEntry attrLcDeviceTypeRangeTableEntries[] =
	{
		{C19032P_VAL_LC_DEVICE_UL544NP, 0, 0, "UL544NP", 0},
		{C19032P_VAL_LC_DEVICE_UL544P, 0, 0, "UL544P", 0},
		{C19032P_VAL_LC_DEVICE_UL1563, 0, 0, "UL1563", 0},
		{C19032P_VAL_LC_DEVICE_UL2601, 0, 0, "UL2601", 0},
		{C19032P_VAL_LC_DEVICE_UL1950, 0, 0, "UL1950", 0},
		{C19032P_VAL_LC_DEVICE_1950U1, 0, 0, "1950-U1", 0},
		{C19032P_VAL_LC_DEVICE_2601U1, 0, 0, "2601-U1", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcDeviceTypeRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_FALSE,
        VI_FALSE,
        VI_NULL,
        attrLcDeviceTypeRangeTableEntries,
	};

static IviRangeTableEntry attrLcPowerModeRangeTableEntries[] =
	{
		{C19032P_VAL_LC_POWER_VOLTAGE, 0, 0, "VOLT", 0},
		{C19032P_VAL_LC_POWER_CURRENT, 0, 0, "CURR", 0},
		{C19032P_VAL_LC_POWER_VA, 0, 0, "VA", 0},
		{C19032P_VAL_LC_POWER_SIMULATE, 0, 0, "SIMU", 0},
		{C19032P_VAL_LC_POWER_SOURCE, 0, 0, "SOUR", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrLcPowerModeRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_FALSE,
        VI_FALSE,
        VI_NULL,
        attrLcPowerModeRangeTableEntries,
	};

static IviRangeTableEntry attrStepCountRangeTableEntries[] =
	{
		{0, 100, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrStepCountRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrStepCountRangeTableEntries,
	};

static IviRangeTableEntry attrAcVoltageRangeTableEntries[] =
	{
		{50, 5000, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrAcVoltageRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrAcVoltageRangeTableEntries,
	};

static IviRangeTableEntry ChrTestTimeRangeTableEntries[] =
	{
		{0, 0, 0, "CONT", 0},
		{0.3, 999, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrTestTimeRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrTestTimeRangeTableEntries,
	};

static IviRangeTableEntry attrDeleteStepRangeTableEntries[] =
	{
		{1, 100, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable attrDeleteStepRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        attrDeleteStepRangeTableEntries
	};

static IviRangeTableEntry ChrMemIndexRangeTableEntries[] =
	{
		{1, 100, 0, "", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrMemIndexRangeTable =
	{
		IVI_VAL_RANGED,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrMemIndexRangeTableEntries,
	};

static IviRangeTableEntry ChrChannelOptionRangeTableEntries[] =
	{
		{C19032P_VAL_X, 0, 0, "X", 0},
		{C19032P_VAL_LX, 0, 0, "L,X", 0},
		{C19032P_VAL_HX, 0, 0, "H,X", 0},
		{C19032P_VAL_HLX, 0, 0, "H,L,X", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrChannelOptionRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrChannelOptionRangeTableEntries,
	};

static IviRangeTableEntry ChrFetchValueTypeRangeTableEntries[] =
	{
		{C19032P_VAL_FETCH_STEP, 0, 0, "STEP", 0},
		{C19032P_VAL_FETCH_MODE, 0, 0, "MODE", 0},
		{C19032P_VAL_FETCH_OMET, 0, 0, "OMET", 0},
		{C19032P_VAL_FETCH_MMET, 0, 0, "MMET", 0},
		{C19032P_VAL_FETCH_RELA, 0, 0, "RELA", 0},
		{C19032P_VAL_FETCH_RLEA, 0, 0, "RLEA", 0},
		{C19032P_VAL_FETCH_TELA, 0, 0, "TELA", 0},
		{C19032P_VAL_FETCH_TLEA, 0, 0, "TLEA", 0},
		{C19032P_VAL_FETCH_DELA, 0, 0, "DELA", 0},
		{C19032P_VAL_FETCH_DLEA, 0, 0, "DLEA", 0},
		{C19032P_VAL_FETCH_FELA, 0, 0, "FELA", 0},
		{C19032P_VAL_FETCH_FLEA, 0, 0, "FLEA", 0},
		{C19032P_VAL_FETCH_LACMET, 0, 0, "LACMET", 0},
		{C19032P_VAL_FETCH_LDCMET, 0, 0, "LDCMET", 0},
		{IVI_RANGE_TABLE_LAST_ENTRY}
	};

static IviRangeTable ChrFetchValueTypeRangeTable =
	{
		IVI_VAL_DISCRETE,
        VI_TRUE,
        VI_TRUE,
        VI_NULL,
        ChrFetchValueTypeRangeTableEntries,
	};

static ViBoolean gbRS232existed;
static ViInt32	 gt_nBaudRate, gt_nParity, gt_nDataBits, gt_nStopBits;

static ViInt32 iGB_FloatingChState[STEP_MODE][MAX_SCAN_CH] =
			   {{-1, -1, C19032P_VAL_X,   -1, -1, -1, -1, -1},
			    {-1, -1, C19032P_VAL_HLX, -1, -1, -1, -1, -1},
			    {-1, -1, C19032P_VAL_HLX, -1, -1, -1, -1, -1},
			    {-1, -1, C19032P_VAL_HLX, -1, -1, -1, -1, -1},
			    {-1, -1, C19032P_VAL_LX,  -1, -1, -1, -1, -1}, 
			    {-1, -1, C19032P_VAL_HLX, -1, -1, -1, -1, -1}};

static ViInt32 i6000_01ChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X}, 
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X}, 
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}};

static ViInt32 i6000_02ChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X}, 
			    {C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}, 
			    {C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}, 
			    {C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}, 
			    {C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X}, 
			    {C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}};

static ViInt32 i6000_03ChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X},
			    {C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}, 
			    {C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X}, 
			    {C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}};

static ViInt32 i6000_04ChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_X,  -1, -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, -1, -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, -1, -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, -1, -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_X,  -1, -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, -1, -1, -1, -1, -1, -1, -1}};

static ViInt32 i6000_06ChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_X,  C19032P_VAL_X,   -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, C19032P_VAL_HLX, -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, C19032P_VAL_HLX, -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, C19032P_VAL_HLX, -1, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_X,  C19032P_VAL_X,   -1, -1, -1, -1, -1, -1},
			    {C19032P_VAL_HX, C19032P_VAL_HLX, -1, -1, -1, -1, -1, -1}};

static ViInt32 i9030AChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_HX,  C19032P_VAL_HX,  C19032P_VAL_HX,  C19032P_VAL_HX,  C19032P_VAL_HX,  C19032P_VAL_HX,  C19032P_VAL_HX,  C19032P_VAL_HX}, 
			    {C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}, 
			    {C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}, 
			    {C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_LX,  C19032P_VAL_LX,  C19032P_VAL_LX,  C19032P_VAL_LX,  C19032P_VAL_LX,  C19032P_VAL_LX,  C19032P_VAL_LX,  C19032P_VAL_LX}, 
			    {C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}};

static ViInt32 i6000_04FloatingChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_X,  -1, C19032P_VAL_X,   -1, -1, -1, -1, -1},
			    {C19032P_VAL_HX, -1, C19032P_VAL_HLX, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, -1, C19032P_VAL_HLX, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, -1, C19032P_VAL_HLX, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_X,  -1, C19032P_VAL_LX,  -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, -1, C19032P_VAL_HLX, -1, -1, -1, -1, -1}};

static ViInt32 i6000_06FloatingChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_X,  C19032P_VAL_X,   C19032P_VAL_X,   -1, -1, -1, -1, -1},
			    {C19032P_VAL_HX, C19032P_VAL_HLX, C19032P_VAL_HLX, -1, -1, -1, -1, -1},
			    {C19032P_VAL_HX, C19032P_VAL_HLX, C19032P_VAL_HLX, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, C19032P_VAL_HLX, C19032P_VAL_HLX, -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_X,  C19032P_VAL_X,   C19032P_VAL_LX,  -1, -1, -1, -1, -1}, 
			    {C19032P_VAL_HX, C19032P_VAL_HLX, C19032P_VAL_HLX, -1, -1, -1, -1, -1}};   
			    
static ViInt32 i6000_11ChState[STEP_MODE][MAX_SCAN_CH] =
			   {{C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_HX, C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X}, 
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX},
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X,   C19032P_VAL_X}, 
			    {C19032P_VAL_X, 	C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_X,  C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX, C19032P_VAL_HLX}};
			    
static ViStatus C19032P_InitAttributes (ViSession vi);
static ViStatus C19032P_DefaultInstrSetup (ViSession openInstrSession);


    /*- File I/O Utility Functions -*/
static ViStatus C19032P_ReadToFile (ViSession vi, ViConstString filename, 
                                     ViInt32 maxBytesToRead, ViInt32 fileAction, ViInt32 *totalBytesWritten);
static ViStatus C19032P_WriteFromFile (ViSession vi, ViConstString filename, 
                                        ViInt32 maxBytesToWrite, ViInt32 byteOffset, 
                                        ViInt32 *totalBytesWritten);

/*****************************************************************************
 *----------------- Callback Declarations (Non-Exported) --------------------*
 *****************************************************************************/

    /*- Global Session Callbacks --------------------------------------------*/
    



    /*- Attribute callbacks -------------------------------------------------*/

static ViStatus _VI_FUNC C19032PAttrDriverRevision_ReadCallback (ViSession vi,
                                                                  ViSession io, 
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId, 
                                                                  const ViConstString cacheValue);
static ViStatus _VI_FUNC C19032PAttrInstrumentManufacturer_ReadCallback (ViSession vi, 
                                                                          ViSession io,
                                                                          ViConstString channelName, 
                                                                          ViAttr attributeId,
                                                                          const ViConstString cacheValue);
static ViStatus _VI_FUNC C19032PAttrInstrumentModel_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrIdQueryResponse_ReadCallback (ViSession vi,
                                                                   ViSession io, 
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId, 
                                                                   const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrMemDel_WriteCallback (ViSession vi,
                                                         ViSession io,
                                                         ViConstString channelName,
                                                         ViAttr attributeId,
                                                         ViInt32 value);


static ViStatus _VI_FUNC C19032PAttrSysVersion_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrTestState_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViBoolean value);

static ViStatus _VI_FUNC C19032PAttrOffsetState_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrRunCompleteStatus_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrAutoResultState_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrAutoResultState_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrDeleteStep_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrStepSetting_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             const ViConstString cacheValue);
												  
static ViStatus _VI_FUNC C19032PAttrGbCurrent_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrGbCurrent_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrGbLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrGbLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrGbLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrGbLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrGbTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrGbTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrGbTwinportState_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrGbTwinportState_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrStepModeType_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrAcVoltage_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcVoltage_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLloState_WriteCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViBoolean value);

static ViStatus _VI_FUNC C19032PAttrAcLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrAcLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrAcArcLimit_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcArcLimit_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrAcArcFilter_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrAcArcFilter_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrAcRampTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcRampTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrAcTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcVoltage_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrDcVoltage_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrDcLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrDcLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcArcLimit_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrDcArcLimit_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcArcFilter_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrDcArcFilter_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrDcRampTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrDcRampTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrDcTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrVoltage_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrVoltage_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrRampTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrRampTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerMode_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerMode_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrLcDeviceType_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrLcDeviceType_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrLcLineInput_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrLcLineInput_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrLcLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitHigh_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitHigh_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitLow_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitLow_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitHigh_RangeTableCallback (ViSession vi,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcPowerSimuTargetVolt_ReadCallback (ViSession vi,
                                                                       ViSession io,
                                                                       ViConstString channelName,
                                                                       ViAttr attributeId,
                                                                       ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSimuTargetVolt_WriteCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrPresetPassHoldTime_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrPresetPassHoldTime_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrPresetStepHoldTime_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrPresetStepHoldTime_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrPresetGbFrequency_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrPresetGbFrequency_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrPresetGbVoltage_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrPresetGbVoltage_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrPresetAcFrequency_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrPresetAcFrequency_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrPresetAutoRange_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrPresetAutoRange_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrPresetSoftAgcState_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrPresetSoftAgcState_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrPresetPartNumber_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrPresetPartNumber_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViConstString value);

static ViStatus _VI_FUNC C19032PAttrPresetLotNumber_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrPresetLotNumber_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViConstString value);

static ViStatus _VI_FUNC C19032PAttrPresetSerialNumber_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrPresetSerialNumber_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViConstString value);
																		   
static ViStatus _VI_FUNC C19032PAttrMemNameDefine_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViConstString value);

static ViStatus _VI_FUNC C19032PAttrLcMeterMethod_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrLcMeterMethod_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrLcGrSwitchState_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrLcGrSwitchState_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrQueryMemIndexNum_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrPauseMessageString_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrPauseMessageString_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViConstString value);

static ViStatus _VI_FUNC C19032PAttrPauseSignalState_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViBoolean *value);

static ViStatus _VI_FUNC C19032PAttrPauseSignalState_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViBoolean value);

static ViStatus _VI_FUNC C19032PAttrPresetIecState_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrPresetIecState_WriteCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrPresetStartWait_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrPresetStartWait_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcDwellTime_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrOscOpenCheck_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrGbLimitHigh_RangeTableCallback (ViSession vi,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrGbLimitLow_RangeTableCallback (ViSession vi,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrAcLimitLow_RangeTableCallback (ViSession vi,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  IviRangeTablePtr *rangeTablePtr);


static ViStatus _VI_FUNC C19032PAttrIrLimitLow_RangeTableCallback (ViSession vi,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrIrLimitHigh_RangeTableCallback (ViSession vi,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrOscShortCheck_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcLimitLow_RangeTableCallback (ViSession vi,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitLow_RangeTableCallback (ViSession vi,
                                                                       ViConstString channelName,
                                                                       ViAttr attributeId,
                                                                       IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcLimitHigh_RangeTableCallback (ViSession vi,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrStandardCapacitance_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViInt32 value);


static ViStatus _VI_FUNC C19032PAttrGbCurrent_RangeTableCallback (ViSession vi,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrOptionGbRate_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrOptionScanCard_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrOptionGbFloating_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrOptionScanNum_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrPauseTestTime_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourTargetFreq_WriteCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourTargetVolt_WriteCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrOptionUseSource_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrAcFallTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcFallTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrCurrRangeUpper_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrFallTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcAcHighLimit_RangeTableCallback (ViSession vi,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcAcHighLimit_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcDcHighLimit_RangeTableCallback (ViSession vi,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcDcHighLimit_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrPresetRampJudgement_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrRemoteStartKey_WriteCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrIrCurrRangeAuto_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value);


static ViStatus _VI_FUNC C19032PAttrAcLimitHigh_RangeTableCallback (ViSession vi,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrQueryMemMaxCapacity_ReadCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrFuncMode_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViBoolean *value);

static ViStatus _VI_FUNC C19032PAttrFuncMode_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViBoolean value);

static ViStatus _VI_FUNC C19032PAttrAutoReportSaved_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViBoolean *value);

static ViStatus _VI_FUNC C19032PAttrAutoReportSaved_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViBoolean value);

static ViStatus _VI_FUNC C19032PAttrAcLimitReal_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcLimitReal_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrGbCurrentOffset_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrGbCurrentOffset_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrAcFrequency_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrAcFrequency_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 value);

static ViStatus _VI_FUNC C19032PAttrAcFallTime_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcCurrentOffset_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrAcCurrentOffset_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value);


static ViStatus _VI_FUNC C19032PAttrDcCurrentOffset_RangeTableCallback (ViSession vi,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrDcCurrentOffset_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrDcCurrentOffset_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrDcDwellTime_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrDcFallTime_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrFallTime_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrCurrentLowRange_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrCurrentLowRange_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrCurrRangeUpper_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrCurrHighRange_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrIrCurrHighRange_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrIrCurrRangeAuto_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrPauseTestTime_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrPresetGfi_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrPresetGfi_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViConstString value);

static ViStatus _VI_FUNC C19032PAttrPresetScreen_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViBoolean value);

static ViStatus _VI_FUNC C19032PAttrOscOpenCheck_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrOscShortCheck_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrOscChannelState_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrOscCrange_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrOscCstandard_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViConstString value);

static ViStatus _VI_FUNC C19032PAttrOscCurrentOffset_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViConstString value);

static ViStatus _VI_FUNC C19032PAttrLcDisplay_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            const ViConstString cacheValue);

static ViStatus _VI_FUNC C19032PAttrLcDisplay_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViConstString value);

static ViStatus _VI_FUNC C19032PAttrLcTimeDwell_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcTimeDwell_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentHigh_ReadCallback (ViSession vi,
                                                                         ViSession io,
                                                                         ViConstString channelName,
                                                                         ViAttr attributeId,
                                                                         ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentHigh_WriteCallback (ViSession vi,
                                                                          ViSession io,
                                                                          ViConstString channelName,
                                                                          ViAttr attributeId,
                                                                          ViReal64 value);


static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentHigh_RangeTableCallback (ViSession vi,
                                                                               ViConstString channelName,
                                                                               ViAttr attributeId,
                                                                               IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentLow_RangeTableCallback (ViSession vi,
                                                                              ViConstString channelName,
                                                                              ViAttr attributeId,
                                                                              IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentLow_ReadCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentLow_WriteCallback (ViSession vi,
                                                                         ViSession io,
                                                                         ViConstString channelName,
                                                                         ViAttr attributeId,
                                                                         ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourTargetFreq_ReadCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaHigh_RangeTableCallback (ViSession vi,
                                                                          ViConstString channelName,
                                                                          ViAttr attributeId,
                                                                          IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaHigh_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaHigh_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaLow_RangeTableCallback (ViSession vi,
                                                                         ViConstString channelName,
                                                                         ViAttr attributeId,
                                                                         IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaLow_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaLow_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerHigh_RangeTableCallback (ViSession vi,
                                                                             ViConstString channelName,
                                                                             ViAttr attributeId,
                                                                             IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerHigh_ReadCallback (ViSession vi,
                                                                       ViSession io,
                                                                       ViConstString channelName,
                                                                       ViAttr attributeId,
                                                                       ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerHigh_WriteCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerLow_RangeTableCallback (ViSession vi,
                                                                            ViConstString channelName,
                                                                            ViAttr attributeId,
                                                                            IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerLow_ReadCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerLow_WriteCallback (ViSession vi,
                                                                       ViSession io,
                                                                       ViConstString channelName,
                                                                       ViAttr attributeId,
                                                                       ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPfactor_ReadCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPfactor_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcUpm_ReadCallback (ViSession vi,
                                                        ViSession io,
                                                        ViConstString channelName,
                                                        ViAttr attributeId,
                                                        ViBoolean *value);

static ViStatus _VI_FUNC C19032PAttrLcUpm_WriteCallback (ViSession vi,
                                                         ViSession io,
                                                         ViConstString channelName,
                                                         ViAttr attributeId,
                                                         ViBoolean value);

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffset_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffset_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffsetLac_ReadCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffsetLac_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffsetLdc_ReadCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffsetLdc_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 value);

static ViStatus _VI_FUNC C19032PAttrLcAcHighLimit_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrLcDcHighLimit_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 *value);

static ViStatus _VI_FUNC C19032PAttrPresetRampJudgement_ReadCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViInt32 *value);

static ViStatus _VI_FUNC C19032PAttrAcCurrentOffset_RangeTableCallback (ViSession vi,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        IviRangeTablePtr *rangeTablePtr);


/*****************************************************************************
 *------------ User-Callable Functions (Exportable Functions) ---------------*
 *****************************************************************************/
ViStatus ModeCheck(ViSession vi,ViInt32 step,ViInt32 sMode)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		stepCount=0,iMode;
	ViChar		ChanId[4];
	
	Fmt( ChanId, "%s<%d", step);   
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != sMode)
	   	return VI_ERROR_INVALID_MODE_COMMAND;

Error:
	return error;
}
 
ViStatus _VI_FUNC C19032P_GetError_IfTrigStart (ViSession vi)
                                       
 {
	ViStatus	error = VI_SUCCESS; 
	ViBoolean   startTest;
	ViInt32     runningCompleted;
	
	viCheckErr( Ivi_GetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_TEST_STATE, 0, &startTest));
	
	if (startTest)
	{

 		viCheckErr(C19032P_GetRunCompleted (vi,&runningCompleted));
 		
 		if(runningCompleted)
 			viCheckErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_TEST_STATE, IVI_VAL_SET_CACHE_ONLY, 0));   	

		error=VI_ERROR_COMMAND_PROTECTED;		 
			 
	}		 
	

Error:

	return error;
}                                      
                                        
 
 
ViStatus CheckStepRange(ViSession vi, ViInt32 step)
{                                   
	ViStatus	error = VI_SUCCESS;
	ViInt32		stepCount=0;
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (!(step>=1 && step<=100 && step<=stepCount))
		return VI_ERROR_STEP_COUNT;
	else
		return error;
Error:
	return error;
} 
 	 
ViStatus C19032P_Check_Parameter_Error(ViSession vi)
{

    ViStatus    error = VI_SUCCESS;
    ViInt32 	errCode=VI_NULL,retCnt; 
	ViChar      errMessage[BUFFER_SIZE],cWrite[BUFFER_SIZE];     
    
    ViSession   io = Ivi_IOSession(vi); 

	viCheckErr( viPrintf(io, "SYST:ERR?\n"));	
	viCheckErr( viScanf (io, "%ld,\"%256[^\"]", &errCode, errMessage));

	if (errCode==0)
		return VI_SUCCESS; 
	else
		viCheckErrElab( IVI_ERROR_INVALID_VALUE, errMessage);   		
Error:

    return error;
}

/*
  ViInt32 *errCode=0;
  ViChar errMessage[512];
	ViStatus	error = VI_SUCCESS;
	ViChar      cBuf[BUFFER_SIZE], sysError[BUFFER_SIZE];
	ViSession	io = Ivi_IOSession(vi); 
	ViInt32		retCnt;
	
	
	checkErr(C19032P_error_query (vi,errCode, errMessage));	
/*	
    checkErr( Ivi_LockSession (vi, VI_NULL));   	
	Fmt       ( cBuf, "%s<SYST:ERR?\n");
	viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
	viCheckErr( viRead  ( io, sysError, BUFFER_SIZE-1, &retCnt)); 
	sysError[retCnt] = 0;
	if (!strncmp(sysError, "+0,", 3))
		return VI_SUCCESS;
	else
		return VI_ERROR_DATA_OUT_OF_RANGE;  
		
/*		
	if (!strncmp(sysError, "-222,", 5)) 
		viCheckErrElab( IVI_ERROR_INVALID_VALUE, "This data of parameter is out of range");
	else
		viCheckErrElab( IVI_ERROR_INVALID_VALUE, sysError);   
	
Error:
    Ivi_UnlockSession(vi, VI_NULL);      
	return error;
	*/


ViStatus _VI_FUNC StepResultString(ViInt32 nCode, ViChar ResultString[])
{
	ViStatus	error = VI_SUCCESS;
	
	memset(ResultString, 0, strlen(ResultString));
	switch (nCode)
	{
		case 112:
			strncpy(ResultString, "STOP", 4);
			break;
		
		case 113:
			strncpy(ResultString, "USER STOP", 9);
			break;	 
			 
		case 114:
			strncpy(ResultString, "CANT NOT TEST", 13);
			break;

		case 115:
			strncpy(ResultString, "TESTING", 7);
			break;	 
		
		case 116:
			strncpy(ResultString, "PASS", 4);
			break;	 
				
		case 53:
			strncpy(ResultString, "CHECK FAIL", 10);
			break;	 
				
		case 88:
			strncpy(ResultString, "POWER HIGH FAIL", 15);
			break;	 
		
		case 89:
			strncpy(ResultString, "POWER LOW FAIL", 14);
		 	break;	 	 
		 	
		case 90:
			strncpy(ResultString, "LAC HIGH FAIL", 13);
		 	break;	 	 

		case 91:
			strncpy(ResultString, "LDC HIGH FAIL", 13);
		 	break;	 	 
		
		case 97:
			strncpy(ResultString, "SHORT FAIL", 10);
		 	break;	 	 

		case 98:
			strncpy(ResultString, "OPEN FAIL", 9);
		 	break;	 	 
	
		case 17:
		case 33:
		case 49:
		case 65:
		case 81:
			strncpy(ResultString, "HIGH FAIL", 9);
			break;
		
		case 18:
		case 34:
		case 50:
		case 66:
		case 82:
			strncpy(ResultString, "LOW FAIL", 8);
			break;

		case 35:
		case 51:
			strncpy(ResultString, "ARC FAIL", 8); 
			break;
		
		case 36:
		case 52:
		case 84:
		case 68://ck 2009/07/07 add to use on 19032P
		case 100:
			strncpy(ResultString, "IO FAIL", 7);
			break;
	
		case 22:
		case 38:
		case 54:
		case 70:
		case 86:
		case 102:
			strncpy(ResultString, "OUTPUT A/D OVER", 15);
			break;
	
		case 23:
		case 39:
		case 55:
		case 71:
		case 87:
		case 103:
			strncpy(ResultString, "METER A/D OVER", 14);
			break;

		default:
			return -1;
	}
	return error;
}




void _VI_FUNC CreateIviIniIfNeeded(void) 
{
	ViStatus    error = VI_SUCCESS;
	FILE *fIviIni = NULL;
	int iTmp=0;
	char sTmp[IVI_MAX_PATHNAME_LEN]={0};

	checkErr( Ivi_GetIviIniDir(sTmp));  
	//if IVI engine is greater than 1.83, user can remark this function to prevent CVI debug run-time error, Bill
	if (SetDir (sTmp)!=0)
		MakeDir(sTmp);
	if(sTmp[strlen(sTmp)-1] == '\\') sTmp[strlen(sTmp)-1]=0;
	strcat(sTmp, "\\ivi.ini");
//	MessagePopup("Before checking", sTmp);
	if(GetFileDate(sTmp, &iTmp, &iTmp, &iTmp) != -1)
		return;
	fIviIni = fopen (sTmp, "a");
//	if(fIviIni==NULL)
//		MessagePopup("File Open Error", sTmp);
	if(fIviIni != NULL)
		fclose(fIviIni);
		
	
Error:
	error = error; 
	
	
}
/*****************************************************************************
 * Function: C19032P_init   
 * Purpose:  VXIplug&play required function.  Calls the   
 *           C19032P_InitWithOptions function.   
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newVi)
{   
    ViStatus    error = VI_SUCCESS;

    if (newVi == VI_NULL)
    {
        Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( IVI_ERROR_INVALID_PARAMETER);
    }
    gbRS232existed = VI_FALSE;
    checkErr( C19032P_InitWithOptions (resourceName, IDQuery, resetDevice, "", newVi));

Error:  
    return error;
}

/*****************************************************************************
 * Function: C19032P_InitWithOptions                                       
 * Purpose:  This function creates a new IVI session and calls the 
 *           IviInit function.                                     
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                                            ViBoolean resetDevice, ViString optionString, 
                                            ViSession *newVi)
{   
    ViStatus    error = VI_SUCCESS;
    ViSession   vi = VI_NULL;
    ViChar      newResourceName[IVI_MAX_MESSAGE_BUF_SIZE];
    ViChar      newOptionString[IVI_MAX_MESSAGE_BUF_SIZE];
    ViBoolean   isLogicalName;
    
    if (newVi == VI_NULL)
    {
        Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( IVI_ERROR_INVALID_PARAMETER);
    }

    *newVi = VI_NULL;
     CreateIviIniIfNeeded();
    checkErr( Ivi_GetInfoFromResourceName (resourceName, optionString, newResourceName, newOptionString, &isLogicalName));
    
        /* create a new interchangeable driver */
    checkErr( Ivi_SpecificDriverNew ("C19032P", newOptionString, &vi));  
    
        /* init the driver */
    checkErr( C19032P_IviInit (newResourceName, IDQuery, resetDevice, vi)); 
    if (isLogicalName)
        checkErr( Ivi_ApplyDefaultSetup (vi));
    *newVi = vi;
    
Error:
    if (error < VI_SUCCESS) 
        Ivi_Dispose (vi);
        
    return error;
}

/*****************************************************************************
 * Function: C19032P_InitializeWithRs232                                       
 * Purpose:  This function creates a new IVI session and calls the 
 *           IviInit function for RS232 interface.                                     
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_InitializeWithRs232 (ViRsrc resourceName,
                                                ViBoolean IDQuery,
                                                ViBoolean resetDevice,
                                                ViInt32 baudRate, ViInt32 parity,
                                                ViInt32 dataBit, ViInt32 stopBit,
                                                ViSession *newVi)
{
	ViStatus	error = VI_SUCCESS;
	
	if (newVi == VI_NULL)
	{
        Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( IVI_ERROR_INVALID_PARAMETER);
	}
    
    gt_nBaudRate = baudRate; 
    gt_nParity = parity;
    gt_nDataBits = dataBit;
    gt_nStopBits = stopBit*10;
	gbRS232existed = VI_TRUE;
	
    checkErr( C19032P_InitWithOptions (resourceName, IDQuery, resetDevice, "", newVi));
    
Error:
	return error;
}

/*****************************************************************************
 * Function: C19032P_IviInit                                                       
 * Purpose:  This function is called by C19032P_InitWithOptions  
 *           or by an IVI class driver.  This function initializes the I/O 
 *           interface, optionally resets the device, optionally performs an
 *           ID query, and sends a default setup to the instrument.                
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_IviInit (ViRsrc resourceName, ViBoolean IDQuery,
                                    ViBoolean reset, ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    ViSession   io = VI_NULL;
    ViUInt16	nInterFace=0, i=0, nCount=0;
    ViChar		sChannelName[BUFFER_SIZE], StepModeType[BUFFER_SIZE], *allMode[BUFFER_SIZE],all_Mode[BUFFER_SIZE];
   	ViBoolean	LLO_State;
   	ViInt32		stepCount;
	ViInt32		memNum, retCnt, iStep;  
	
	for(i=1;i<=100;i++)
	{
		if (i==100)
			nCount += sprintf(sChannelName + nCount, "%d", i);
		else
			nCount += sprintf(sChannelName + nCount, "%d,", i); 
	}
	
	checkErr( Ivi_BuildChannelTable (vi, sChannelName, VI_FALSE, VI_NULL));

//	checkErr( Ivi_BuildChannelTable (vi, CHANNEL_LIST, VI_FALSE, VI_NULL));
    
     /* Add attributes */
    checkErr( C19032P_InitAttributes (vi));			          

    if (!Ivi_Simulating(vi))
    {
        ViSession   rmSession = VI_NULL;

        /* Open instrument session */
        checkErr  ( Ivi_GetAttributeViSession (vi, VI_NULL, IVI_ATTR_VISA_RM_SESSION, 0, &rmSession)); 
        viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &io));
        /* io session owned by driver now */
        checkErr  ( Ivi_SetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, io));  

		/* Configure VISA Formatted I/O */
		viCheckErr( viSetAttribute (io, VI_ATTR_TMO_VALUE, 5000 ));
		viCheckErr( viSetBuf (io, VI_READ_BUF | VI_WRITE_BUF, 4000));
		viCheckErr( viSetAttribute (io, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
		viCheckErr( viSetAttribute (io, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    
		// Support RS232
		if (gbRS232existed == VI_TRUE)
		{
			viCheckErr( viSetAttribute (io, VI_ATTR_ASRL_BAUD, gt_nBaudRate));
			viCheckErr( viSetAttribute (io, VI_ATTR_ASRL_PARITY, gt_nParity));
	//		viCheckErr( viSetAttribute (io, VI_ATTR_ASRL_DATA_BITS, gt_nDataBits));
		//	viCheckErr( viSetAttribute (io, VI_ATTR_ASRL_STOP_BITS, gt_nStopBits));
			viCheckErr( viSetAttribute (io, VI_ATTR_ASRL_FLOW_CNTRL, VI_ASRL_FLOW_NONE));
			viCheckErr( viSetAttribute (io, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR));
			viCheckErr( viSetAttribute (io, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_TERMCHAR));
			viCheckErr( viSetAttribute (io, VI_ATTR_SEND_END_EN, VI_TRUE));
			viCheckErr( viSetAttribute (io, VI_ATTR_TERMCHAR, '\n'));
			viCheckErr( viSetAttribute (io, VI_ATTR_TERMCHAR_EN, VI_TRUE));
		}
	}
    
	/*- Identification Query ------------------------------------------------*/
	if (IDQuery)                               
	{
	    ViChar 		rdBuffer[BUFFER_SIZE], sCompany[BUFFER_SIZE], sType[BUFFER_SIZE], sSerial[BUFFER_SIZE], sVersion[BUFFER_SIZE];
	    ViInt32		nFWVersion;
	    ViInt32		nModel;

	  	#define VALID_RESPONSE_STRING_START_G6000         "QuadTech,G6000"
	    #define VALID_RESPONSE_STRING_START_C19031        "Chroma,19031"
	    #define VALID_RESPONSE_STRING_START_C19032P        "Chroma,19055"
	    #define VALID_RESPONSE_STRING_START_C19032PP		  "Chroma,19032-P"
  


	    checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_ID_QUERY_RESPONSE, 0, BUFFER_SIZE, rdBuffer));
	    
		if (!strncmp (rdBuffer, VALID_RESPONSE_STRING_START_G6000, strlen(VALID_RESPONSE_STRING_START_G6000)))
		{
			checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, C19032P_VAL_MODEL_QUADTECH)); 
		}
		else if ( !strncmp (rdBuffer, VALID_RESPONSE_STRING_START_C19032PP, strlen(VALID_RESPONSE_STRING_START_C19032PP)))
		{
			checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, C19032P_VAL_MODEL_19032P)); 
		}
		else if ( !strncmp (rdBuffer, VALID_RESPONSE_STRING_START_C19031, strlen(VALID_RESPONSE_STRING_START_C19031)))
		{
			checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, C19032P_VAL_MODEL_19031)); 
		}
		else if ( !strncmp (rdBuffer, VALID_RESPONSE_STRING_START_C19032P, strlen(VALID_RESPONSE_STRING_START_C19032P)))
		{
			checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, C19032P_VAL_MODEL_19032)); 
		}
		else
		{             
		    viCheckErr( VI_ERROR_FAIL_ID_QUERY);
		}
		
		if (Scan(rdBuffer, "%s>%s[t44],%s[t44],%s[t44],%s[t44]", sCompany, sType, sSerial, sVersion)!=4)
		{
			viCheckErr( VI_ERROR_FAIL_ID_QUERY);
		}
		
		
		checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));   
		
    	nFWVersion = (ViInt32)(atof(sVersion)*100+0.5);
		checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, nFWVersion));
		if (nFWVersion>=520 || nModel == C19032P_VAL_MODEL_19032P)
			checkErr(C19032P_DefaultScannerSetup(vi));
	}
    
    /*- Reset instrument ----------------------------------------------------*/
    if (reset) 
        checkErr( C19032P_reset (vi));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( C19032P_DefaultInstrSetup (vi));
	
//	checkErr(  Ivi_GetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_LLO_STATE, 
//				IVI_VAL_DIRECT_USER_CALL, &LLO_State)); 

/*    checkErr( C19032P_GetStepCount (vi, &stepCount));		   // delete by angel 2006/03/02
    
	if (stepCount!=0)
	{
    	checkErr( C19032P_GetAllMode (vi, allMode));        
	} */




                                    
                                        
                                        
Error:
    if (error < VI_SUCCESS)
    {
        if (!Ivi_Simulating (vi) && io)
            viClose (io);
    }
    return error;
}

/*****************************************************************************
 * Function: C19032P_close                                                           
 * Purpose:  This function closes the instrument.                            
 *
 *           Note:  This function must unlock the session before calling
 *           Ivi_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_close (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    checkErr( C19032P_IviClose (vi));

Error:    
    Ivi_UnlockSession (vi, VI_NULL);

    Ivi_Dispose (vi);  

    return error;
}

/*****************************************************************************
 * Function: C19032P_IviClose                                                        
 * Purpose:  This function performs all of the drivers clean-up operations   
 *           except for closing the IVI session.  This function is called by 
 *           C19032P_close or by an IVI class driver. 
 *
 *           Note:  This function must close the I/O session and set 
 *           IVI_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_IviClose (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    ViSession io = VI_NULL;

        /* Do not lock here.  The caller manages the lock. */

    checkErr( Ivi_GetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, &io));

Error:
    Ivi_SetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, VI_NULL);
    if (io)                                                      
    {
        viClose (io);
    }
    return error;   
}

/*****************************************************************************
 * Function: C19032P_CheckFloat                                                         
 * Purpose:  This function transfers float to integer.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_CheckFloat (ViSession vi, ViReal64 floatVal, ViInt32 index,
                                     ViInt32 *returnVal)	  // add by angel 2006/04/14
{
	ViStatus	error = VI_SUCCESS;
	
	floatVal *= pow(10, index);	
	*returnVal = RoundRealToNearestInteger (floatVal);    
Error:
	return error;
}

/*****************************************************************************
 * Function: C19032P_reset                                                         
 * Purpose:  This function resets the instrument.                          
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_reset (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32		retCnt;
    ViChar		cBuf[BUFFER_SIZE];

    checkErr( Ivi_LockSession (vi, VI_NULL));

	if (!Ivi_Simulating(vi))                /* call only when locked */
	{
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	    
	    checkErr  ( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    Fmt       ( cBuf, "%s<*RST\n");
	    viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	}
	
	checkErr( C19032P_DefaultInstrSetup (vi));                                

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: C19032P_self_test                                                       
 * Purpose:  This function executes the instrument self-test and returns the 
 *           result.                                                         
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_self_test (ViSession vi, ViInt16 *testResult, 
                                      ViChar testMessage[])
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    if (testResult == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

	viCheckWarn( VI_WARN_NSUP_SELF_TEST);

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: VI_ERRORor_query                                                     
 * Purpose:  This function queries the instrument error queue and returns   
 *           the result.                                                     
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_error_query (ViSession vi, ViInt32 *errCode, 
                                        ViChar errMessage[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (errCode == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

	if (!Ivi_Simulating(vi))                /* call only when locked */
	{
	    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	
	    checkErr  ( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    viCheckErr( viPrintf(io, "SYST:ERR?\n"));	
		viCheckErr( viScanf (io, "%ld,\"%256[^\"]", errCode, errMessage));
	}
	else
	{
	        /* Simulate Error Query */
	    *errCode = 0;
	    strcpy (errMessage, "No error.");
	}

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: VI_ERRORor_message                                                  
 * Purpose:  This function translates the error codes returned by this       
 *           instrument driver into user-readable strings.  
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This 
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_error_message (ViSession vi, ViStatus errorCode,
                                          ViChar errorMessage[256])
{
    ViStatus    error = VI_SUCCESS;
    
    static      IviStringValueTable errorTable = 
    {
		{VI_ERROR_DATA_OUT_OF_RANGE,			 "ERROR: The parameter setting value is out of range."},
		{VI_ERROR_COMMAND_PROTECTED,			 "ERROR: Command Protected."},		
		{VI_ERROR_STEP_COUNT,					 "ERROR: The parameter of step is invalid"},
		{VI_ERROR_INVALID_LIMIT_VALUE,			 "ERROR: The value of limit is invalid."},
		{VI_ERROR_INVALID_MODE_COMMAND,			 "ERROR: The mode of the step is invalid."},
		{VI_ERROR_SCAN_NUM,						 "ERROR: Scanner number setting is not correct."},
		{VI_ERROR_NOT_SUPPORT_LC,				 "ERROR: LC mode is only used in 6000-04, 6000-05, 6000-06, 6000-07 and 6000-08 Scanner."},
		{VI_ERROR_SCANNER_BOX_NULL,				 "ERROR: The scanner box is not existed."},
		{VI_ERROR_NOT_SUPPORT_VERSION,			 "ERROR: Current F/W version doesn't supprot this function."},
		{VI_ERROR_NOT_SUPPORT_SCANNER,			 "ERROR: This function is only used in 6000-05 ,6000-07 and 6000-08 Scanner."},	   
		{VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER,	 "ERROR: This function is only used in 6000-08 Scanner."},
		{VI_ERROR_NOT_MATCH_POWMODE,			 "ERROR: Power mode not match."},
		{VI_ERROR_MEM_INDEX,					 "ERROR: The specified memory name is not correct."},
		{VI_ERROR_NO_STEP_COUNT,				 "ERROR: There is no step data could be query."},  
		{VI_NULL,                                VI_NULL}
    };
        
    if (vi)
        Ivi_LockSession(vi, VI_NULL);

        /* all VISA and IVI error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Ivi_GetSpecificDriverStatusDesc(vi, errorCode, errorMessage, errorTable));

Error:
    if (vi)
        Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: C19032P_revision_query                                                  
 * Purpose:  This function returns the driver and instrument revisions.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_revision_query (ViSession vi, ViChar driverRev[], 
                                           ViChar instrRev[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
        
    if (instrRev == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, driverRev));
    checkErr( Ivi_GetAttributeViString (vi, "", C19032P_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, instrRev));
 
Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}
/*****************************************************************************
 * Function: C19032P_SetStandardC                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetStandardC (ViSession vi)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));     
	checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STANDARD_CAPACITANCE, 0, 1));
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );       
	return error;
}

/*****************************************************************************
 * Function: C19032P_SetFunctionMode                                                  
 * Purpose:  This function switches GENERAL MODE or BREAKDOWN VOLT MODE.  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetFunctionMode (ViSession vi,
                                           ViBoolean functionMode)
{
	ViStatus	error = VI_SUCCESS;          
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
	
	checkErr(C19032P_GetError_IfTrigStart (vi));  	

	viCheckParm( Ivi_SetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_FUNC_MODE, 0, functionMode), 2, "function Mode");       	

	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_FatchFunctionStatus                                                  
 * Purpose:  This function returns the fetch value.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_FetchFunctionStatus (ViSession vi,
                                              ViInt32 functionStatus,
                                              ViReal64 *returnValue)
{
	ViStatus	error = VI_SUCCESS;
	ViString	CurrCmd;
	ViChar		cBuf[BUFFER_SIZE], returnChar[BUFFER_SIZE];
	ViInt32		nFWVersion, retCnt;
	ViInt32		nModel;			
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));  	
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	
	if (nFWVersion<520 && functionStatus>=10 && nModel != C19032P_VAL_MODEL_19032P)
		return VI_ERROR_NOT_SUPPORT_VERSION;
	if (returnValue==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for \"return Value\"" );
		
	if (!Ivi_Simulating(vi))
    {
		ViSession io = Ivi_IOSession(vi);
		
		checkErr  ( Ivi_GetViInt32EntryFromValue( functionStatus, &ChrFetchValueTypeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &CurrCmd, VI_NULL ));
		Fmt       ( cBuf, "%s<SOUR:SAFE:FETC? %s\r", CurrCmd);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, returnChar, BUFFER_SIZE-1, &retCnt));
		returnChar[retCnt] = 0;
		if (functionStatus!=1)  // 1:Fetch Mode
			*returnValue = atof(returnChar);
		else 
		{
			if (!strncmp(returnChar, "GB", 2))   // modify by angel 2006/04/14
				*returnValue = C19032P_VAL_MODE_GB;
			
			if (!strncmp(returnChar, "AC", 2))
				*returnValue = C19032P_VAL_MODE_AC;
			
			if (!strncmp(returnChar, "DC", 2))
				*returnValue = C19032P_VAL_MODE_DC;	
			
			if (!strncmp(returnChar, "IR", 2))
				*returnValue = C19032P_VAL_MODE_IR;	
			
			if (!strncmp(returnChar, "LC", 2))
				*returnValue = C19032P_VAL_MODE_LC;	
			
			if (!strncmp(returnChar, "PA", 2))
				*returnValue = C19032P_VAL_MODE_PA;	
			
			if (!strncmp(returnChar, "OSC", 3))
				*returnValue = C19032P_VAL_MODE_OSC;	
			
			if (!strncmp(returnChar, "NONE", 4))
				*returnValue = 0;		
		}
	}
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_FetchAllStatus                                                  
 * Purpose:  This function returns the all of the fetch value.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_FetchAllStatus (ViSession vi,
                                          ViInt16 *f_Step, ViChar f_Mode[],
                                          ViReal64 *f_Output_Value,
                                          ViReal64 *f_Reading,
                                          ViReal64 *f_Ramp_Laps,
                                          ViReal64 *f_Ramp_Leave,
                                          ViReal64 *f_Test_Lapse,
                                          ViReal64 *f_Test_Leave)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], fs_Mode[20], fs_Output_Value[20], fs_Reading[20],
				fs_Ramp_Laps[20], fs_Ramp_Leave[20], fs_Test_Lapse[20], fs_Test_Leave[20], fs_Step[20];
	ViInt32		retCnt;
	 
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SOUR:SAFE:FETCH? STEP,MODE,OMET,MMET,RELA,RLEA,TELA,TLEA\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		Scan(TempValue, "%s>%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s",
		fs_Step,f_Mode,fs_Output_Value,fs_Reading,fs_Ramp_Laps,fs_Ramp_Leave,fs_Test_Lapse,
		fs_Test_Leave);
		
		*f_Step = atoi(fs_Step);
		*f_Output_Value = atof(fs_Output_Value);
		*f_Reading = atof(fs_Reading);  
		*f_Ramp_Laps = atof(fs_Ramp_Laps);  
		*f_Ramp_Leave = atof(fs_Ramp_Leave);  
		*f_Test_Lapse = atof(fs_Test_Lapse);  
		*f_Test_Leave = atof(fs_Test_Leave);  
	
	 }
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_FetchAllStatus_19055                                                  
 * Purpose:  This function returns the all of the fetch value.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_FetchAllStatus_19055 (ViSession vi,
                                                ViInt16 *f_Step, ViChar f_Mode[],
                                                ViReal64 *f_Output,
                                                ViReal64 *f_Measure,
                                                ViReal64 *f_RealCurrent,
                                                ViReal64 *f_Corona,
                                                ViReal64 *f_HFCC,
                                                ViReal64 *f_RampElapsed,
                                                ViReal64 *f_RampLeave,
                                                ViReal64 *f_DwellElapsed,
                                                ViReal64 *f_DwellLeave,
                                                ViReal64 *f_ElapsedTime,
                                                ViReal64 *f_LeaveTime,
                                                ViReal64 *f_ElapsedFall,
                                                ViReal64 *f_LeaveFall)
{
	ViStatus	error = VI_SUCCESS;
	
	ViChar cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], fs_Step[20],fs_Mode[20],fs_Output[20],fs_Measure[20],fs_RealCurrent[20],fs_Corona[20],
	fs_HFCC[20],fs_RampElapsed[20],fs_RampLeave[20],fs_DwellElapsed[20],fs_DwellLeave[20],fs_ElapsedTime[20],
		fs_LeaveTime[20],fs_ElapsedFall[20],fs_LeaveFall[20]; 
				
			
	ViInt32		retCnt;
	 
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SOUR:SAFE:FETCH? STEP,MODE,OMET,MMET,RMET,CMET,CCMET,RELA,RLEA,DELA,DLEA,TELA,TLEA,FELA,FLEA\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		Scan(TempValue, "%s>%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s",
		fs_Step,f_Mode,fs_Output,fs_Measure,fs_RealCurrent,fs_Corona,fs_HFCC,fs_RampElapsed,fs_RampLeave,fs_DwellElapsed,fs_DwellLeave,fs_ElapsedTime,
		fs_LeaveTime,fs_ElapsedFall,fs_LeaveFall);																 
	
		*f_Step = atoi(fs_Step);
		*f_Output = atof(fs_Output);
		*f_Measure = atof(fs_Measure);  
		*f_RealCurrent = atof(fs_RealCurrent);  
		*f_Corona = atof(fs_Corona);  
		*f_HFCC = atof(fs_HFCC);  
		*f_RampElapsed = atof(fs_RampElapsed); 
		
		*f_RampLeave = atof(fs_RampLeave);
		*f_DwellElapsed = atof(fs_DwellElapsed);  
		*f_DwellLeave = atof(fs_DwellLeave);  
		*f_ElapsedTime = atof(fs_ElapsedTime);  
		*f_LeaveTime = atof(fs_LeaveTime);  
		*f_RampElapsed = atof(fs_RampElapsed); 		
		
		*f_ElapsedFall = atof(fs_ElapsedFall);
		*f_LeaveFall = atof(fs_LeaveFall);  
	
		

	 }
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
 /*****************************************************************************
 * Function: C19032P_FetchAllStatus_19032P                                                  
 * Purpose:  This function sets the system to start test.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_FetchAllStatus_19032P (ViSession vi,
                                                 ViInt16 *f_Step,
                                                 ViChar f_Mode[], ViReal64 *OMET,
                                                 ViReal64 *MMET,
                                                 ViReal64 *LACMET,
                                                 ViReal64 *LDCMET,
                                                 ViReal64 *RELA, ViReal64 *RLEA,
                                                 ViReal64 *DELA, ViReal64 *DLEA,
                                                 ViReal64 *TELA, ViReal64 *TLEA,
                                                 ViReal64 *FELA, ViReal64 *FLEA,
                                                 ViReal64 *CHAN)
{
	ViStatus	error = VI_SUCCESS;
	
	ViChar cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], 
	fs_Step[20],fs_Mode[20],fOMET[20],fMMET[20],fLACMET[20],fLDCMET[20],
	fRELA[20],fRLEA[20],fDELA[20],fDLEA[20],fTELA[20],fTLEA[20],
		fFELA[20],fFLEA[20],fCHAN[20]; 
				
			
	ViInt32		retCnt;
	 
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SOUR:SAFE:FETCH? STEP,MODE,OMET,MMET,LACMET,LDCMET,RELA,RLEA,DELA,DLEA,TELA,TLEA,FELA,FLEA,CHAN\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		Scan(TempValue, "%s>%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s[xt59]%s",
		fs_Step,f_Mode,fOMET,fMMET,fLACMET,fLDCMET,fRELA,fRLEA,fDELA,fDLEA,fTELA,fTLEA,
		fFELA,fFLEA,fCHAN);																 
	
		*f_Step = atoi(fs_Step);
		*OMET = atof(fOMET);
		*MMET = atof(fMMET);  
		*LACMET = atof(fLACMET);  
		*LDCMET = atof(fLDCMET);  
		*RELA = atof(fRELA);  
		*RLEA = atof(fRLEA); 
		
		*DELA = atof(fDELA);
		*DLEA = atof(fDLEA);  
		*TELA = atof(fTELA);  
		*TLEA = atof(fTLEA);  
		*FELA = atof(fFELA);  
		*FLEA = atof(fFLEA); 		
		
		*CHAN = atof(fCHAN);
	
		

	 }
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}





/*****************************************************************************
 * Function: C19032P_SetStartTest                                                  
 * Purpose:  This function sets the system to start test.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetStartTest (ViSession vi,
                                       ViBoolean startTest)
{
	ViStatus	error = VI_SUCCESS, temp_error=VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE]; 
	ViInt32     retCnt;		
	
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
	if (!Ivi_Simulating(vi))
	{
//		ViSession	io = Ivi_IOSession(vi);  	
//		Fmt       ( cBuf, "%s<SOUR:SAFE:%s\n", (startTest)? "STAR" : "STOP");
//		viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));	
	
		viCheckParm( Ivi_SetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_TEST_STATE, 0, startTest), 2, "Start test");  

	}
 	checkErr(C19032P_Check_Parameter_Error(vi)); 	
           	
 	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	
	 	if(error!=0 )
	 	{
		 temp_error=error;
		viCheckParm( Ivi_SetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_TEST_STATE, 0, 0), 2, "Start test");  
		}
		
		
	return temp_error;
}
/*****************************************************************************
 * Function: C19032P_SetOffsetState                                                  
 * Purpose:  This function sets the system to offset test.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetOffsetState (ViSession vi, ViInt32 offsetState)
{
	ViStatus	error = VI_SUCCESS;          
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
	
	checkErr(C19032P_GetError_IfTrigStart (vi));  	
	
	if (offsetState<0 || offsetState>1)
		return VI_ERROR_PARAMETER2;
	viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_OFFSET_STATE, 0, offsetState), 2, "Offset state");  

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}


/*****************************************************************************
 * Function: C19032P_SetCsGet                                                  
 * Purpose:  This function is for starting GET Cs function of short/open detection mode. 
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetStartGetCs (ViSession vi)
{
	ViStatus	error = VI_SUCCESS;          
	ViChar cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		retCnt; 
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));  	
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SAFE:STAR:CST GET\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		TempValue[retCnt] = 0;
		
	}	
 	checkErr(C19032P_Check_Parameter_Error(vi));  		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetAutoReport                                                  
 * Purpose:  This function sets auto report on/off.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetAutoReport (ViSession vi,
                                        ViInt32 autoResult)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
	
	checkErr(C19032P_GetError_IfTrigStart (vi));  	
	viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_AUTO_RESULT_STATE, 0, autoResult), 2, "Auto result ON/OFF");       	
				
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetAutoReportSaved                                                  
 * Purpose:  This function sets auto report on/off.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetAutoReportSaved (ViSession vi,
                                              ViBoolean autoReportSaved)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
	
	checkErr(C19032P_GetError_IfTrigStart (vi));  	
	viCheckParm( Ivi_SetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_AUTO_REPORT_SAVED, 0, autoReportSaved), 2, "Auto rautoReportSaved");       	
				
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetOptionParameter
 * Purpose:  This function gets option information.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetOptionParameter (ViSession vi, ViChar scannerType[], ViInt32 GBFloating,
                                             ViInt32 scannerNumber, ViInt32 GBRate, ViInt32 useSource)
{
	ViStatus	error = VI_SUCCESS;
	checkErr   ( C19032P_LockSession( vi, VI_NULL )); 	
	checkErr( Ivi_SetAttributeViString (vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, scannerType));
	checkErr( Ivi_SetAttributeViInt32  (vi, VI_NULL, C19032P_ATTR_OPTION_GB_FLOATING, 0, GBFloating));
	checkErr( Ivi_SetAttributeViInt32  (vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_NUM, 0, scannerNumber));
	checkErr( Ivi_SetAttributeViInt32  (vi, VI_NULL, C19032P_ATTR_OPTION_GB_RATE, 0, GBRate));
	checkErr( Ivi_SetAttributeViInt32  (vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, useSource));
	checkErr( C19032P_DefaultScannerSetup(vi));

Error:
	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_DefaultScannerSetup
 * Purpose:  This function gets each scanner default value.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_DefaultScannerSetup (ViSession vi)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cInsideType[BUFFER_SIZE], ChanId[4];
	ViInt32		iScanNum, iFloating, i, iUseSource;

	checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, BUFFER_SIZE, cInsideType));
	checkErr( Ivi_GetAttributeViInt32  (vi, VI_NULL, C19032P_ATTR_OPTION_GB_FLOATING, 0, &iFloating));
	checkErr( Ivi_GetAttributeViInt32  (vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_NUM, 0, &iScanNum));
	checkErr( Ivi_GetAttributeViInt32  (vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));
		
	if (iScanNum==0)
		return error;
	
	i=1;
	Fmt(ChanId, "%s<%d", i);
	if (iFloating)
	{
		if (!strncmp( cInsideType, "6000-01", 7) || !strncmp( cInsideType, "6000-02", 7) || !strncmp( cInsideType, "6000-03", 7) || !strncmp( cInsideType, "6000-11", 7))
		{
			checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_GB_FLOATING));
			Fmt     ( ChanId, "%s<%d", ++i);
			if (!strncmp( cInsideType, "6000-01", 7))
				checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_01));
			else if (!strncmp( cInsideType, "6000-02", 7))
				checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_02));
			else if (!strncmp( cInsideType, "6000-03", 7))
				checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_03));
			else if (!strncmp( cInsideType, "6000-11", 7))
				checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_11));
		}
		else
		{
			if (!strncmp( cInsideType, "NONE", 4))
				checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_GB_FLOATING));
			else if (!strncmp( cInsideType, "6000-04", 7) || !strncmp( cInsideType, "6000-05", 7) || !strncmp( cInsideType, "6000-07", 7))
				checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_04_FLOATING));
			else if (!strncmp( cInsideType, "6000-06", 7) || !strncmp( cInsideType, "6000-08", 7))
				checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_06_FLOATING));
		}
	}
	else
	{
		if (!strncmp( cInsideType, "NONE", 4))
			checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_9030A));
		else if (!strncmp( cInsideType, "6000-01", 7))
			checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_01));
		else if (!strncmp( cInsideType, "6000-02", 7))
			checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_02));
		else if (!strncmp( cInsideType, "6000-03", 7))
			checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_03));
		else if (!strncmp( cInsideType, "6000-04", 7) || !strncmp( cInsideType, "6000-05", 7) || !strncmp( cInsideType, "6000-07", 7))
			checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_04));
		else if (!strncmp( cInsideType, "6000-06", 7) || !strncmp( cInsideType, "6000-08", 7))
			checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_06));
		else if (!strncmp( cInsideType, "6000-11", 7))
			checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_6000_11));
	}
	if (i>iScanNum)
		return VI_ERROR_SCAN_NUM;
	i++;
	for (;i<=iScanNum;i++)
	{
		Fmt     ( ChanId, "%s<%d", i);
		checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_9030A));
	}
	for (;i<=8;i++)
	{
		Fmt     ( ChanId, "%s<%d", i);
		checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OPTION_SCAN_TYPE, 0, C19032P_VAL_NULL));
	}

Error:
	return error;
}

/*****************************************************************************
 * Function: C19032P_SetRemoteStartState
 * Purpose:  This function sets if start key is active in the remote mode.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetRemoteStartState (ViSession vi, ViInt32 startState)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		nFWVersion;
	ViInt32		nModel;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  
	checkErr(C19032P_GetError_IfTrigStart (vi));  	
//	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
//	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
//		return VI_ERROR_NOT_SUPPORT_VERSION;
	viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_REMOTE_START_KEY, 0, startState), 2, "The status of the start key in the remote mode");

Error:
	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_DeleteStep                                                  
 * Purpose:  This function delete the all of the step contents.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_DeleteStep (ViSession vi, ViInt32 deleteStepNumber)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		stepCount=0;   
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (deleteStepNumber>=1 && deleteStepNumber<=stepCount)
	{		
		viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_DELETE_STEP, 0, deleteStepNumber), 2, "Delete step number");
	 	checkErr   ( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, stepCount-1));				
	}
	else
		return VI_ERROR_STEP_COUNT;
				
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}			  



/*****************************************************************************
 * Function: C19032P_GetAttribute<type> Functions                                    
 * Purpose:  These functions enable the instrument driver user to get 
 *           attribute values directly.  There are typesafe versions for 
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.                                         
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                ViAttr attributeId, ViInt32 *value)
{                                                                                                           
    return Ivi_GetAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                    value);
}                                                                                                           
ViStatus _VI_FUNC C19032P_GetAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViReal64 *value)
{                                                                                                           
    return Ivi_GetAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}                                                                                                           
ViStatus _VI_FUNC C19032P_GetAttributeViString (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViInt32 bufSize, 
                                                 ViChar value[]) 
{   
    return Ivi_GetAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     bufSize, value);
}   
ViStatus _VI_FUNC C19032P_GetAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViBoolean *value)
{                                                                                                           
    return Ivi_GetAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           
ViStatus _VI_FUNC C19032P_GetAttributeViSession (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViSession *value)
{                                                                                                           
    return Ivi_GetAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           
/*****************************************************************************
 * Function: C19032P_GetFunctionMode                                                  
 * Purpose:  This function gets the offset status of system.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetFunctionMode (ViSession vi,
                                           ViBoolean *functionMode)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (functionMode == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for functionMode");
        
    checkErr( Ivi_GetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_FUNC_MODE, 0, functionMode));
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_SetReportDataItem_19032P                                                  
 * Purpose:  This function gets the offset status of system.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetReportDataItem_19032P (ViSession vi,
                                                    ViChar dataItem[])
{
	ViStatus	error = VI_SUCCESS;
	
	ViChar cBuf[BUFFER_SIZE],TempValue[BUFFER_SIZE];

			
	ViInt32		retCnt;
	 
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
//		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:AREP:ITEM MODE,OMET,MMET,LACMET,LDCMET,RELA,DELA,TELA,FELA,STAT\n");
 		Fmt( cBuf, "%s<SOUR:SAFE:RES:AREP:ITEM %s\n", dataItem);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));

	 }
 	checkErr(C19032P_Check_Parameter_Error(vi)); 	 
	 
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_GetOffsetStatus                                                  
 * Purpose:  This function gets the offset status of system.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetOffsetStatus (ViSession vi, ViInt32 *offsetStatus)
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi);
	ViInt32		retCnt;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  
	
	
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:STAR:OFFS?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		*offsetStatus = atoi(cResult);
		if (atoi(cResult)!=2)
			checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_OFFSET_STATE, IVI_VAL_SET_CACHE_ONLY, 0));
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL ); 
	return error;
}

/*****************************************************************************
 * Function: C19032P_GetTestStatus                                                  
 * Purpose:  This function gets the test status of system.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetTestStatus (ViSession vi,
                                        ViChar testStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		retCnt;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (testStatus==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for \"Test Status\"" );
	
	if (!Ivi_Simulating(vi))
    {
		ViSession io = Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SOUR:SAFE:STAT?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( testStatus, cResult, retCnt);
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetStepCount                                                  
 * Purpose:  This function gets the sum of the step setting into system already.     
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetStepCount (ViSession vi, ViInt32 *stepCount)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		retCnt;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViSession	io = Ivi_IOSession(vi);
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (stepCount==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for \"Step Count\"" );
							
	if (!Ivi_Simulating(vi))
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:SNUM?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		*stepCount = atoi(cResult);
	}
		
   	checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, *stepCount)); 
				
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetAllOmet                                                  
 * Purpose: This function gets the all of the output meter.      
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllOmet (ViSession vi, ViReal64 allOutMeter[])
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, retCnt,stepCount;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (stepCount<=0)
		return VI_ERROR_NO_STEP_COUNT;
 
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (allOutMeter==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for all out meter" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:OMET?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			allOutMeter[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetAllRmet                                                  
 * Purpose: This function gets the all of the output meter.      
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllRmet (ViSession vi,
                                      ViReal64 allRealCurrent[])
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (allRealCurrent==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for all out real current meter" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:RMET?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			allRealCurrent[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetAllCmet                                                  
 * Purpose: This function gets the all of the output meter.      
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllCmet (ViSession vi,
                                      ViReal64 allCORONA[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (allCORONA==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for all CORONA meter" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			allCORONA[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetAllCCmet                                                  
 * Purpose: This function gets the all of the output meter.      
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllCCmet (ViSession vi,
                                       ViReal64 allC_ofHFCC[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (allC_ofHFCC==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for all C of HFCC meter" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:CCMET?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			allC_ofHFCC[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetAllMmet                                                  
 * Purpose:  This function gets the all of the measure meter .     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllMmet (ViSession vi, ViReal64 allMeaureMeter[])
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, retCnt,stepCount;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (stepCount<=0)
		return VI_ERROR_NO_STEP_COUNT;
		
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if(allMeaureMeter==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for all meaure meter" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:MMET?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;

		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			allMeaureMeter[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetAllMmetLadc
 * Purpose:  This function gets the all of the LAC/LDC measure meter.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllMmetLadc (ViSession vi, ViInt32 select,
                                         ViReal64 allMeaureMeter[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, nFWVersion, retCnt,stepCount;
	ViSession	io = Ivi_IOSession(vi);
	ViInt32		nModel;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (stepCount<=0)
		return VI_ERROR_NO_STEP_COUNT;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
		return VI_ERROR_NOT_SUPPORT_VERSION;
	if (allMeaureMeter==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for allMeaureMeter" );

   if (select>1 || select<0)
   		return VI_ERROR_DATA_OUT_OF_RANGE; 				
		
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:MMET:%s?\n", (select==0)? "LAC":"LDC");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;

		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			allMeaureMeter[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetAllMode                                                  
 * Purpose:  This function gets the all mode of the step.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllMode (ViSession vi,
                                      ViChar allMode[])
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], StepModeType[BUFFER_SIZE], ChanId[3],
				seps[]= ",\r\n", *token;
	ViInt32		i=0, iMode, retCnt,stepCount;
	char         *all_Mode[30];
	
	
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (stepCount<=0)
		return VI_ERROR_NO_STEP_COUNT;
		
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (allMode==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for allMode" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:MODE?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
	   	strcpy  ( allMode, TempValue); 
		if (error==0)
		{
			token = strtok(TempValue, seps);
		
			while (token != NULL)		 
			{
				all_Mode[i] = token;
				Fmt     ( ChanId, "%s<%d", i+1);
				checkErr( Ivi_GetViInt32EntryFromString (token, &attrStepModeTypeRangeTable, &iMode, VI_NULL, VI_NULL, VI_NULL, VI_NULL));
				checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, IVI_VAL_SET_CACHE_ONLY, iMode));
				token = strtok(NULL, seps);  
		 		i++;
			}
		}
		else 
		{
			*all_Mode = VI_NULL;
			error = 0;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetAllRampTime                                                  
 * Purpose:  This function gets the all of the ramp time.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllRampTime (ViSession vi, ViReal64 allRampTime[])
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, retCnt,stepCount;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL));
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (stepCount<=0)
		return VI_ERROR_NO_STEP_COUNT;
		
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (allRampTime==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for allRampTime" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:TIME:ELAP:RAMP?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;

		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			allRampTime[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetAllTestTime                                                  
 * Purpose:  This function gets the all of the test time.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllTestTime (ViSession vi, ViReal64 allTestTime[])
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, retCnt,stepCount;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (stepCount<=0)
		return VI_ERROR_NO_STEP_COUNT;	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (allTestTime==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for allTestTime" );	
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:TIME:ELAP:TEST?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;

		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			allTestTime[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetAllTimeDwell                                                  
 * Purpose:  This function queries dwell time of testing all steps.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllTimeDwell (ViSession vi,
                                           ViReal64 getAllTimeDwell[])
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, retCnt,stepCount;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (stepCount<=0)
		return VI_ERROR_NO_STEP_COUNT;		
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (getAllTimeDwell==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getAllTimeDwell" );	
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:TIME:ELAP:DWEL?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;

		token = strtok( TempValue, seps );
		while (token != NULL)		 
		{
			getAllTimeDwell[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetAllFallTime
 * Purpose:  This function gets the all of the fall time.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllFallTime (ViSession vi, ViReal64 allFallTime[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE], seps[]= ",\r\n", *token;
	ViInt32		i=0, nFWVersion, retCnt,step=0;
	ViSession	io = Ivi_IOSession(vi);
	ViInt32		nModel;
	
	checkErr( C19032P_LockSession( vi, VI_NULL));
	checkErr( CheckStepRange(vi,step)); 	
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
		return VI_ERROR_NOT_SUPPORT_VERSION;
	if (allFallTime==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for allFallTime" );	
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:TIME:ELAP:FALL?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;

		token = strtok( TempValue, seps );
		while (token != NULL)
		{
			allFallTime[i] = atof(token); 
			token = strtok( NULL, seps );  
	 		i++;
		}
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetAllResult                                                  
 * Purpose:  This function gets the all code of the result .     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAllResult (ViSession vi, ViInt32 allResultCode[],
                                       ViChar codeMean[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE]="0", codeMean_temp[BUFFER_SIZE]="0",
				seps[]= ",\r\n", *token;
	ViInt32		i=0, nCode, length=0, nCount=0, retCnt,stepCount;
	ViSession	io = Ivi_IOSession(vi);

	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &stepCount));
				
	if (stepCount<=0)
		return VI_ERROR_NO_STEP_COUNT;	
		
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	
	if (allResultCode==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for allResultCode" );
	if (codeMean==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for codeMean" );			
									 
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ALL:JUDG?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;

		token = strtok( TempValue, seps );
		while (token != NULL)
		{
			allResultCode[i] = atoi(token);
			nCode = atoi(token);
			 
			viCheckErr( StepResultString(nCode, codeMean_temp));   //return the string of code. 

			nCount += sprintf(codeMean + nCount, "%s,", codeMean_temp);
			token = strtok( NULL, seps );  
	 		i++;			 
		}				  
		length = strlen(codeMean);
		codeMean[length-1] = '\0';
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetRunCompleted                                                  
 * Purpose:  This function gets the running completed status .     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetRunCompleted (ViSession vi,
                                          ViInt32 *runningCompleted)
{
	ViStatus	error = VI_SUCCESS;
	ViBoolean	bOnOff;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
//	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (runningCompleted == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for runningCompleted");

	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_RUN_COMPLETE_STATUS, 0, runningCompleted));
//	bOnOff =( *runningCompleted==1)? VI_OFF : VI_ON;
//	checkErr( Ivi_SetAttributeViBoolean(vi, VI_NULL, C19032P_ATTR_TEST_STATE, 0, bOnOff));
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetFinalStepResult                                                  
 * Purpose:  This function gets the final step result.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetFinalStepResult (ViSession vi, ViInt32 *lastCode,
                                             ViChar codeMean[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	
	if (lastCode == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for lastCode");
	if (codeMean == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for codeMean");
        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:LAST:JUDG?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*lastCode = atoi(TempValue);
		viCheckErr( StepResultString(*lastCode, codeMean));   //return the string of code. 
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetAutoReport                                                  
 * Purpose:  This function gets auto report on/off.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAutoReport (ViSession vi, ViInt32 *getAutoReport)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (getAutoReport == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getAutoReport");
        
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_AUTO_RESULT_STATE, 0, getAutoReport));
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_FetchAutoReportData_19032P                                                  
 * Purpose:  This function gets test data for auto report,     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_FetchAutoReportData_19032P (ViSession vi,
                                                      ViChar mode[],
                                                      ViReal64 *OMET,
                                                      ViReal64 *MMET,
                                                      ViReal64 *LACMET,
                                                      ViReal64 *LDCMET,
                                                      ViReal64 *RELA,
                                                      ViReal64 *DELA,
                                                      ViReal64 *TELA,
                                                      ViReal64 *FELA,
                                                      ViReal64 *STATE)
{
	ViStatus	error = VI_SUCCESS;
	
	ViChar cBuf[BUFFER_SIZE],TempValue[BUFFER_SIZE],fOMET[BUFFER_SIZE],fMMET[BUFFER_SIZE],fLACMET[30],fLDCMET[30],fCCMET[30],fRELA[30],
	fDELA[30],fTELA[30],fFELA[30],fSTATE[30];
				
			
	ViInt32		retCnt;
	 
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:AREP:ITEM MODE,OMET,MMET,RMET,CMET,CCMET,RELA,DELA,TELA,FELA,STAT\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		Scan(TempValue, "%s>%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s",
		mode,fOMET,fMMET,fLACMET,fLDCMET,fCCMET,fRELA,fDELA,fTELA,fFELA,fSTATE);																 
	
		*OMET = atoi(fOMET);
		*MMET = atof(fMMET);
		*LACMET = atof(fLACMET);  
		*LDCMET = atof(fLDCMET);  
		*RELA = atof(fRELA);  
		*DELA = atof(fDELA); 
		*TELA = atof(fTELA);
		*FELA = atof(fFELA);  
		*STATE = atof(fSTATE);  


	 }
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_FetchAutoReportData                                                  
 * Purpose:  This function gets test data for auto report,     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_FetchAutoReportData (ViSession vi,
                                               ViChar mode[], ViReal64 *OMET,
                                               ViReal64 *MMET, ViReal64 *RMET,
                                               ViReal64 *CMET, ViReal64 *CCMET,
                                               ViReal64 *RELA, ViReal64 *DELA,
                                               ViReal64 *TELA, ViReal64 *FELA,
                                               ViReal64 *STATE)
{
	ViStatus	error = VI_SUCCESS;
	
	ViChar cBuf[BUFFER_SIZE],TempValue[BUFFER_SIZE],fOMET[BUFFER_SIZE],fMMET[BUFFER_SIZE],fRMET[30],fCMET[30],fCCMET[30],fRELA[30],
	fDELA[30],fTELA[30],fFELA[30],fSTATE[30];
				
			
	ViInt32		retCnt;
	 
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SOUR:SAFE:RES:AREP:ITEM MODE,OMET,MMET,RMET,CMET,CCMET,RELA,DELA,TELA,FELA,STAT\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		
		Scan(TempValue, "%s>%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s[xt44]%s",
		mode,fOMET,fMMET,fRMET,fCMET,fCCMET,fRELA,fDELA,fTELA,fFELA,fSTATE);																 
	
		*OMET = atoi(fOMET);
		*MMET = atof(fMMET);
		*RMET = atof(fRMET);  
		*CMET = atof(fCMET);  
		*CCMET = atof(fCCMET);  
		*RELA = atof(fRELA);  
		*DELA = atof(fDELA); 
		*TELA = atof(fTELA);
		*FELA = atof(fFELA);  
		*STATE = atof(fSTATE);  


	 }
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetAutoReportSaved                                                  
 * Purpose:  This command is for setting if the function of auto report is saved until power on 
			  next time. (For RS232 interface only)
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAutoReportSaved (ViSession vi,
                                              ViBoolean *autoReportSaved)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	if (autoReportSaved == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for autoReportSaved");
        
    checkErr( Ivi_GetAttributeViBoolean (vi, VI_NULL, C19032P_ATTR_AUTO_REPORT_SAVED, 0, autoReportSaved));
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
 /*******************************************************************************
 * Function: C19032P_GetLCPowerSourceVoltFreq                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCPowerSourceVoltFreq (ViSession vi,
                                                    ViInt32 step,
                                                    ViReal64 *targetVoltage,
                                                    ViReal64 *targetFrequency)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (targetVoltage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for targetVoltage");
    if (targetFrequency == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for targetFrequency");
        
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_TARGET_VOLT, 0, targetVoltage));											
 		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_TARGET_FREQ, 0, targetFrequency));
	}		
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCChanHighState                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCChanHighState (ViSession vi,
                                              ViInt32 step,
                                              ViChar LCChannelHighStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt;
	ViSession	io = Ivi_IOSession (vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	
    if (LCChannelHighStatus == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCChannelHighStatus");
        								
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:LC:CHAN:HIGH?\n", step);
		viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( LCChannelHighStatus, cResult, retCnt);
	}
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCCurrentOffset                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCCurrentOffset (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 *LCCurrentOffset)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCCurrentOffset == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCCurrentOffset");

	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_CURRENT_OFFSET, 0, LCCurrentOffset));											
 
	}		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCCurrentOffsetLAC                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCCurrentOffsetLAC (ViSession vi,
                                                 ViInt32 step,
                                                 ViReal64 *LCCurrentOffsetLAC)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3],cScanner[20];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCCurrentOffsetLAC == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCCurrentOffsetLAC");
	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));

	if (strncmp (cScanner, "6000-08", 7)!=0)  //for 6000-08 only 
		return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_CURRENT_OFFSET_LAC, 0, LCCurrentOffsetLAC));											
 
	}		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCCurrentOffsetLDC                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCCurrentOffsetLDC (ViSession vi,
                                                 ViInt32 step,
                                                 ViReal64 *LCCurrentOffsetLDC)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3],cScanner[20];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCCurrentOffsetLDC == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCCurrentOffsetLDC");
        
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));

	if (strncmp (cScanner, "6000-08", 7)!=0)  //for 6000-08 only 
		return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;	       

	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_CURRENT_OFFSET_LDC, 0, LCCurrentOffsetLDC));											
 
	}		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetRemoteStartState                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetRemoteStartState (ViSession vi,
                                               ViInt32 *startState)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (startState==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for startState" );	
				
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_REMOTE_START_KEY, 0, startState));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetBreakResult                                                  
 * Purpose:  This function queries the judgment result code under BREAK DOWN VOLT 
              MODE. 
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakResult (ViSession vi,
                                          ViInt32 *getBreakResult)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakResult == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakResult");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakResult = atoi(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetSettingOfStep                                                  
 * Purpose:  This function gets the setting contents of the step.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetSettingOfStep (ViSession vi,
                                           ViInt32 stepNumber,
                                           ViChar gettingInfo[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[4];
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    if (gettingInfo == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for gettingInfo");
        
	checkErr( CheckStepRange  (vi, stepNumber));
	Fmt     ( ChanId, "%s<%d", stepNumber); 
	checkErr(ModeCheck(vi,stepNumber,C19032P_VAL_MODE_GB));  
	checkErr( Ivi_GetAttributeViString( vi, ChanId, C19032P_ATTR_STEP_SETTING, 0, AUTO_BUFFER_SIZE, gettingInfo));  

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*******************************************************************************
 * Function: C19032P_GetModeOfStep                                                  
 * Purpose:  This function gets the mode of the step .     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetModeOfStep (ViSession vi, ViInt32 step,
                                        ViChar mode_ofStep[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[BUFFER_SIZE];
    ViInt32		iMode;
    ViString	cMode;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (mode_ofStep == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for mode of Step");
        
	checkErr( CheckStepRange(vi,step));        
    Fmt     ( ChanId, "%s<%d", step);  
    
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_GB));      
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
	checkErr( Ivi_GetViInt32EntryFromValue (iMode, &attrStepModeTypeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cMode, VI_NULL));
	strcpy  ( mode_ofStep, cMode);

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakMMET                                                  
 * Purpose:  This function queries MEASURE METER reading under BREAK DOWN VOLT 
			MODE.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakMMET (ViSession vi,
                                        ViReal64 *getBreakMMET)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakMMET == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakMMET");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:MMET?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakMMET = atof(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakOMET                                                  
 * Purpose:  This function queries OUTPUT METER reading under BREAK DOWN VOLT 
             MODE.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakOMET (ViSession vi,
                                        ViReal64 *getBreakOMET)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakOMET == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakOMET");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:OMET?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakOMET = atof(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakCMET                                                  
 * Purpose:  This function queries CORONA  METER reading under BREAK DOWN VOLT 
             MODE.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakCMET (ViSession vi,
                                        ViReal64 *getBreakCMET)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakCMET == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakCMET");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:CMET?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakCMET = atof(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakMode                                                  
 * Purpose:  This function queries the selected MODE under BREAK DOWN VOLT
			MODE,and returns character data AC or DC.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakMode (ViSession vi,
                                        ViChar getBreakMode[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakMode == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakMode");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:MODE?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, getBreakMode, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;

	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakStep                                                  
 * Purpose:  This function queries the executed STEP number under BREAK DOWN
			 VOLT MODE.	
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakStep (ViSession vi,
                                        ViInt32 *getBreakStep)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakStep == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakStep");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:STEP?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakStep = atoi(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakTime                                                  
 * Purpose:  This function queries the executed STEP number under BREAK DOWN
			 VOLT MODE.	
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakTime (ViSession vi,
                                        ViReal64 *getBreakTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakTime");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:TIME?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakTime = atof(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakTimeDwell                                                  
 * Purpose: This function queries the executed dwell time of the STEP under
			BREAK DOWN VOLT MODE.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakTimeDwell (ViSession vi,
                                             ViReal64 *getBreakTimeDwell)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakTimeDwell == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakTimeDwell");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:TIME:DWEL?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakTimeDwell = atof(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakTimeRamp                                                  
 * Purpose: This fucntion queries the executed dwell time of the STEP under
			BREAK DOWN VOLT MODE.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakTimeRamp (ViSession vi,
                                            ViReal64 *getBreakTimeRamp)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakTimeRamp == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakTimeRamp");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:TIME:RAMP?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakTimeRamp = atof(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetBreakWV                                                  
 * Purpose: This fucntion queries the executed dwell time of the STEP under
			BREAK DOWN VOLT MODE.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetBreakWV (ViSession vi,
                                      ViReal64 *getBreakWV)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	if (getBreakWV == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getBreakWV");

        
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SAFE:RES:BRE:WV?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getBreakWV = atof(TempValue);
	
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_GetSysErr                                                  
 * Purpose:  This function gets system errors.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetSysErr (ViSession vi,
                                    ViChar sysError[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], rdBuffer[BUFFER_SIZE];
	ViInt32		retCnt;

	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (sysError==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for \"System Error\"" );
	 
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SYST:ERR?\r");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, rdBuffer, BUFFER_SIZE-1, &retCnt));
		rdBuffer[retCnt] = 0;
		strncpy   ( sysError, rdBuffer, retCnt);
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetGBCurrentRatio                                                  
 * Purpose:  It queries ratio value of GB current.  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetGBCurrentRatio (ViSession vi,
                                             ViChar GBCurrentRatio[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], rdBuffer[BUFFER_SIZE];
	ViInt32		retCnt;

	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (GBCurrentRatio==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for \"GBCurrentRatio\"" );
	 
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<SYSTem:OPTion:GBCurrent:RATE?\r");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, rdBuffer, BUFFER_SIZE-1, &retCnt));
		rdBuffer[retCnt] = 0;
		strncpy   ( GBCurrentRatio, rdBuffer, retCnt);
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetGBFloatBoard                                                  
 * Purpose:  This function gets the system version for SCPI .      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetGBFloatBoard (ViSession vi,
                                           ViInt32 *GBCurrentRatio)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (GBCurrentRatio == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for GBCurrentRatio");
        
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_OPTION_GB_FLOATING, 0, GBCurrentRatio));
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetSCANType                                                  
 * Purpose:  This function gets the system version for SCPI .      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetSCANType (ViSession vi,
                                       ViChar SCANTYPE[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (SCANTYPE == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for SCANTYPE");	
        
	checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD,0, AUTO_BUFFER_SIZE, SCANTYPE));


Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetOptionSource                                                  
 * Purpose:  This function gets the system version for SCPI .      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetOptionSource (ViSession vi,
                                           ViInt32 *GBCurrentRatio)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (GBCurrentRatio == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for GBCurrentRatio");
        
   	checkErr( Ivi_GetAttributeViInt32  (vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, GBCurrentRatio));
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_GetSysVersion                                                  
 * Purpose:  This function gets the system version for SCPI .      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetSysVersion (ViSession vi,
                                        ViChar systemVersion[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		rdBuffer[BUFFER_SIZE];
	ViUInt32 	retCount;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (systemVersion==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for \"System Version\"" );
		
	checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_SYS_VERSION, 0, AUTO_BUFFER_SIZE, systemVersion));
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_QueryIDSting                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_CLS (ViSession vi)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	

	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<*CLS\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_SetsESE                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetsESE (ViSession vi, ViInt32 setsESE)
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
   if (setsESE>255 || setsESE<0)
   		return VI_ERROR_DATA_OUT_OF_RANGE; 
   		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<*ESE %d\n",setsESE);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_QueryIDSting                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetsOPC (ViSession vi)
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<*OPC\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_SetPSC                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetPSC (ViSession vi, ViInt32 PSC)
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
   if (PSC>1 || PSC<0)
   		return VI_ERROR_DATA_OUT_OF_RANGE; 
   		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<*PSC %d\n",PSC);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_QueryIDSting                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_QueryIDSting (ViSession vi,
                                        ViChar queryIDString[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (queryIDString==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for queryIDString");

		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<*IDN?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, queryIDString, BUFFER_SIZE-1, &retCnt));
		queryIDString[retCnt] = 0;

	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_RCL                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_RCL (ViSession vi, ViInt32 regNumber)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
   if (regNumber>100 || regNumber<0)
   		return VI_ERROR_DATA_OUT_OF_RANGE; 	

	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<:*RCL %d\n",regNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
	}	
 	checkErr(C19032P_Check_Parameter_Error(vi));
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_RCL                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SAV (ViSession vi, ViInt32 SAVNumber)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
   if (SAVNumber>100 || SAVNumber<0)
   		return VI_ERROR_DATA_OUT_OF_RANGE; 		

	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<:*SAV %d\n",SAVNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
	}	
 	checkErr(C19032P_Check_Parameter_Error(vi));
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetsESR                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetsESR (ViSession vi, ViInt32 *ESR)
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (ESR==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for ESR" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<*ESR?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*ESR = atoi(TempValue);   		

	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
 /*****************************************************************************
 * Function: C19032P_SetSRE                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetSRE (ViSession vi, ViInt32 SRE)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
   if (SRE>255 || SRE<0)
   		return VI_ERROR_DATA_OUT_OF_RANGE; 		

	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<:*SRE %d\n",SRE);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_GetsESR                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetsESE (ViSession vi, ViInt32 *getsESE)
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (getsESE==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getsESE" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<*ESE?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getsESE = atoi(TempValue);   		

	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetsOPC                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetsOPC (ViSession vi, ViChar OPC[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (OPC==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for OPC");

		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<*OPC?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, OPC, BUFFER_SIZE-1, &retCnt));
		cBuf[retCnt] = 0;

	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetsPSC                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetsPSC (ViSession vi, ViChar PSC[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (PSC==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for PSC");

		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<*PSC?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, PSC, BUFFER_SIZE-1, &retCnt));
		PSC[retCnt] = 0;

	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_GetsSRE                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetsSRE (ViSession vi, ViInt32 *getsSRE)
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (getsSRE==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getsSRE" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<*SRE?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getsSRE = atoi(TempValue);   		

	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetsSTB                                                  
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetsSTB (ViSession vi, ViInt32 *getsSTB)
{
 	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], TempValue[BUFFER_SIZE];
	ViInt32		i=0, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (getsSTB==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getsSTB" );
		
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<*STB?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, TempValue, BUFFER_SIZE-1, &retCnt));
		TempValue[retCnt] = 0;
		*getsSTB = atoi(TempValue);   		

	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_MemNameDel                                                  
 * Purpose:  This function deletes the parameter data of the indicated in the main memory.      
 *****************************************************************************/

ViStatus _VI_FUNC C19032P_MemNameDel (ViSession vi,
                                      ViChar memoryName[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<:MEMory:DELete %s\n",memoryName);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
	}	
 	checkErr(C19032P_Check_Parameter_Error(vi));
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_MemDel                                                  
 * Purpose:  This function delete the specify the memory setting value.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_MemDel (ViSession vi,
                                  ViInt32 deleteNumber)
{
	ViStatus	error = VI_SUCCESS;  
							    
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (!Ivi_Simulating(vi))
    {
		viCheckParm( Ivi_SetAttributeViInt32( vi, VI_NULL, C19032P_ATTR_MEM_DEL, 0, deleteNumber), 2, "Memory number deleted" ); 		
	}
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_MemDefine                                                  
 * Purpose:  This function defines the specify the memory setting value.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_MemDefine (ViSession vi, ViInt32 memNum, ViChar memName[])
{
	ViStatus	error = VI_SUCCESS;  
	ViChar		ChanId[3], ConbinMemName[BUFFER_SIZE];
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
	Fmt        ( ChanId, "%s<%d", memNum);          
	viCheckParm( Ivi_SetAttributeViString( vi, ChanId, C19032P_ATTR_MEM_NAME_DEFINE, 0,memName), 3, "Memory name");
 	checkErr(C19032P_Check_Parameter_Error(vi));  
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_MemDefineIndex                                                  
 * Purpose:  This function defines the specify the memory setting value.      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_MemDefineIndex (ViSession vi, ViChar memName[], ViInt32 *memNum)
{
	ViStatus	error = VI_SUCCESS;  

	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_QUERY_MEM_INDEX, IVI_VAL_SET_CACHE_ONLY, memName), 2, "Memory Name");
	checkErr   ( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_QUERY_MEM_INDEX_NUM, 0, memNum));
	if (*memNum==0)
		return VI_ERROR_MEM_INDEX;

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetMemFreeState                                                  
 * Purpose:  This function queries the rest PRESET parameter number in the main memory.     
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetMemFreeState (ViSession vi,
                                           ViInt32 *remain, ViInt32 *used)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (remain==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for remain");
	if (used==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for used");	
		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<MEM:FREE:STAT?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, returnStr1, BUFFER_SIZE-1, &retCnt));
		returnStr1[retCnt] = 0;
		Scan      ( returnStr1, "%s>%s[xt44]%s", returnStr1, returnStr2);
		
		*remain = atoi(returnStr1);
		*used = atoi(returnStr2);
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_GetMemFreeStep                                                  
 * Purpose:  This function gets the free steps of memory .      
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetMemFreeStep (ViSession vi,
                                         ViInt32 *remnants, ViInt32 *used)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (remnants==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for remants");
	if (used==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for used");	
		
	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
		Fmt       ( cBuf, "%s<MEM:FREE:STEP?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, returnStr1, BUFFER_SIZE-1, &retCnt));
		returnStr1[retCnt] = 0;
		Scan      ( returnStr1, "%s>%s[xt44]%s", returnStr1, returnStr2);
		
		*remnants = atoi(returnStr1);
		*used = atoi(returnStr2);
	}	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SaveMemory                                                  
 * Purpose:  This function saves current setting to the specified memory location.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SaveMemory (ViSession vi, ViInt32 memNum,
                                     ViChar memName[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi);
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;

	checkErr  ( C19032P_LockSession( vi, VI_NULL ));
	if (memNum<1 || memNum>100)
		return VI_ERROR_PARAMETER2;
	Fmt       ( cBuf, "%s<*SAV %d\n", memNum);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	checkErr  ( C19032P_MemDefine (vi, memNum, memName));
	
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_RecallMemory                                                  
 * Purpose:  This function recall the specified memory.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_RecallMemory (ViSession vi, ViChar memName[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi);
	ViChar		cBuf[BUFFER_SIZE], all_Mode[100];
	ViInt32		memNum, retCnt, iStep;

	checkErr  ( C19032P_LockSession( vi, VI_NULL ));
	checkErr  ( C19032P_MemDefineIndex (vi, memName, &memNum));
	Fmt       ( cBuf, "%s<*RCL %d\n", memNum);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	checkErr  ( C19032P_GetStepCount (vi, &iStep));
	checkErr  ( C19032P_GetAllMode (vi, all_Mode));
	
 	checkErr(C19032P_Check_Parameter_Error(vi));  	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetMemoryMaxCapacity                                                  
 * Purpose:  This command queries the maximum value plus 1 of the analyzer *SAV / *RCL.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetMemoryMaxCapacity (ViSession vi,
                                                ViInt32 *getMemoryMaxCapacity)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		returnStr1[BUFFER_SIZE], returnStr2[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
				
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (getMemoryMaxCapacity==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getMemoryMaxCapacity");

	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_QUERY_MEM_MAX_CAPACITY, 0, getMemoryMaxCapacity));  
	

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetParameter                                                  
 * Purpose:  This function preset all parameters.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetParameter (ViSession vi, ViReal64 passHold, ViReal64 stepHold,
                                             ViInt32 ACFreq, ViReal64 GBFreq, ViInt32 IEC,
                                             ViReal64 GBVolt, ViInt32 autoRange, ViInt32 softAGC,
                                             ViReal64 startWait)
{
	ViStatus	error = VI_SUCCESS;

	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_PASS_HOLD_TIME, 0, passHold), 2, "Preset pass hold time");   
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_STEP_HOLD_TIME, 0, stepHold), 3, "Preset step hold time"); 
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_AC_FREQUENCY, 0, ACFreq), 4, "Preset AC frequency");   
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_GB_FREQUENCY, 0, GBFreq), 5, "Preset GB frequency");
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_IEC_STATE, 0, IEC), 6, "Preset IEC-601 ON/OFF");
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_GB_VOLTAGE, 0, GBVolt), 7, "Preset GB voltage");  
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_AUTO_RANGE, 0, autoRange), 8, "Preset auto range ON/OFF");  
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_SOFT_AGC_STATE, 0, softAGC), 9, "Preset soft AGC ON/OFF");  
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_START_WAIT, 0, startWait), 10, "Preset start wait time");
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetProductNumber
 * Purpose:  This function preset part number, lot number and serial number.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetProductNumber (ViSession vi, ViChar partNo[],
                                                 ViChar lotNo[], ViChar serialNo[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  	
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_PRESET_PART_NUMBER, 0, partNo), 2, "Preset part number"); 
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_PRESET_LOT_NUMBER, 0, lotNo), 3, "Preset lot number");   
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_PRESET_SERIAL_NUMBER, 0, serialNo), 4, "Preset serial number");

Error:
	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetPassHold                                                  
 * Purpose:  This function preset pass hold time.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetPassHold (ViSession vi, ViReal64 passHold)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		CoercedVal, index=2;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	
//	checkErr   ( C19032P_CheckFloat( vi, passHold, index, &CoercedVal));
//	passHold = ( ViReal64) CoercedVal/pow(10, index);
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_PASS_HOLD_TIME, 0, passHold), 2, "Preset pass hold time");  
	
Error:
 	C19032P_UnlockSession( vi, VI_NULL );   
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetStepHold                                                  
 * Purpose:  This function preset step hold time.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetStepHold (ViSession vi, ViReal64 stepHold)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		CoercedVal, index=2;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
//	checkErr   ( C19032P_CheckFloat( vi, stepHold, index, &CoercedVal));
//	stepHold = ( ViReal64) CoercedVal/pow(10, index);
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_STEP_HOLD_TIME, 0, stepHold), 2, "Preset step hold time"); 
Error:
 	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetACFreq                                                  
 * Purpose:  This function preset ac frequency.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetACFreq (ViSession vi, ViInt32 ACFreq)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_AC_FREQUENCY, 0, ACFreq), 2, "Preset AC frequency");   
Error:
 	C19032P_UnlockSession( vi, VI_NULL );   
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetGBFreq                                                  
 * Purpose:  This function preset gb frequency.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetGBFreq (ViSession vi, ViReal64 GBFreq)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		CoercedVal, index=2;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
//	checkErr   ( C19032P_CheckFloat( vi, GBFreq, index, &CoercedVal));
//	GBFreq   = ( ViReal64) CoercedVal/pow(10, index);
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_GB_FREQUENCY, 0, GBFreq), 2, "Preset GB frequency");
Error:
 	C19032P_UnlockSession( vi, VI_NULL );   
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetIEC                                                  
 * Purpose:  This function preset IEC on/off.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetIEC (ViSession vi, ViInt32 IEC)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_IEC_STATE, 0, IEC), 2, "Preset IEC-601 ON/OFF");
Error:
 	C19032P_UnlockSession( vi, VI_NULL );   
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetGBVolt                                                  
 * Purpose:  This function preset gb voltage.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetGBVolt (ViSession vi, ViReal64 GBVolt)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		CoercedVal, index=2;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
//	checkErr   ( C19032P_CheckFloat( vi, GBVolt, index, &CoercedVal));
//	GBVolt   = ( ViReal64) CoercedVal/pow(10, index);
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_GB_VOLTAGE, 0, GBVolt), 2, "Preset GB voltage");  
Error:
 	C19032P_UnlockSession( vi, VI_NULL ); 
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetAutoRange                                                  
 * Purpose:  This function preset auto range on/off.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetAutoRange (ViSession vi, ViInt32 autoRange)
{
	ViStatus	error = VI_SUCCESS;
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_AUTO_RANGE, 0, autoRange), 2, "Preset auto range ON/OFF");  
Error:
 	C19032P_UnlockSession( vi, VI_NULL ); 
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetAGC
 * Purpose:  This function preset soft AGC on/off.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetAGC (ViSession vi, ViInt32 softAGC)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_SOFT_AGC_STATE, 0, softAGC), 2, "Preset soft AGC ON/OFF");  
Error:
  	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetStartWait                                                  
 * Purpose:  This function preset start wait time.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetStartWait (ViSession vi, ViReal64 startWait)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		CoercedVal, index=2;
	
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
//	checkErr   ( C19032P_CheckFloat( vi, startWait, index, &CoercedVal));
//	startWait =( ViReal64) CoercedVal/pow(10, index);
	viCheckParm( Ivi_SetAttributeViReal64( vi, VI_NULL, C19032P_ATTR_PRESET_START_WAIT, 0, startWait), 2, "Preset start wait time");
Error:
  	C19032P_UnlockSession( vi, VI_NULL );   
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetPartNumber                                                  
 * Purpose:  This function preset part number.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetPartNumber (ViSession vi, ViChar partNo[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_PRESET_PART_NUMBER, 0, partNo), 2, "Preset part number"); 
Error:
  	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetLotNumber
 * Purpose:  This function preset lot number.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetLotNumber (ViSession vi, ViChar lotNo[])
{
	ViStatus	error = VI_SUCCESS;
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_PRESET_LOT_NUMBER, 0, lotNo), 2, "Preset lot number");   
Error:
  	C19032P_UnlockSession( vi, VI_NULL );   
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetSerialNumber                                                  
 * Purpose:  This function preset serial number.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetSerialNumber (ViSession vi, ViChar serialNo[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_PRESET_SERIAL_NUMBER, 0, serialNo), 2, "Preset serial number");
Error:
  	C19032P_UnlockSession( vi, VI_NULL );     
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPresetRjudgement
 * Purpose:  This function preset ramp ON/OFF.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetRjudgement (ViSession vi, ViInt32 rampJudg)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		nFWVersion;
	ViInt32		nModel;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
//	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
//	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
//		return VI_ERROR_NOT_SUPPORT_VERSION;						 //2011/12/1 modify by rock
	viCheckParm( Ivi_SetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_PRESET_RAMP_JUDGEMENT, 0, rampJudg), 2, "Preset ramp ON/OFF");  
Error:
  	C19032P_UnlockSession( vi, VI_NULL );     
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetPresetGFI
 * Purpose:  This function preset ramp ON/OFF.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetGFI (ViSession vi,
                                        ViInt32 setPresetGFI)
{
	ViStatus	error = VI_SUCCESS; 
	ViString   Cmd_Str;
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
 	checkErr(C19032P_GetError_IfTrigStart (vi));   	
 	checkErr( Ivi_GetViInt32EntryFromValue(setPresetGFI, &ChrPresetGFIRangeTable, VI_NULL, 
												VI_NULL, VI_NULL, VI_NULL, &Cmd_Str, VI_NULL ));
												
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_PRESET_GFI, 
												IVI_VAL_DIRECT_USER_CALL, Cmd_Str), 2, "setPresetGFI");       	


 	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
  
/*******************************************************************************
 * Function: C19032P_SetPresetScreen                                                  
*******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPresetScreen (ViSession vi,
                                           ViBoolean presetScreen)
{
	ViStatus	error = VI_SUCCESS; 
	ViString   Cmd_Str;
	
	checkErr   ( C19032P_LockSession( vi, VI_NULL ));
 	checkErr(C19032P_GetError_IfTrigStart (vi));   	

	viCheckParm( Ivi_SetAttributeViBoolean( vi, VI_NULL, C19032P_ATTR_PRESET_SCREEN, 
									IVI_VAL_DIRECT_USER_CALL, presetScreen), 2, "presetScreen");    	

	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetBuzzerPassHoldTime                                                  
 * Purpose: This function gets the buzzer warning time in pass result.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetBuzzerPassHoldTime (ViSession vi,
                                                ViReal64 *buzzerPassTime)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (buzzerPassTime==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for buzzerPassTime" );
	checkErr( Ivi_GetAttributeViReal64 (vi, VI_NULL, C19032P_ATTR_PRESET_PASS_HOLD_TIME, 0, buzzerPassTime));   		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetStepHoldTime                                                  
 * Purpose:  This function gets the time between with step and step or next start.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetStepHoldTime (ViSession vi, ViReal64 *stepHoldTime)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (stepHoldTime==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for stepHoldTime" );	
	
	checkErr( Ivi_GetAttributeViReal64 (vi, VI_NULL, C19032P_ATTR_PRESET_STEP_HOLD_TIME, 0, stepHoldTime));   
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetPresetStartWait                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetPresetStartWait (ViSession vi,
                                              ViReal64 *startWait)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (startWait==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for startWait" );	
	
	checkErr( Ivi_GetAttributeViReal64 (vi, VI_NULL, C19032P_ATTR_PRESET_START_WAIT, 0, startWait));   
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetPresetACFreq                                                  
 * Purpose:  This function gets the AC frequency on preset mode.   
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetPresetACFreq (ViSession vi,
                                          ViInt32 *presACVoltFrequency)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (presACVoltFrequency==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for presACVoltFrequency" );	
			
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_PRESET_AC_FREQUENCY, 0, presACVoltFrequency));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetPresetGBCurrFreq                                                  
 * Purpose:  This function gets the current freqency of GB mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetPresetGBCurrFreq (ViSession vi,
                                              ViReal64 *presGBCurrFreq)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (presGBCurrFreq==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for presGBCurrFreq" );		
		
	checkErr( Ivi_GetAttributeViReal64 (vi, VI_NULL, C19032P_ATTR_PRESET_GB_FREQUENCY, 0, presGBCurrFreq));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetPresetGBVoltage                                                  
 * Purpose:  This function gets the voltage of GB mode on preset mode.   
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetPresetGBVoltage (ViSession vi,
                                             ViReal64 *presGBVoltage)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (presGBVoltage==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for presGBVoltage" );	
		
	checkErr( Ivi_GetAttributeViReal64 (vi, VI_NULL, C19032P_ATTR_PRESET_GB_VOLTAGE, 0, presGBVoltage));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetAutoRange                                                  
 * Purpose:  This function gets the auto range on preset mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAutoRange (ViSession vi,
                                       ViInt32 *presAutoRange)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (presAutoRange==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for presAutoRange" );	
				
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_PRESET_AUTO_RANGE, 0, presAutoRange));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetSoftAGCState                                                  
 * Purpose:  This function gets the soft AGC state on preset mode.   
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetSoftAGCState (ViSession vi,
                                          ViInt32 *presSoftAGCState)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (presSoftAGCState==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for presSoftAGCState" );	
					
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_PRESET_SOFT_AGC_STATE, 0, presSoftAGCState));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetPartNo                                                  
 * Purpose:  This function gets the part number of product.   
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetPartNo (ViSession vi,
                                    ViChar getPartNo[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (getPartNo==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getPartNo" );	
						
	checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_PRESET_PART_NUMBER, 0, AUTO_BUFFER_SIZE, getPartNo));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLotNo                                                  
 * Purpose:  This function gets the lot number of product.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLotNo (ViSession vi, ViChar getLotNo[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (getLotNo==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getLotNo" );	
							
	checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_PRESET_LOT_NUMBER, 0, AUTO_BUFFER_SIZE,getLotNo));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetSerialNo                                                  
 * Purpose:  This function gets the serial number of product.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetSerialNo (ViSession vi,
                                      ViChar getSerialNo[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (getSerialNo==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getSerialNo" );	
								
	checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_PRESET_SERIAL_NUMBER, 0, AUTO_BUFFER_SIZE ,getSerialNo));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetGBCurrentOffset                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBCurrentOffset (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 GBCurrentOffset)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4]; 
	ViInt32		iStepType, CoercedVal, index=2;
	
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_GB)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, GBCurrentOffset, index, &CoercedVal));
	//	GBCurrentOffset = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_CURRENT_OFFSET, 0, GBCurrentOffset), 3, "GB Current Offset");
	}
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetGBModeParameter                                                  
 * Purpose:  This function sets parameters in GB mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBModeParameter (ViSession vi, ViReal64 current, ViReal64 highLimit,
                                             ViReal64 lowLimit, ViReal64 testTime, ViInt32 twinPort, ViInt32 step)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount=0;

	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
				
	if (step>=1 && step<=100 && step<=nStepCount+1)														 
	{
		Fmt(ChanId, "%s<%d", step);
	 	if (step==nStepCount+1)
	 		checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1));		
		if (highLimit < lowLimit)
			return VI_ERROR_INVALID_LIMIT_VALUE;
		
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_GB));
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_CURRENT, 0, current), 2, "GB Current");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_LIMIT_HIGH, 0, highLimit), 3, "GB high limit"); 
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_LIMIT_LOW, 0, lowLimit), 4, "GB low limit"); 
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_TEST_TIME, 0, testTime), 5, "GB test time");
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_GB_TWINPORT_STATE, 0, twinPort), 6, "GB twin port ON/OFF");       
	}												
	else
		return VI_ERROR_STEP_COUNT;

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetReportDataItem_19032P                                                  
 
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetReportDataItem_19032P (ViSession vi,
                                                    ViChar dataItem[])
{
	ViStatus	error = VI_SUCCESS;
	
	ViChar cBuf[BUFFER_SIZE],TempValue[BUFFER_SIZE];

			
	ViInt32		retCnt;
	 
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   

	if (!Ivi_Simulating(vi))
    {
		ViSession io= Ivi_IOSession(vi);
		
 		Fmt( cBuf, "%s<SOUR:SAFE:RES:AREP:ITEM?\n");
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, dataItem, BUFFER_SIZE-1, &retCnt));
		dataItem[retCnt] = 0;

	 }
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetGBCurrent                                                  
 * Purpose:  This function sets current in GB mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBCurrent (ViSession vi, ViInt32 step, ViReal64 current)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount=0, CoercedVal, index=3;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
				
	if (step>=1 && step<=100 && step<=nStepCount+1)														 
	{
		Fmt(ChanId, "%s<%d", step);
	 	if (step==nStepCount+1)
	 		viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1), 2, "Step count");
	 		
	 	checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_GB));
//		checkErr   ( C19032P_CheckFloat( vi, current, index, &CoercedVal));
//		current =  ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_CURRENT, 0, current), 3, "GB Current");
	}
	else
		return VI_ERROR_STEP_COUNT;
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetGBHighLimit                                                  
 * Purpose:  This function sets high limit in GB mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBHighLimit (ViSession vi, ViInt32 step, ViReal64 highLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=5;
	ViReal64	fLowLimit;
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_GB)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, highLimit, index, &CoercedVal));
//		highLimit =( ViReal64) CoercedVal/pow(10, index);

		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_LIMIT_LOW, 0, &fLowLimit));  
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_LIMIT_HIGH, 0, highLimit), 3, "GB high limit"); 
		if (highLimit<fLowLimit)
			checkErr( Ivi_SetAttributeViReal64 (vi, ChanId, C19032P_ATTR_GB_LIMIT_LOW, IVI_VAL_SET_CACHE_ONLY, highLimit));

			
	}
	       	
Error:
	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetGBLowLimit                                                  
 * Purpose:  This function sets low limit in GB mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBLowLimit (ViSession vi, ViInt32 step, ViReal64 lowLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4]; 
	ViInt32		iStepType, CoercedVal, index=5;
	ViReal64	fHighLimit;
	
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));  
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_GB)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, lowLimit, index, &CoercedVal));
//		lowLimit = ( ViReal64) CoercedVal/pow(10, index);
		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_LIMIT_HIGH, 0, &fHighLimit));  
	    if (lowLimit>0 && lowLimit<0.0001)
   			return VI_ERROR_DATA_OUT_OF_RANGE; 		
   		
		if (fHighLimit<lowLimit)
			return VI_ERROR_INVALID_LIMIT_VALUE;
		else
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_LIMIT_LOW, 0, lowLimit), 3, "GB low limit"); 
	}
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );  
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetGBTestTime                                                  
 * Purpose:  This function sets test time in GB mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBTestTime (ViSession vi, ViInt32 step, ViReal64 testTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4]; 
	ViInt32		iStepType, CoercedVal, index=2;
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_GB)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, testTime, index, &CoercedVal));
//		testTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_GB_TEST_TIME, 0, testTime), 3, "GB test time");
	}
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetGBTwinPort                                                  
 * Purpose:  This function sets twin port on/off in GB mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBTwinPort (ViSession vi, ViInt32 step, ViInt32 twinPort)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4]; 
	ViInt32		iStepType;
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_GB)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_GB_TWINPORT_STATE, 0, twinPort), 3, "GB twin port ON/OFF");
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetCurrOfStep                                                  
 * Purpose:  This function gets the current of the step .     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetGBCurrOfStep (ViSession vi,
                                          ViInt32 step, ViReal64 *GBCurrent)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (GBCurrent == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for GB Current");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_GB));  	
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_GB_CURRENT, 0, GBCurrent)); 
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetGBLimitHigh                                                  
 * Purpose:  This function gets the GB high limit of the step.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetGBLimitHigh (ViSession vi,
                                         ViInt32 step, ViReal64 *GBRLimit_ofHigh)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (GBRLimit_ofHigh == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for GB R Limit of High");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_GB));  	
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_GB_LIMIT_HIGH, 0, GBRLimit_ofHigh));   
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetGBLimitLow                                                  
 * Purpose:  This function gets the GB low limit of the step.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetGBLimitLow (ViSession vi, ViInt32 step,
                                        ViReal64 *GBRLimit_ofLow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (GBRLimit_ofLow == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for GB R Limit of Low");
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_GB));          
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_GB_LIMIT_LOW, 0, GBRLimit_ofLow)); 
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetGBTestTime                                                  
 * Purpose:  This function gets the GB test time of the step.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetGBTestTime (ViSession vi, ViInt32 step,
                                        ViReal64 *GBTestTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (GBTestTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for GB Test Time");
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_GB));          
	checkErr(CheckStepRange(vi,step));
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_GB_TEST_TIME, 0, GBTestTime));
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetTwinPortState                                                  
 * Purpose:  This function gets the Twin Port Status of the step.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetTwinPortState (ViSession vi, ViInt32 step,
                                           ViInt32 *twinPortState)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (twinPortState == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for twin Port State");
 	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_GB));         
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_GB_TWINPORT_STATE, 0, twinPortState));
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetGBChanState                                                  
 * Purpose:  This function gets the channel status of the GB mode.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetGBChanState (ViSession vi, ViInt32 step, ViChar channelStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 

	if (channelStatus==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for channelStatus" );	
						
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( CheckStepRange(vi,step));
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
	if (iMode != C19032P_VAL_MODE_GB)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:GB:CHAN:HIGH?\n", step);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( channelStatus, cResult, retCnt);
	}
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetGBCurrentOffset                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetGBCurrentOffset (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 *GBCurrentOffset)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (GBCurrentOffset == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for GB Current Offset");	
 	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_GB));        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_GB_CURRENT_OFFSET, 0, GBCurrentOffset));   
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetGBCurrentOffset                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACChanScanState (ViSession vi,
                                              ViInt32 step,
                                              ViChar ACChannelHighScanState[],
                                              ViChar ACChannelLowScanState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4],  cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL ;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);

	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
 
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:AC:CHAN:HIGH %s\n", step, ACChannelHighScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:AC:CHAN:LOW %s\n", step, ACChannelLowScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACCurrentOffset                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACCurrentOffset (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 ACCurrentOffset)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, ACCurrentOffset, index, &CoercedVal));
//		ACCurrentOffset = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_CURRENT_OFFSET, 0, ACCurrentOffset), 3, "AC test time");
	}	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACModeParameter                                                  
 * Purpose:  This function sets parameters in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACModeParameter (ViSession vi, ViReal64 voltage, ViReal64 highLimit,
                                             ViReal64 lowLimit, ViReal64 arcLimit, ViInt32 arcFilter,
                                             ViReal64 testTime, ViReal64 rampTime, ViInt32 step)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount;
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));	
	
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{
		Fmt(ChanId, "%s<%d", step);
		if (step==nStepCount+1)
	 		checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1));													
		if (highLimit < lowLimit)
			return VI_ERROR_INVALID_LIMIT_VALUE;
	
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_AC));	
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_VOLTAGE, 0, voltage), 2, "AC voltage"); 
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_LIMIT_HIGH, 0, highLimit), 3, "AC high limit");       
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_LIMIT_LOW, 0, lowLimit), 4, "AC low limit");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_ARC_LIMIT, 0, arcLimit), 5, "AC arc limit");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_ARC_FILTER, 0, (ViReal64)arcFilter), 6, "AC arc filter");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_TEST_TIME, 0, testTime), 7, "AC test time");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_RAMP_TIME, 0, rampTime), 8, "AC ramp time");       	
	}												
	else
		return VI_ERROR_STEP_COUNT;	
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACVoltage                                                  
 * Purpose:  This function sets voltage in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACVoltage (ViSession vi, ViInt32 step, ViReal64 voltage)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount, CoercedVal, index=1;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));	
	
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{
		Fmt(ChanId, "%s<%d", step);
		if (step==nStepCount+1)
	 		viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1), 2, "Step count");													
	
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_AC));	
//		checkErr   ( C19032P_CheckFloat( vi, voltage, index, &CoercedVal));
//		voltage =  ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_VOLTAGE, 0, voltage), 3, "AC voltage"); 
	}												
	else
		return VI_ERROR_STEP_COUNT;	
		

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACHighLimit                                                  
 * Purpose:  This function sets high limit in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACHighLimit (ViSession vi, ViInt32 step, ViReal64 highLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=8;
	ViReal64	fLowLimit;
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, highLimit, index, &CoercedVal));
//		highLimit =( ViReal64) CoercedVal/pow(10, index);

		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_LIMIT_LOW, 0, &fLowLimit)); 
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_LIMIT_HIGH, 0, highLimit), 3, "AC high limit"); 
		if (highLimit<fLowLimit)
			checkErr( Ivi_SetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_LIMIT_LOW, IVI_VAL_SET_CACHE_ONLY, highLimit));
	}
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACLowLimit                                                  
 * Purpose:  This function sets low limit in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACLowLimit (ViSession vi, ViInt32 step, ViReal64 lowLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=8;
	ViReal64	fHighLimit;
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, lowLimit, index, &CoercedVal));
//		lowLimit = ( ViReal64) CoercedVal/pow(10, index);

		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_LIMIT_HIGH, 0, &fHighLimit));   
		if (fHighLimit<lowLimit)
			return VI_ERROR_INVALID_LIMIT_VALUE;
		else
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_LIMIT_LOW, 0, lowLimit), 3, "AC low limit"); 
	}
	
Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetACLimitReal                                                  
 * Purpose: This command sets the selected STEP, the high limit value of AC withstand 
			leakage Real Current under GENERAL MODE. 
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACLimitReal (ViSession vi,
                                          ViInt32 step, ViReal64 ACLimitReal)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=8;
	ViReal64	fHighLimit;
	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, ACLimitReal, index, &CoercedVal));
//		ACLimitReal = ( ViReal64) CoercedVal/pow(10, index);

		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_LIMIT_HIGH, 0, &fHighLimit)); 
		if (fHighLimit<ACLimitReal)
			return VI_ERROR_INVALID_LIMIT_VALUE;
		else
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_LIMIT_REAL, 0, ACLimitReal), 3, "AC Arc Limit Real"); 
	}
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACArcLimit                                                  
 * Purpose:  This function sets arc limit in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACArcLimit (ViSession vi, ViInt32 step, ViReal64 arcLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=5;
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, arcLimit, index, &CoercedVal));
//		arcLimit = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_ARC_LIMIT, 0, arcLimit), 3, "AC arc limit");
	}
		
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACArcFilter                                                  
 * Purpose:  This function sets arc filter in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACArcFilter (ViSession vi, ViInt32 step, ViInt32 arcFilter)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_AC_ARC_FILTER, 0, arcFilter), 3, "AC arc filter");   // modify by angel 2006/04/14
	
Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetACFrequency                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACFrequency (ViSession vi,
                                          ViInt32 step, ViInt32 ACFrequency)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType;
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_AC_FREQUENCY, 0, ACFrequency), 3, "AC Frequency"); 
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACTestTime                                                  
 * Purpose:  This function sets test time in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACTestTime (ViSession vi, ViInt32 step, ViReal64 testTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, testTime, index, &CoercedVal));
//		testTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_TEST_TIME, 0, testTime), 3, "AC test time");
	}	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACRampTime                                                  
 * Purpose:  This function sets ramp time in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACRampTime (ViSession vi, ViInt32 step, ViReal64 rampTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, rampTime, index, &CoercedVal));
//		rampTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_RAMP_TIME, 0, rampTime), 3, "AC ramp time");
	}
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACFallTime                                                  
 * Purpose:  This function sets fall time in AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACFallTime (ViSession vi, ViInt32 step, ViReal64 fallTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=2;
	ViInt32		nModel;
	
	
	/*
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)	
		return VI_ERROR_NOT_SUPPORT_VERSION;
		
	*/
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, fallTime, index, &CoercedVal));
//		fallTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_AC_FALL_TIME, 0, fallTime), 3, "AC fall time");
	}
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetAcVoltOfStep                                                  
 * Purpose:  This function gets the WAC voltage value of the AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetAcVoltOfStep (ViSession vi, ViInt32 step, ViReal64 *ACVoltage)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (ACVoltage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for AC Voltage");
        
 	checkErr( CheckStepRange(vi,step));       
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_VOLTAGE, 0, ACVoltage));   
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
			  
/*******************************************************************************
 * Function: C19032P_GetWacHiCurrentLimitOfStep                                                  
 * Purpose:  This function gets the current limit of high of the AC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWacHiCurrentLimitOfStep (ViSession vi, ViInt32 step,
                                                     ViReal64 *WACHighCurrentLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WACHighCurrentLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WAC high Current limit");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_LIMIT_HIGH, 0, WACHighCurrentLimit));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWacLoCurrentLimitOfStep                                                  
 * Purpose: This function gets the current limit of low of the AC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWacLoCurrentLimitOfStep (ViSession vi, ViInt32 step,
                                                     ViReal64 *WACLowCurrentLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WACLowCurrentLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WAC low Current limit");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_LIMIT_LOW, 0, WACLowCurrentLimit));       
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetACCurrentOffset                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetACCurrentOffset (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 *ACCurrentOffset)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (ACCurrentOffset == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for AC Current Offset");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_CURRENT_OFFSET, 0, ACCurrentOffset));       
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetACLimitReal                                                  
 * Purpose: This function gets the ARC limit of the AC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetACLimitReal (ViSession vi,
                                          ViInt32 step, ViReal64 *ACLimitReal)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (ACLimitReal == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for AC Limit Real");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_LIMIT_REAL, 0, ACLimitReal));       
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWacArcLimitOfStep                                                  
 * Purpose: This function gets the ARC limit of the AC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWacArcLimitOfStep (ViSession vi, ViInt32 step,
                                               ViReal64 *WACARCLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WACARCLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WAC ARC Limit");	
        
 	checkErr( CheckStepRange(vi,step));       
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_ARC_LIMIT, 0, WACARCLimit));      
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWacArcFilterOfStep                                                  
 * Purpose: This function gets the ARC filter of the AC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWacArcFilterOfStep (ViSession vi, ViInt32 step,
                                                ViInt32 *WACARCFilter)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WACARCFilter == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WAC ARC Filter");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_AC_ARC_FILTER, 0, WACARCFilter)); 

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetACFrequency                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetACFrequency (ViSession vi,
                                          ViInt32 step, ViInt32 *ACFrequency)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (ACFrequency == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for AC Frequency");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_AC_FREQUENCY, 0, ACFrequency));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWacRampTimeOfStep                                                  
 * Purpose: This function gets the ramp time of the WAC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWacRampTimeOfStep (ViSession vi, ViInt32 step,
                                               ViReal64 *WACRampTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WACRampTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WAC ramp Time");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_RAMP_TIME, 0, WACRampTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWacTestTimeOfStep                                                  
 * Purpose: This function gets the test time of the AC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWacTestTimeOfStep (ViSession vi, ViInt32 step,
                                               ViReal64 *WACTestTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;

	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WACTestTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WAC test Time");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_TEST_TIME, 0, WACTestTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetACFallTime                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetACFallTime (ViSession vi,
                                         ViInt32 step, ViReal64 *fallTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;

	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (fallTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for fall Time");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_AC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_AC_FALL_TIME, 0, fallTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetACChanHighState                                                  
 * Purpose: This function gets the channel status of high of the WAC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetACChanHighState (ViSession vi, ViInt32 step,
                                             ViChar WACHighChannelStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt; 
	ViSession	io = Ivi_IOSession (vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
    if (WACHighChannelStatus == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WAC high channel Status");
        
	checkErr( CheckStepRange(vi,step));
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_AC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:AC:CHAN:HIGH?\n", step);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( WACHighChannelStatus, cResult, retCnt);
	}

Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetACChanLowState                                                  
 * Purpose: This function gets the channel status of low of the WAC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetACChanLowState (ViSession vi, ViInt32 step,
                                            ViChar WACLowChannelStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
    if (WACLowChannelStatus == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for channel Status");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_AC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:AC:CHAN:LOW?\n", step);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( WACLowChannelStatus, cResult, retCnt);
	}

Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetDCCurrentOffset                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCCurrentOffset (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 DCCurrentOffset)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
		viCheckParm( Ivi_SetAttributeViReal64 ( vi, ChanId, C19032P_ATTR_DC_CURRENT_OFFSET, 0, DCCurrentOffset), 3, "DC Current Offset"); 
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCModeParameter                                                  
 * Purpose:  This function sets parameters in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCModeParameter (ViSession vi, ViReal64 voltage, ViReal64 highLimit,
                                             ViReal64 lowLimit, ViReal64 dwellTime, ViReal64 arcLimit,
                                             ViInt32 arcFilter, ViReal64 testTime, ViReal64 rampTime, ViInt32 step)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount, iVersion;   
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &iVersion));

	if (step>=1 && step<=100 && step<=nStepCount+1)
	{	
		Fmt(ChanId, "%s<%d", step);
	 	if (step==nStepCount+1)
	 		checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1));												
		if (highLimit < lowLimit)
			return VI_ERROR_INVALID_LIMIT_VALUE;
											
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_DC));	
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_VOLTAGE, 0, voltage), 2, "DC voltage"); 
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_LIMIT_HIGH, 0, highLimit), 3, "DC high limit");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_LIMIT_LOW, 0, lowLimit), 4, "DC low limit");
		if (iVersion>=500)
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_DWELL_TIME, 0, dwellTime), 5, "DC dwell time");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_ARC_LIMIT, 0, arcLimit), 6, "DC arc limit");       						
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_ARC_FILTER, 0, (ViReal64)arcFilter), 7, "DC arc filter");  
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_TEST_TIME, 0, testTime), 8, "DC test time");  
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_RAMP_TIME, 0, rampTime), 9, "DC ramp time");       
	}										
	else
		return VI_ERROR_STEP_COUNT;												

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCVoltage                                                  
 * Purpose:  This function sets voltage in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCVoltage (ViSession vi, ViInt32 step, ViReal64 voltage)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount, CoercedVal, index=1; 
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));	

	if (step>=1 && step<=100 && step<=nStepCount+1)
	{	
		Fmt(ChanId, "%s<%d", step);
	 	if (step==nStepCount+1)
	 		viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1), 2, "Step Count");												
											
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_DC));	
//		checkErr   ( C19032P_CheckFloat( vi, voltage, index, &CoercedVal));
//		voltage =  ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_VOLTAGE, 0, voltage), 3, "DC voltage"); 
	}										
	else
		return VI_ERROR_STEP_COUNT;												

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCHighLimit                                                  
 * Purpose:  This function sets high limit in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCHighLimit (ViSession vi, ViInt32 step, ViReal64 highLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=8;
	ViReal64	fLowLimit;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, highLimit, index, &CoercedVal));
//		highLimit =( ViReal64) CoercedVal/pow(10, index);

		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_LIMIT_LOW, 0, &fLowLimit)); 
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_LIMIT_HIGH, 0, highLimit), 3, "DC high limit"); 
		if (highLimit<fLowLimit)
			checkErr( Ivi_SetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_LIMIT_LOW, IVI_VAL_SET_CACHE_ONLY, highLimit));
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCLowLimit                                                  
 * Purpose:  This function sets low limit in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCLowLimit (ViSession vi, ViInt32 step, ViReal64 lowLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=8;
	ViReal64	fHighLimit;

	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, lowLimit, index, &CoercedVal));
//		lowLimit = ( ViReal64) CoercedVal/pow(10, index);

		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_LIMIT_HIGH, 0, &fHighLimit));    
		if (fHighLimit<lowLimit)
			return VI_ERROR_INVALID_LIMIT_VALUE;
		else
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_LIMIT_LOW, 0, lowLimit), 3, "DC low limit"); 
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCDwellTime                                                  
 * Purpose:  This function sets dwell time in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCDwellTime (ViSession vi, ViInt32 step, ViReal64 dwellTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iVersion, CoercedVal, index=2;
	ViInt32		nModel;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
//	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &iVersion));
	
//	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel)); 
	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		if (iVersion>=500 || nModel == C19032P_VAL_MODEL_19032P)
//		{
//			checkErr   ( C19032P_CheckFloat( vi, dwellTime, index, &CoercedVal));
//			dwellTime =( ViReal64) CoercedVal/pow(10, index);
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_DWELL_TIME, 0, dwellTime), 3, "DC dwell time");
//		}
//		else
//			return VI_ERROR_NOT_SUPPORT_VERSION;
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCArcLimit                                                  
 * Purpose:  This function sets arc limit in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCArcLimit (ViSession vi, ViInt32 step, ViReal64 arcLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=5;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, arcLimit, index, &CoercedVal));
//		arcLimit = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_ARC_LIMIT, 0, arcLimit), 3, "DC arc limit");       						
	}
		
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCArcFilter                                                  
 * Purpose:  This function sets arc filter in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCArcFilter (ViSession vi, ViInt32 step, ViInt32 arcFilter)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_DC_ARC_FILTER, 0, arcFilter), 3, "DC arc filter");   // modify by angel 2006/04/14
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCTestTime                                                  
 * Purpose:  This function sets test time in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCTestTime (ViSession vi, ViInt32 step, ViReal64 testTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, testTime, index, &CoercedVal));
//		testTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_TEST_TIME, 0, testTime), 3, "DC test time");  
	}	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCRampTime                                                  
 * Purpose:  This function sets ramp time in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCRampTime (ViSession vi, ViInt32 step, ViReal64 rampTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, rampTime, index, &CoercedVal));
//		rampTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_RAMP_TIME, 0, rampTime), 3, "DC ramp time");  
	}
		
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCFallTime                                                  
 * Purpose:  This function sets fall time in DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCFallTime (ViSession vi, ViInt32 step, ViReal64 fallTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=2;
	ViInt32		nModel;

	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
//		return VI_ERROR_NOT_SUPPORT_VERSION;
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, fallTime, index, &CoercedVal));
//		fallTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_DC_FALL_TIME, 0, fallTime), 3, "DC fall time");  
	}
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetdcVoltOfStep                                                  
 * Purpose:  This function gets the WDC voltage value of the DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetdcVoltOfStep (ViSession vi, ViInt32 step, ViReal64 *DCVoltage)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (DCVoltage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for AC Voltage");
        
 	checkErr( CheckStepRange(vi,step));       
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_VOLTAGE, 0, DCVoltage));  
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWdcHiCurrentLimitOfStep                                                  
 * Purpose:  This function gets the current limit of high of the DC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWdcHiCurrentLimitOfStep (ViSession vi, ViInt32 step,
                                                     ViReal64 *WDCHighCurrentLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WDCHighCurrentLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for high Current limit");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_LIMIT_HIGH, 0, WDCHighCurrentLimit));   
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWdcLoCurrentLimitOfStep                                                  
 * Purpose: This function gets the current limit of low of the DC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWdcLoCurrentLimitOfStep (ViSession vi, ViInt32 step,
                                                     ViReal64 *WDCLowCurrentLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WDCLowCurrentLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WDC low Current limit");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_LIMIT_LOW, 0, WDCLowCurrentLimit));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetDCDwellTime                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetDCDwellTime (ViSession vi,
                                          ViInt32 step, ViReal64 *dwellTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (dwellTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for dwell Time");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_DWELL_TIME, 0, dwellTime)); 
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWdcArcLimitOfStep                                                  
 * Purpose: This function gets the ARC limit of the DC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWdcArcLimitOfStep (ViSession vi, ViInt32 step,
                                               ViReal64 *WDCARCLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WDCARCLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WDC ARC Limit");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_ARC_LIMIT, 0, WDCARCLimit)); 
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWdcArcFilterOfStep                                                  
 * Purpose: This function gets the ARC filter of the DC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWdcArcFilterOfStep (ViSession vi, ViInt32 step,
                                                ViInt32 *WDCARCFilter)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WDCARCFilter == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WDC ARC Filter");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_DC_ARC_FILTER, 0, WDCARCFilter));

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWdcRampTimeOfStep                                                  
 * Purpose: This function gets the ramp time of the DC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWdcRampTimeOfStep (ViSession vi,
                                               ViInt32 step,
                                               ViReal64 *WDCRampTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WDCRampTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WDC ramp Time");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_RAMP_TIME, 0, WDCRampTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWdcTestTimeOfStep                                                  
 * Purpose: This function gets the test time of the DC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWdcTestTimeOfStep (ViSession vi, ViInt32 step,
                                               ViReal64 *WDCTestTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (WDCTestTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WDC test Time");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_TEST_TIME, 0, WDCTestTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetDCFallTime                                                  
********************************************************************************/
ViStatus _VI_FUNC C19032P_GetDCFallTime (ViSession vi,
                                         ViInt32 step, ViReal64 *fallTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (fallTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WDC fall Time");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_FALL_TIME, 0, fallTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWDCCChanHighState                                                  
 * Purpose: This function gets the channel status of high of the DC mode.  
********************************************************************************/
ViStatus _VI_FUNC C19032P_GetWDCChanHighState (ViSession vi, ViInt32 step,
                                              ViChar WDCHighChannelStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
    if (WDCHighChannelStatus == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WDC high channel Status");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DC:CHAN:HIGH?\n", step);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( WDCHighChannelStatus, cResult, retCnt);
	}
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetWDCChanLowState                                                  
 * Purpose: This function gets the channel status of low of the WDC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetWDCChanLowState (ViSession vi, ViInt32 step,
                                             ViChar WDCLowChannelStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
    if (WDCLowChannelStatus == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for WDC low channel Status");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DC:CHAN:LOW?\n", step);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( WDCLowChannelStatus, cResult, retCnt);
	}
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetIRModeParameter                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetDCCurrentOffset (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 *DCCurrentOffset)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (DCCurrentOffset == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for DC Current Offset");	
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode != C19032P_VAL_MODE_DC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_DC_CURRENT_OFFSET, 0, DCCurrentOffset));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRModeParameter                                                  
 * Purpose:  This function sets parameters in IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRModeParameter (ViSession vi, ViReal64 voltage, ViReal64 lowLimit, ViReal64 highLimit,
                                             ViReal64 testTime, ViReal64 rampTime, ViInt32 step)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount;
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));	
	
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{	
		Fmt(ChanId, "%s<%d", step);
		if (step==nStepCount+1)
		 	checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1));											
		if (highLimit < lowLimit)
		{
			if (highLimit!=0)
				return VI_ERROR_INVALID_LIMIT_VALUE;
		}
	
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_IR));	
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_VOLTAGE, 0, voltage), 2, "IR voltage");  
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_LIMIT_LOW, 0, lowLimit), 3, "IR low imit");       	
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_LIMIT_HIGH, 0, highLimit), 4, "IR high limit"); 		 
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_TEST_TIME, 0, testTime), 5, "IR test time");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_RAMP_TIME, 0, rampTime), 6, "IR ramp time");
	}										
	else
		return VI_ERROR_STEP_COUNT;													

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRVoltage                                                  
 * Purpose:  This function sets voltage in IR mode.    
 *******************************************************************************/
 ViStatus _VI_FUNC C19032P_SetIRVoltage (ViSession vi, ViInt32 step, ViReal64 voltage)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount, CoercedVal, index=1;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));	
	
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{	
		Fmt(ChanId, "%s<%d", step);
		if (step==nStepCount+1)
		 	viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1), 2, "Step count");											
	
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_IR));	
//		checkErr   ( C19032P_CheckFloat( vi, voltage, index, &CoercedVal));
//		voltage =  ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_VOLTAGE, 0, voltage), 3, "IR voltage");  
	}										
	else
		return VI_ERROR_STEP_COUNT;													

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRLowLimit                                                  
 * Purpose:  This function sets low limit in IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRLowLimit (ViSession vi, ViInt32 step, ViReal64 lowLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType;
	ViReal64	fHighLimit;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
	
		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_LIMIT_HIGH, 0, &fHighLimit));   	
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_LIMIT_LOW, 0, lowLimit), 3, "IR low limit"); 
		if (lowLimit>fHighLimit && fHighLimit!=0)
			checkErr( Ivi_SetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_LIMIT_HIGH, IVI_VAL_SET_CACHE_ONLY, lowLimit));
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRHighLimit                                                  
 * Purpose:  This function sets high limit in IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRHighLimit (ViSession vi, ViInt32 step, ViReal64 highLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType;
	ViReal64	fLowLimit;

	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_LIMIT_LOW, 0, &fLowLimit));  	
	
		if (fLowLimit>highLimit && highLimit!=0)
			return VI_ERROR_INVALID_LIMIT_VALUE;
		else
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_LIMIT_HIGH, 0, highLimit), 3, "IR high limit"); 
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRTestTime                                                  
 * Purpose:  This function sets test time in IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRTestTime (ViSession vi, ViInt32 step, ViReal64 testTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, testTime, index, &CoercedVal));
//		testTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_TEST_TIME, 0, testTime), 3, "IR test time");
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRRampTime                                                  
 * Purpose:  This function sets ramp time in IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRRampTime (ViSession vi, ViInt32 step, ViReal64 rampTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, rampTime, index, &CoercedVal));
//		rampTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_RAMP_TIME, 0, rampTime), 3, "IR ramp time");
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRFallTime                                                  
 * Purpose:  This function sets fall time in IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRFallTime (ViSession vi, ViInt32 step, ViReal64 fallTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=2;
	ViInt32		nModel;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)		
//		return VI_ERROR_NOT_SUPPORT_VERSION;
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, fallTime, index, &CoercedVal));
//		fallTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_FALL_TIME, 0, fallTime), 3, "IR fall time");
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRCurrRange                                                  
 * Purpose:  This function sets current range in IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRCurrRange (ViSession vi, ViInt32 step, ViInt32 range)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nFWVersion, iStepType;
	ViReal64	fRange[6]={200.0e-9, 2.0e-6, 20.0e-6, 200.0e-6, 2.0e-3, 9.0e-3};
	ViInt32		nModel;
	
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
		return VI_ERROR_NOT_SUPPORT_VERSION;
		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		if (range==0)
			viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_IR_CURR_RANGE_AUTO, 0, 1), 3, "IR current range auto on");
		else
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_CURR_RANGE_UPPER, 0, fRange[range-1]), 3, "IR current range");	   // modify by angel 2006/04/14
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetIRVoltOfStep                                                  
 * Purpose:  This function gets the current limit of high of the WAC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRVoltOfStep (ViSession vi, ViInt32 step, ViReal64 *IRTestVoltage)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (IRTestVoltage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IR Test Voltage");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return error=VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_VOLTAGE, 0, IRTestVoltage));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}



/*******************************************************************************
 * Function: C19032P_GetIRLoResiLimitOfStep                                                  
 * Purpose: This function gets the resistor limit of low of the IR mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRLoResiLimitOfStep (ViSession vi, ViInt32 step,
                                                 ViReal64 *IRLowResistorLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (IRLowResistorLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IR Low Resistor Limit");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_LIMIT_LOW, 0, IRLowResistorLimit));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetIRHiResiLimitOfStep                                                  
 * Purpose: This function gets the resistor limit of high of the IR mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRHiResiLimitOfStep (ViSession vi, ViInt32 step,
                                                  ViReal64 *IRHighResistorLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (IRHighResistorLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IR High Resistor Limit");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_LIMIT_HIGH, 0, IRHighResistorLimit));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetIRRampTimeOfStep                                                  
 * Purpose: This function gets the ramp time of the IR mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRRampTimeOfStep (ViSession vi, ViInt32 step, ViReal64 *IRRampTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (IRRampTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IR Ramp Time");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_RAMP_TIME, 0, IRRampTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetIRTestTimeOfStep                                                  
 * Purpose: This function gets the test time of the IR mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRTestTimeOfStep (ViSession vi, ViInt32 step, ViReal64 *IRTestTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (IRTestTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IR Test Time");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_TEST_TIME, 0, IRTestTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetIRFallTime                                                  
*******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRFallTime (ViSession vi,
                                         ViInt32 step, ViReal64 *fallTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (fallTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IR fall Time");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_FALL_TIME, 0, fallTime));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetIRRangeAuto                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRRangeAuto (ViSession vi,
                                          ViInt32 step, ViInt32 *rangeAuto)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (rangeAuto == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for range Auto");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_IR_CURR_RANGE_AUTO, 0, rangeAuto));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
 /*******************************************************************************
 * Function: C19032P_GetIRCurrHighRange                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRCurrHighRange (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 *IRCurrHighRange)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (IRCurrHighRange == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IRCurrHighRange");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_CURR_HIGH_RANGE, 0, IRCurrHighRange));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
 /*******************************************************************************
 * Function: C19032P_GetIRCurrLowRange                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRCurrLowRange (ViSession vi,
                                             ViInt32 step,
                                             ViReal64 *IRCurrLowRange)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (IRCurrLowRange == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IRCurrLowRange");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_IR_CURRENT_LOW_RANGE, 0, IRCurrLowRange));       
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetIRChanHighState                                                  
 * Purpose:  This function gets the channel status of high of the IR mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRChanHighState (ViSession vi, ViInt32 step,
                                             ViChar IRHighChannelStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	
    if (IRHighChannelStatus == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IR High Channel Status");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:IR:CHAN:HIGH?\n", step);
		viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( IRHighChannelStatus, cResult, retCnt);
	}
					
Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetIRChanLowState                                                  
 * Purpose: This function gets the channel status of low of the IR mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetIRChanLowState (ViSession vi, ViInt32 step,
                                            ViChar IRLowChannelStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt;
	ViSession	io = Ivi_IOSession(vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (IRLowChannelStatus == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IR Low Channel Status");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_IR)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	else  
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:IR:CHAN:LOW?\n", step);
		viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( IRLowChannelStatus, cResult, retCnt);
	}

Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetPAUnderTest                                                  
 * Purpose:  This function sets parameters in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetPAUnderTest (ViSession vi,
                                          ViInt32 step,
                                          ViBoolean *getSignal_ofPause)
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3];
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    if (getSignal_ofPause == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getSignal_ofPause");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	
		checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_PA));  
	checkErr( Ivi_GetAttributeViBoolean (vi, ChanId, C19032P_ATTR_PAUSE_SIGNAL_STATE, 0, getSignal_ofPause));  

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*******************************************************************************
 * Function: C19032P_GetPATestTime                                                  
 * Purpose:  This function sets parameters in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetPATestTime (ViSession vi,
                                         ViInt32 step, ViReal64 *testTime)
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3];
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    if (testTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for testTime");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_PA));  	
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_PAUSE_TEST_TIME, 0, testTime));  

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCChanScanState                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCChanScanState (ViSession vi,
                                              ViInt32 step,
                                              ViChar LCChannelHighScanState[],
                                              ViChar LCChannelLowScanState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4],  cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL ;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);

	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
 
	if (iMode != C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:LC:CHAN:HIGH %s\n", step, LCChannelHighScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:LC:CHAN:LOW %s\n", step, LCChannelLowScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCChanLowState                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCChanLowState (ViSession vi,
                                             ViInt32 step, ViInt32 scannerNumber,
                                             ViInt32 channelState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4], Scanner[4], cLcState[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCount, onoff[8]={0,0,0,0,0,0,0,0}, i, iScanType, iTemp[8]={0,0,0,0,0,0,0,0};
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%d", step);
	Fmt     ( Scanner, "%d", scannerNumber);
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));  		
	if (iMode != C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	

	checkErr( Ivi_GetAttributeViInt32 (vi, Scanner, C19032P_ATTR_OPTION_SCAN_TYPE, 0, &iScanType));
	if (scannerNumber<1 || scannerNumber>8)
		return VI_ERROR_SCAN_NUM;
	if (iScanType==0)
		return VI_ERROR_SCANNER_BOX_NULL;
		
		

	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:LC:CHAN:LOW (@%d(0))\n", step, scannerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		switch (iScanType)
		{
			case C19032P_VAL_GB_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = iGB_FloatingChState[4][i];
				break;
			case C19032P_VAL_6000_01:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_01ChState[4][i];
				break;
			case C19032P_VAL_6000_02:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_02ChState[4][i];
				break;
			case C19032P_VAL_6000_03:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_03ChState[4][i];
				break;
			case C19032P_VAL_6000_04:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04ChState[4][i];
				break;
			case C19032P_VAL_6000_06:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06ChState[4][i];
				break;
			case C19032P_VAL_9030A: 
				for (i=0;i<8;i++)
					iTemp[i] = i9030AChState[4][i];
				break;
			case C19032P_VAL_6000_04_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04FloatingChState[4][i];
				break;
			case C19032P_VAL_6000_06_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06FloatingChState[4][i];
				break;
			case C19032P_VAL_6000_11:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_11ChState[4][i];
				break;
		}
	
		for (i=0;i<8;i++)
		{
			onoff[i] = (iTemp[i]>=0)? 1 : 0;
			if (onoff[i])
			{
				switch (iTemp[i])
				{
					case C19032P_VAL_X:
						if (channelState[i]!=0)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_LX:
						if (channelState[i]!=0 && channelState[i]!=1)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HX:
						if (channelState[i]!=0 && channelState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HLX:
						if (channelState[i]!=0 && channelState[i]!=1 && channelState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
				}	  
			}
		}
	
		nCount = 0;
		memset( cLcState, 0, sizeof(cLcState));
		nCount += sprintf(cLcState + nCount, "(@%d(", scannerNumber);

		for (i=0;i<8;i++)
		{
			if (onoff[i] && channelState[i]==1)
				nCount += sprintf (cLcState + nCount, "%d,", i+1);
		//	sprintf ( ChanId, "%d", scanerNumber*100+i+1);
		//	checkErr( Ivi_AddToChannelTable (vi, ChanId));
		//	checkErr( Ivi_ApplyDefaultSetup (vi));
		//	checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_CHANNEL_STATE, IVI_VAL_SET_CACHE_ONLY, (onoff[i])? ChState[i] : -1));
		}
		
		if (nCount==4)	
			nCount += sprintf(cLcState + (nCount-1), "(0))");  	//(nCount_H-1) to  avoid ","
		else
			nCount += sprintf(cLcState + (nCount-1), "))");  		//(nCount_H-1) to avoid ","  

		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:LC:CHAN:LOW %s\n", step, cLcState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCChanHighState                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCChanHighState (ViSession vi,
                                              ViInt32 step,
                                              ViInt32 scannerNumber,
                                              ViInt32 channelState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4], Scanner[4], cLcState[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCount, onoff[8]={0,0,0,0,0,0,0,0}, i, iScanType, iTemp[8]={0,0,0,0,0,0,0,0};
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%d", step);
	Fmt     ( Scanner, "%d", scannerNumber);
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
		if (iMode != C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	checkErr( Ivi_GetAttributeViInt32 (vi, Scanner, C19032P_ATTR_OPTION_SCAN_TYPE, 0, &iScanType));
	if (scannerNumber<1 || scannerNumber>8)
		return VI_ERROR_SCAN_NUM;
	if (iScanType==0)
		return VI_ERROR_SCANNER_BOX_NULL;

	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:LC:CHAN:HIGH (@%d(0))\n", step, scannerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		switch (iScanType)
		{
			case C19032P_VAL_GB_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = iGB_FloatingChState[4][i];
				break;
			case C19032P_VAL_6000_01:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_01ChState[4][i];
				break;
			case C19032P_VAL_6000_02:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_02ChState[4][i];
				break;
			case C19032P_VAL_6000_03:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_03ChState[4][i];
				break;
			case C19032P_VAL_6000_04:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04ChState[4][i];
				break;
			case C19032P_VAL_6000_06:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06ChState[4][i];
				break;
			case C19032P_VAL_9030A: 
				for (i=0;i<8;i++)
					iTemp[i] = i9030AChState[4][i];
				break;
			case C19032P_VAL_6000_04_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04FloatingChState[4][i];
				break;
			case C19032P_VAL_6000_06_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06FloatingChState[4][i];
				break;
			case C19032P_VAL_6000_11:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_11ChState[4][i];
				break;
		}
	
		for (i=0;i<8;i++)
		{
			onoff[i] = (iTemp[i]>=0)? 1 : 0;
			if (onoff[i])
			{
				switch (iTemp[i])
				{
					case C19032P_VAL_X:
						if (channelState[i]!=0)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_LX:
						if (channelState[i]!=0 && channelState[i]!=1)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HX:
						if (channelState[i]!=0 && channelState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HLX:
						if (channelState[i]!=0 && channelState[i]!=1 && channelState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
				}	  
			}
		}
	
		nCount = 0;
		memset( cLcState, 0, sizeof(cLcState));
		nCount += sprintf(cLcState + nCount, "(@%d(", scannerNumber);

		for (i=0;i<8;i++)
		{
			if (onoff[i] && channelState[i]==1)
				nCount += sprintf (cLcState + nCount, "%d,", i+1);
		//	sprintf ( ChanId, "%d", scanerNumber*100+i+1);
		//	checkErr( Ivi_AddToChannelTable (vi, ChanId));
		//	checkErr( Ivi_ApplyDefaultSetup (vi));
		//	checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_CHANNEL_STATE, IVI_VAL_SET_CACHE_ONLY, (onoff[i])? ChState[i] : -1));
		}
		
		if (nCount==4)	
			nCount += sprintf(cLcState + (nCount-1), "(0))");  	//(nCount_H-1) to  avoid ","
		else
			nCount += sprintf(cLcState + (nCount-1), "))");  		//(nCount_H-1) to avoid ","  

		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:LC:CHAN:HIGH %s\n", step, cLcState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCCurrentOffset                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCCurrentOffset (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 LCCurrentOffset)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4],cScanner[20],RMS[10];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2,iDevice;
	ViReal64    LCLimitHigh,SumOffset;
	
 	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	if(  LCCurrentOffset<=0  )	
			return VI_ERROR_DATA_OUT_OF_RANGE;  
			
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

		
//	checkErr   ( C19032P_CheckFloat( vi, LCLimitHigh, index, &CoercedVal));
//	LCLimitHigh =( ViReal64) CoercedVal/pow(10, index);
		
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
  	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_DEVICE_TYPE, 0, &iDevice));     
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_LIMIT_HIGH, 0, &LCLimitHigh)); 
	
	
	
		SumOffset=LCLimitHigh+LCCurrentOffset; 
//		checkErr   ( C19032P_CheckFloat( vi, SumOffset, index, &CoercedVal));
//		SumOffset =( ViReal64) CoercedVal/pow(10, index);	
	
	 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0,AUTO_BUFFER_SIZE, cScanner));
	
		if (!strncmp(cScanner, "6000-05", 7) || !strncmp(cScanner, "6000-07", 7))
		{
			if( LCLimitHigh<=0.0000599 && LCLimitHigh >=0.0000001 ) 
				if (SumOffset>0.000066)
				   return VI_ERROR_DATA_OUT_OF_RANGE;  
				   
			if( LCLimitHigh<=0.000599 && LCLimitHigh >=0.00006 ) 
				if (SumOffset>0.00066)
				   return VI_ERROR_DATA_OUT_OF_RANGE; 
				   
			if(  LCLimitHigh>=0.0006  )	
			{
			  if((iDevice==0 && SumOffset>0.0066)
			    || (iDevice==1 && SumOffset>0.011) 
				|| (iDevice==2 && SumOffset>0.011) 		    
				|| (iDevice==3 && SumOffset>0.011) 
				|| (iDevice==4 && SumOffset>0.011)) 				
					return VI_ERROR_DATA_OUT_OF_RANGE;
			}
				   
		 }
	
		if (!strncmp(cScanner, "6000-04", 7) || !strncmp(cScanner, "6000-06", 7) )
		{				   
				   
			if( LCLimitHigh<=0.000599 && LCLimitHigh >=0.0000001 ) 
					if (SumOffset>0.00066)
					   return VI_ERROR_DATA_OUT_OF_RANGE;  

				if(  LCLimitHigh>=0.0006  )	
				{
				  if((iDevice==0 && SumOffset>0.0066)
				    || (iDevice==1 && SumOffset>0.011) 
					|| (iDevice==2 && SumOffset>0.011) 		    
					|| (iDevice==3 && SumOffset>0.011) 
					|| (iDevice==4 && SumOffset>0.011))
					 return VI_ERROR_DATA_OUT_OF_RANGE;      

				}			

		 }
		
		if (!strncmp(cScanner, "6000-08", 7))
		{				   
	 	checkErr( Ivi_GetAttributeViString (vi, ChanId, C19032P_ATTR_LC_DISPLAY, 0,AUTO_BUFFER_SIZE, RMS)); 				   
			if( LCLimitHigh<=0.000599 && LCLimitHigh >=0.0000001 ) 
					if (SumOffset>0.00066)
					   return VI_ERROR_DATA_OUT_OF_RANGE;  
				   

				if(  LCLimitHigh>=0.0006  )	
				{
				  if((iDevice==0 && SumOffset>0.0066)
				    || (iDevice==1 && SumOffset>0.011) 
					|| (iDevice==2 && SumOffset>0.011) 		    
					|| (iDevice==3 && SumOffset>0.011) 
					|| (iDevice==4 && SumOffset>0.011)
					|| (iDevice==5 && SumOffset>0.055 && !strncmp(RMS, "RMS", 3)  )
					|| (iDevice==5 && SumOffset>0.077 && !strncmp(RMS, "PEAK", 4)  ) 	
					|| (iDevice==6 && SumOffset>0.011))						
					 return VI_ERROR_DATA_OUT_OF_RANGE;      

				}			

		 }		
		
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_CURRENT_OFFSET, 0, LCCurrentOffset), 3, "LC Current Offset");  		
	 }
Error:
	return error;
}
	
/*******************************************************************************
 * Function: C19032P_SetLCCurrentOffsetLAC                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCCurrentOffsetLAC (ViSession vi,
                                                 ViInt32 step,
                                                 ViReal64 LCCurrentOffsetLAC)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4],cScanner[20],RMS[10];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2,iDevice;
	ViReal64    LCLimitHigh,SumOffset,LACLimitHigh;

	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	if(  LCCurrentOffsetLAC<=0  )	
			return VI_ERROR_DATA_OUT_OF_RANGE;  

	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));

	if (strncmp (cScanner, "6000-08", 7)!=0)  //for 6000-08 only 
		return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;				
			
			
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

		
//	checkErr   ( C19032P_CheckFloat( vi, LCLimitHigh, index, &CoercedVal));
//	LCLimitHigh =( ViReal64) CoercedVal/pow(10, index);
		
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
	  	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_DEVICE_TYPE, 0, &iDevice)); 
  	
	 	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_LIMIT_HIGH, 0, &LCLimitHigh));   	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_AC_HIGH_LIMIT, 0, &LACLimitHigh)); 

	
	
		SumOffset=LACLimitHigh+LCCurrentOffsetLAC; 
//		checkErr   ( C19032P_CheckFloat( vi, SumOffset, index, &CoercedVal));
//		SumOffset =( ViReal64) CoercedVal/pow(10, index);	
	
 	 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, AUTO_BUFFER_SIZE, cScanner));  
/*	 	
	
		if (strncmp(cScanner, "6000-08", 7)!=0)
		{	 
	  	 	
			if( LCLimitHigh<=0.000599 && LCLimitHigh >=0.0000001 ) 
				if (SumOffset>0.00066)
				   return VI_ERROR_DATA_OUT_OF_RANGE;  

			if(  LCLimitHigh>=0.0006  )	
			{
			  if((iDevice==0 && SumOffset>0.0066)
			    || (iDevice==1 && SumOffset>0.011) 
				|| (iDevice==2 && SumOffset>0.011) 		    
				|| (iDevice==3 && SumOffset>0.011) 
				|| (iDevice==4 && SumOffset>0.011)) 				
					return VI_ERROR_DATA_OUT_OF_RANGE;
			}
				   
		 }
*/		 
	 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));  
	 	
	
		if (!strncmp(cScanner, "6000-08", 7))
		{				   
    	 	checkErr( Ivi_GetAttributeViString (vi, ChanId, C19032P_ATTR_LC_DISPLAY, 0,AUTO_BUFFER_SIZE, RMS)); 	 	
			if( LCLimitHigh<=0.000599 && LCLimitHigh >=0.0000001 ) 
				if (SumOffset>0.00066)
				   return VI_ERROR_DATA_OUT_OF_RANGE;  				   

				   
				if(  LCLimitHigh>=0.0006  )	
				{
				  if((iDevice==0 && SumOffset>0.0066)
				    || (iDevice==1 && SumOffset>0.011) 
					|| (iDevice==2 && SumOffset>0.011) 		    
					|| (iDevice==3 && SumOffset>0.011) 
					|| (iDevice==4 && SumOffset>0.011)
					|| (iDevice==5 && SumOffset>0.055 && !strncmp(RMS, "RMS", 3)  )
					|| (iDevice==5 && SumOffset>0.077 && !strncmp(RMS, "PEAK", 4)  ) 	
					|| (iDevice==6 && SumOffset>0.011))						
					 return VI_ERROR_DATA_OUT_OF_RANGE;      

				}			

		 }		
		
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_CURRENT_OFFSET_LAC, 0, LCCurrentOffsetLAC), 3, "LC Current Offset LAC");  		
	 }
Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCCurrentOffsetLDC                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCCurrentOffsetLDC (ViSession vi,
                                                 ViInt32 step,
                                                 ViReal64 LCCurrentOffsetLDC)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4],cScanner[20],RMS[10];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2,iDevice;
	ViReal64    LDCLimitHigh,SumOffset;

	checkErr(C19032P_GetError_IfTrigStart (vi)); 
	
	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));

	if (strncmp (cScanner, "6000-08", 7)!=0)  //for 6000-08 only 
		return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;		
	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

		
//	checkErr   ( C19032P_CheckFloat( vi, LCCurrentOffsetLDC, index, &CoercedVal));
//	LCCurrentOffsetLDC =( ViReal64) CoercedVal/pow(10, index);
		
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
	  	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_DEVICE_TYPE, 0, &iDevice));     
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_DC_HIGH_LIMIT, 0, &LDCLimitHigh)); 
	
	
		SumOffset=LDCLimitHigh+LCCurrentOffsetLDC; 
//		checkErr   ( C19032P_CheckFloat( vi, SumOffset, index, &CoercedVal));
//		SumOffset =( ViReal64) CoercedVal/pow(10, index);
		
		if(SumOffset>0.0011)
				return VI_ERROR_DATA_OUT_OF_RANGE;
				
		else
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_CURRENT_OFFSET_LDC, 0, LCCurrentOffsetLDC), 3, "LC Current Offset LDC");  		
		
		
		
	}	
		
		

Error:
	return error;		
}

/*******************************************************************************
 * Function: C19032P_SetLCModeParameter                                                  
 * Purpose:  This function sets parameters in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCModeParameter (ViSession vi, ViInt32 device, ViInt32 lineInput,
                                             ViInt32 GRSwitch, ViInt32 meter, ViReal64 highLimit,
                                             ViReal64 lowLimit, ViInt32 powerMode, ViReal64 powerHighLimit,
                                             ViReal64 powerLowLimit, ViReal64 testTime, ViInt32 step)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4], cScanner[10];
	ViInt32		nStepCount;
	
	checkErr  ( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
	if (strncmp(cScanner, "6000-04", 7) && strncmp(cScanner, "6000-05", 7) && strncmp(cScanner, "6000-06", 7) && strncmp(cScanner, "6000-07", 7) && strncmp (cScanner, "6000-08", 7))
		return VI_ERROR_NOT_SUPPORT_LC;
				
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{	
		Fmt(ChanId, "%s<%d", step);
		if (step==nStepCount+1)
		 	checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1));											
		if (highLimit < lowLimit)
			return VI_ERROR_INVALID_LIMIT_VALUE;

		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_LC));
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_DEVICE_TYPE, 0, device), 2, "LC device"); 
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_LINE_INPUT, 0, lineInput), 3, "LC line input"); 
		if (!strncmp(cScanner, "6000-05", 7) || !strncmp(cScanner, "6000-07", 7))
		{
			viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_GR_SWITCH_STATE, 0, GRSwitch), 4, "LC GR switch");
			viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_METER_METHOD, 0, meter), 5, "LC meter");
		}
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_LIMIT_HIGH, 0, highLimit), 6, "LC high limit");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_LIMIT_LOW, 0, lowLimit), 7, "LC low limit");
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, powerMode), 8, "LC power mode");
		if (powerMode != C19032P_VAL_LC_POWER_SIMULATE)
		{
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_LIMIT_HIGH, 0, powerHighLimit), 9, "LC power high limit");
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_LIMIT_LOW, 0, powerLowLimit), 10, "LC power low limit");
		}
		else
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT, 0, powerHighLimit), 9, "LC target voltage");
                                    		
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_TEST_TIME, 0, testTime), 11, "LC test time");
	}	
	else
		return VI_ERROR_STEP_COUNT;

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetOSCCurrentOffset                                                  
*******************************************************************************/
ViStatus _VI_FUNC C19032P_SetOSCCurrentOffset (ViSession vi,
                                               ViInt32 step, ViInt32 rangeNumber,
                                               ViReal64 OSCCurrentOffset)
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3],cBuf[50];
    ViSession	io = Ivi_IOSession(vi);
    ViInt32 retCnt,iMode;
    ViInt32		nStepCount, CoercedVal, index=1; 
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
	checkErr(C19032P_GetError_IfTrigStart (vi));   	    
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	
//		checkErr   ( C19032P_CheckFloat( vi, OSCCurrentOffset, index, &CoercedVal));
//		OSCCurrentOffset =  ( ViReal64) CoercedVal/pow(10, index);	
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_OSC)
	   	return VI_ERROR_INVALID_MODE_COMMAND; 
	else if (rangeNumber<1 || rangeNumber>3 )	   	
	   	return VI_ERROR_STEP_COUNT; 	   	
	else if (OSCCurrentOffset<0 || OSCCurrentOffset>=0.000001)	   	
	   	return VI_ERROR_DATA_OUT_OF_RANGE; 
	   	
	else
	{
	   	
//	Fmt( cBuf, "%s<SAFE:STEP%d:OSC:CURR %d:OFFS %.10f\n",step,rangeNumber,OSCCurrentOffset);
 	sprintf ( cBuf, "SAFE:STEP%d:OSC:CURR %d:OFFS %.10f\n",step,rangeNumber,OSCCurrentOffset);  
	viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_OSC_CURRENT_OFFSET, 0, cBuf)
		, 4, "OSCCurrentOffset");   
	}	   	

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}
/*******************************************************************************
 * Function: C19032P_SetOSCCStandard                                                  
*******************************************************************************/
ViStatus _VI_FUNC C19032P_SetOSCCStandard (ViSession vi,
                                           ViInt32 step, ViInt32 rangeNumber,
                                           ViReal64 OSCCStandard)
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3];
    ViSession	io = Ivi_IOSession(vi);
    ViInt32 retCnt,iMode;
    ViChar		cBuf[BUFFER_SIZE];
 	ViInt32		nStepCount, CoercedVal, index=1; 
 	
    checkErr( Ivi_LockSession (vi, VI_NULL));
	checkErr(C19032P_GetError_IfTrigStart (vi));   	    
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	
//		checkErr   ( C19032P_CheckFloat( vi, OSCCStandard, index, &CoercedVal));
//		OSCCStandard =  ( ViReal64) CoercedVal/pow(10, index);	
	
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_OSC)
	   	return VI_ERROR_INVALID_MODE_COMMAND; 
	   	
	else if (rangeNumber<1 || rangeNumber>3 )	   	
	   	return VI_ERROR_STEP_COUNT; 
	   	
	else if (
	(rangeNumber==1 && (OSCCStandard<0.000000000001 || OSCCStandard>0.000000009999))
			|| (rangeNumber==2 && (OSCCStandard<0.00000000001 || OSCCStandard>0.00000009999))
			|| (rangeNumber==3 && (OSCCStandard<0.0000000001 || OSCCStandard>0.0000005))
			)
	   	return VI_ERROR_DATA_OUT_OF_RANGE; 
	   	
																							
	else
	{

  		sprintf ( cBuf, "SAFE:STEP%d:OSC:CST %d,%.12f\n",step,rangeNumber,OSCCStandard);    
		viCheckParm( Ivi_SetAttributeViString( vi, VI_NULL, C19032P_ATTR_OSC_CSTANDARD, 0, cBuf)
		, 4, "OSCCStandard");		
 
    
  	}
Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*******************************************************************************
 * Function: C19032P_GetOSCOpenCheck                                                  
*******************************************************************************/
ViStatus _VI_FUNC C19032P_GetOSCOpenCheck (ViSession vi,
                                           ViInt32 step, ViReal64 *openCheck)
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3];
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    if (openCheck == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for openCheck");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_OSC));  	
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_OSC_OPEN_CHECK, 0, openCheck));  

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}
/*******************************************************************************
 * Function: C19032P_GetOSCShortCheck                                                 
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetOSCShortCheck (ViSession vi,
                                            ViInt32 step,
                                            ViReal64 *OSCShortCheck)
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3];
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    if (OSCShortCheck == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for OSCShortCheck");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_OSC));  	
	
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_OSC_SHORT_CHECK, 0, OSCShortCheck));  

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}
/*******************************************************************************
 * Function: C19032P_GetOSCChanState                                                 
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetOSCChanState (ViSession vi,
                                           ViInt32 step, ViInt32 scannerNumber,
                                           ViChar channelHighState[],
                                           ViChar channelLowState[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3],cBuf[100];
 	ViSession	io = Ivi_IOSession(vi);   
 	ViUInt32      retCnt;
 	
    checkErr( Ivi_LockSession (vi, VI_NULL));
    if (channelHighState == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for channelHighState");
    if (channelLowState == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for channelLowState");

        
    	checkErr( CheckStepRange(vi,step));        
		Fmt     ( ChanId, "%s<%d", step); 
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_OSC));    
	if (!Ivi_Simulating(vi))
    {
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:OSC:CHAN:HIGH?\n",ChanId);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, channelHighState, BUFFER_SIZE-1, &retCnt));
	//	TempValue[retCnt] = 0;
	
  		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:OSC:CHAN:LOW?\n",ChanId);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  ( io, channelLowState, BUFFER_SIZE-1, &retCnt));
	//	TempValue[retCnt] = 0;
	}	        

 
Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*******************************************************************************
 * Function: C19032P_GetOSCCRange                                                 
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetOSCCRange (ViSession vi, ViInt32 step,
                                        ViInt32 rangeSelect, ViReal64 *OSCCRange)
{
    ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3];
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    if (OSCCRange == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for OSCCRange");
     
        
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_OSC));  	
	viCheckParm( Ivi_SetAttributeViInt32( vi, VI_NULL, C19032P_ATTR_OSC_RANGE_SELECT, 0, rangeSelect)
	, 3, "rangeSelect");	
																			  
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_OSC_CRANGE, 0, OSCCRange));  

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCDevice                                                  
 * Purpose:  This function sets device in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCDevice (ViSession vi, ViInt32 step, ViInt32 device)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4], cScanner[10];
	ViInt32		nStepCount;
	
    checkErr( Ivi_LockSession (vi, VI_NULL));   	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr  ( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
	if (strncmp(cScanner, "6000-04", 7)!=0 && strncmp(cScanner, "6000-05", 7)!=0 && strncmp(cScanner, "6000-06", 7)!=0 
	&& strncmp(cScanner, "6000-07", 7)!=0 && strncmp (cScanner, "6000-08", 7)!=0)
		return VI_ERROR_NOT_SUPPORT_LC;
		
	if (device>4 && strncmp (cScanner, "6000-08", 7)!=0)  //19050-U1 & 2601-U1 for 6000-08 only 
		return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;	
		
				
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{	
		Fmt(ChanId, "%s<%d", step);
		if (step==nStepCount+1)
		 	viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1), 2, "Step count");

		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_LC));
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_DEVICE_TYPE, 0, device), 3, "LC device"); 
	}	
	else
		return VI_ERROR_STEP_COUNT;

Error:
    Ivi_UnlockSession(vi, VI_NULL);    
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCLineInput                                                  
 * Purpose:  This function sets line input in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCLineInput (ViSession vi, ViInt32 step, ViInt32 lineInput)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType;

	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_LINE_INPUT, 0, lineInput), 3, "LC line input"); 

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCGrSwitch                                                  
 * Purpose:  This function sets GR switch in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCGrSwitch (ViSession vi, ViInt32 step, ViInt32 GRSwitch)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4], cScanner[10];
	ViInt32		iStepType;

	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		checkErr( Ivi_GetAttributeViString( vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));  	
	
		if (!strncmp(cScanner, "6000-05", 7) || !strncmp(cScanner, "6000-07", 7) || !strncmp(cScanner, "6000-08", 7))
			viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_GR_SWITCH_STATE, 0, GRSwitch), 4, "LC GR switch");
		else
			return VI_ERROR_NOT_SUPPORT_SCANNER;
	} 

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCMeter                                                  
 * Purpose:  This function sets meter in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCMeter (ViSession vi, ViInt32 step,
                                      ViInt32 LCMeter)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4], cScanner[10];
	ViInt32		iStepType;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		checkErr( Ivi_GetAttributeViString( vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));    	
		if (!strncmp(cScanner, "6000-05", 7) || !strncmp(cScanner, "6000-07", 7) || !strncmp(cScanner, "6000-08", 7))
			viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_METER_METHOD, 0, LCMeter), 3, "LC meter");
		else
			return VI_ERROR_NOT_SUPPORT_SCANNER;
	} 

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCHighLimit                                                 
 * Purpose:  This function sets high limit in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCHighLimit (ViSession vi, ViInt32 step, ViReal64 highLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=5;
	ViReal64	fLowLimit;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
	
	
//		checkErr   ( C19032P_CheckFloat( vi, highLimit, index, &CoercedVal));
//		highLimit =( ViReal64) CoercedVal/pow(10, index);


		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_LIMIT_LOW, 0, &fLowLimit));    
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_LIMIT_HIGH, 0, highLimit), 2, "LC high limit"); 
		if (highLimit<fLowLimit)
			checkErr( Ivi_SetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_LIMIT_LOW, 0, highLimit));
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCLowLimit                                                  
 * Purpose:  This function sets low limit in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCLowLimit (ViSession vi, ViInt32 step, ViReal64 lowLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=5;
	ViReal64	fHighLimit; 
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_LIMIT_HIGH, 0, &fHighLimit));   	
	
	    if(lowLimit<=0)
	    {
			ViSession	io = Ivi_IOSession(vi);    	
			nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%d:LC:LIM:LOW 0\n", step);
			viCheckErr( viWrite (io, cBuf, nCount, &retCnt));	
	//		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_LIMIT_LOW, IVI_VAL_SET_CACHE_ONLY, lowLimit), 3, "LC low limit");    	
	
		}
		else
		{
	
	//			checkErr   ( C19032P_CheckFloat( vi, lowLimit, index, &CoercedVal));
	//			lowLimit = ( ViReal64) CoercedVal/pow(10, index);
				if (fHighLimit<lowLimit)
					return VI_ERROR_INVALID_LIMIT_VALUE;
				else
					viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_LIMIT_LOW, 0, lowLimit), 3, "LC low limit"); 
			
		}		
	}

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCDisplay                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCDisplay (ViSession vi, ViInt32 step,
                                        ViBoolean setLCDisplay)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4], cScanner[10];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=5;
	ViInt32		nModel;
	ViString    commandString;

	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	
	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));  	
	if (strncmp (cScanner, "6000-08", 7)!=0)  //for 6000-08 only 
		return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;	
		
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
	viCheckErr( Ivi_GetViInt32EntryFromValue(setLCDisplay, &attrLCDisplayRangeTable, 
									VI_NULL,VI_NULL,VI_NULL, VI_NULL,&commandString, VI_NULL));
									
	viCheckParm( Ivi_SetAttributeViString( vi,ChanId, C19032P_ATTR_LC_DISPLAY,
											 IVI_VAL_DIRECT_USER_CALL,commandString), 3, "set LC Display" );
	}

Error:
	return error;
}


/*******************************************************************************
 * Function: C19032P_SetLCACHighLimit                                                  
 * Purpose:  This function sets AC high limit in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCACHighLimit (ViSession vi, ViInt32 step, ViReal64 ACHighLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4], cScanner[10];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=5;
	ViInt32		nModel;
	ViReal64	LCLimitHigh;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
//	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
//	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	

	
	
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
//		return VI_ERROR_NOT_SUPPORT_VERSION;

	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step);
	
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_LIMIT_HIGH, 0, &LCLimitHigh)); 	
	if  (ACHighLimit> LCLimitHigh) 
		 return VI_ERROR_INVALID_LIMIT_VALUE; 	
	
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
	
	
//		checkErr     ( C19032P_CheckFloat( vi, ACHighLimit, index, &CoercedVal));
//		ACHighLimit =( ViReal64) CoercedVal/pow(10, index);

 
		checkErr( Ivi_GetAttributeViString( vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));    
		if (!strncmp(cScanner, "6000-08", 7))  //ck modify lc:lac only for 6000-08 2010/8/10       
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_AC_HIGH_LIMIT, 0, ACHighLimit), 3, "LC AC high limit"); 
		else
			return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;
	
	}
		
Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCACHighLimit                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCACHighLimit (ViSession vi,
                                            ViInt32 step, ViReal64 *ACHighLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3],cScanner[20];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (ACHighLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for ACHighLimit");
        	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	
	
	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner)); 	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		if (!strncmp(cScanner, "6000-08", 7))  //ck modify lc:lac only for 6000-08 2010/8/10       
		 		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_AC_HIGH_LIMIT, 0, ACHighLimit));      
		else
			return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;
	
	
	}
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCDCHighLimit                                                  
 * Purpose:  This function sets DC high limit in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCDCHighLimit (ViSession vi, ViInt32 step, ViReal64 DCHighLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4], cScanner[10];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=5;
	ViInt32		nModel;
	ViReal64 	LCLimitHigh;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
//		return VI_ERROR_NOT_SUPPORT_VERSION;

	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	
	checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_LIMIT_HIGH, 0, &LCLimitHigh));   	

	if((LCLimitHigh>0.001 && DCHighLimit>0.001) || (DCHighLimit>0 && DCHighLimit<0.000001) )
		return VI_ERROR_INVALID_LIMIT_VALUE;  	
	
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr     ( C19032P_CheckFloat( vi, DCHighLimit, index, &CoercedVal));
//		DCHighLimit =( ViReal64) CoercedVal/pow(10, index);


		checkErr( Ivi_GetAttributeViString( vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));  
		if (!strncmp(cScanner, "6000-08", 7)) //ck modify lc:lac only for 6000-08 2010/8/10   
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_DC_HIGH_LIMIT, 0, DCHighLimit), 3, "LC dc high limit"); 
		else
			return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;
	}

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCTimeDwell                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCTimeDwell (ViSession vi,
                                          ViInt32 step, ViReal64 LCTimeDwell)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;

	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, LCTimeDwell, index, &CoercedVal));
//		LCTimeDwell = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_TIME_DWELL, 0, LCTimeDwell), 3, "LC test time");
	}
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCPowerMode                                                  
 * Purpose:  This function sets power mode in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCPowerMode (ViSession vi, ViInt32 step, ViInt32 powerMode)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iUseSource;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
	
		checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));   	
		if ((!iUseSource && powerMode==C19032P_VAL_LC_POWER_SOURCE  && powerMode>2) || (iUseSource && powerMode==C19032P_VAL_LC_POWER_SIMULATE && powerMode>2))
			return VI_ERROR_NOT_MATCH_POWMODE;
		else
			viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, powerMode), 3, "LC power mode");   
	}

Error:
	return error;
}





/*******************************************************************************
 * Function: C19032P_SetLCPowerLimitOfStep                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCPowerLimitOfStep (ViSession vi,
                                                 ViInt32 step,
                                                 ViInt32 setLCPowerMode,
                                                 ViReal64 powerHighLimit,
                                                 ViReal64 powerLowLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iPowerMode, CoercedVal, index=2;
	ViReal64	fLowLimit;

	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	
	checkErr( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, setLCPowerMode));
	checkErr( Ivi_GetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_LIMIT_LOW, 0, &fLowLimit));
	
	if (setLCPowerMode>=C19032P_VAL_LC_POWER_SIMULATE)
		return VI_ERROR_NOT_MATCH_POWMODE;	
			

		
	if (powerHighLimit<powerLowLimit && powerHighLimit!=0)
			return VI_ERROR_INVALID_LIMIT_VALUE;		
		
		
		
	else
	{
	
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_LIMIT_HIGH, 0, powerHighLimit), 4, "LC power high limit");    
		
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_LIMIT_LOW, 0, powerLowLimit),5, "LC power low limit");
	}
	
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCPowerSimuVolt                                                  
 * Purpose:  This function sets power simulate target voltage in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCPowerSimuVolt (ViSession vi, ViInt32 step, ViReal64 targetVolt)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2;
	
 	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
	//	checkErr    ( C19032P_CheckFloat( vi, targetVolt, index, &CoercedVal));
	//	targetVolt =( ViReal64) CoercedVal/pow(10, index);
	
		checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
		checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));	
	
	
		if (iPowerMode==C19032P_VAL_LC_POWER_SIMULATE && iUseSource==0)
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT, 0, targetVolt), 3, "LC simulate target voltage");
		else
			return VI_ERROR_NOT_MATCH_POWMODE;
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCPowerSimuVolt                                                  
 * Purpose:  This function sets power source target voltage and frequency in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCPowerSourceVoltFreq (ViSession vi, ViInt32 step,
                                                   ViReal64 targetVolt, ViReal64 targetFreq)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2;

	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr    ( C19032P_CheckFloat( vi, targetVolt, index, &CoercedVal));
//		targetVolt =( ViReal64) CoercedVal/pow(10, index);
//		checkErr    ( C19032P_CheckFloat( vi, targetFreq, index, &CoercedVal));
//		targetFreq =( ViReal64) CoercedVal/pow(10, index);

		checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
		checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));

		if (iPowerMode==C19032P_VAL_LC_POWER_SOURCE && iUseSource==1)
		{
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_TARGET_VOLT, 0, targetVolt), 3, "LC source target voltage");
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_TARGET_FREQ, 0, targetFreq), 4, "LC source target frequency");
		}
		else
			return VI_ERROR_NOT_MATCH_POWMODE;
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetLCTestTime                                                  
 * Purpose:  This function sets test time in LC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCTestTime (ViSession vi, ViInt32 step, ViReal64 testTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=2;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, testTime, index, &CoercedVal));
//		testTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_TEST_TIME, 0, testTime), 3, "LC test time");
	}
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCDeviceOfStep                                                  
 * Purpose: This function gets the LC device type of specify step of the LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCDeviceOfStep (ViSession vi, ViInt32 step, ViChar setLCDevice[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iDevice;
	ViString	cDevice;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (setLCDevice == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for setLCDevice");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_DEVICE_TYPE, 0, &iDevice));   
		checkErr( Ivi_GetViInt32EntryFromValue (iDevice, &attrLcDeviceTypeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cDevice, VI_NULL));
		strcpy  ( setLCDevice, cDevice);
	}
					
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCDisplay                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCDisplay (ViSession vi, ViInt32 step,
                                        ViChar getLCDisplay[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iDevice;
	ViString	cDevice;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (getLCDisplay == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for getLCDisplay");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
	checkErr( Ivi_GetAttributeViString( vi,ChanId, C19032P_ATTR_LC_DISPLAY,
											 IVI_VAL_DIRECT_USER_CALL,-1,getLCDisplay) );
	}
					
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCDCHighLimit                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCDCHighLimit (ViSession vi,
                                            ViInt32 step, ViReal64 *DCHighLimit)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3],cScanner[20];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (DCHighLimit == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for DCHighLimit");
        	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	
	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner)); 		
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		if (!strncmp(cScanner, "6000-08", 7))  //ck modify lc:lac only for 6000-08 2010/8/10       
		 		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_DC_HIGH_LIMIT, 0, DCHighLimit));   
		else
			return VI_ERROR_SUPPORT_ONLY_6000_08_SCANNER;	
	
	}
	
	
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCLineInTypeofStep                                                  
 * Purpose: This function gets the line type of specify step of LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCLineInTypeofStep (ViSession vi,
                                                ViInt32 step, ViChar setLCLine[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iLine;
	ViString	cLine;

	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (setLCLine == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for setLCLine");
        					
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_LINE_INPUT, 0, &iLine));   
		checkErr( Ivi_GetViInt32EntryFromValue (iLine, &attrLcLineInputRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cLine, VI_NULL));
		strcpy  ( setLCLine, cLine);
	}
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCMeterMethod                                                  
 * Purpose: This function gets the meter method of specify step of LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCMeterMethod (ViSession vi, ViInt32 step, ViChar meterMethod[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3],cScanner[20];
	ViInt32		iMode, iMeter;
	ViString	cMeter;

	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (meterMethod == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for meterMethod");
        						
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner)); 	
		if (!strncmp(cScanner, "6000-05", 7) || !strncmp(cScanner, "6000-07", 7) || !strncmp(cScanner, "6000-08", 7)) 	
		{
			checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_METER_METHOD, 0, &iMeter)); 
			checkErr( Ivi_GetViInt32EntryFromValue (iMeter, &attrLcMeterMethodRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cMeter, VI_NULL));
			strcpy  ( meterMethod, cMeter);
		}
		else
		   	return VI_ERROR_NOT_SUPPORT_SCANNER;    
		
		
	}
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCgrSwitch                                                  
 * Purpose: This function gets the ground switch state of specify step of LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCgrSwitch (ViSession vi, ViInt32 step,
                                        ViInt32 *LC_gr_Switch)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3],cScanner[20];
	ViInt32		iMode;

	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LC_gr_Switch == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LC_gr_Switch");
        							
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner)); 	
		if (!strncmp(cScanner, "6000-05", 7) || !strncmp(cScanner, "6000-07", 7) || !strncmp(cScanner, "6000-08", 7)) 	
			checkErr( Ivi_GetAttributeViInt32(vi, ChanId, C19032P_ATTR_LC_GR_SWITCH_STATE, 0, LC_gr_Switch));   
		else
		  	return VI_ERROR_NOT_SUPPORT_SCANNER; 
		
		
		
		
		
	}	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCLimitHighOfStep                                                  
 * Purpose: This function gets the limit of high of specify step of the LC mode.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCLimitHighOfStep (ViSession vi, ViInt32 step,
                                               ViReal64 *LCLimitHigh)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCLimitHigh == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCLimitHigh");
        	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_LIMIT_HIGH, 0, LCLimitHigh));   
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCLimitLowOfStep                                                  
 * Purpose: This function gets the limit of low of specify step of the LC mode.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCLimitLowOfStep (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 *LC_limit_low)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LC_limit_low == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LC_limit_low");
        		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_LIMIT_LOW, 0, LC_limit_low));   
											
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCTestTimeOfStep                                                  
 * Purpose: This function gets the test time of specify step of LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCTestTimeOfStep (ViSession vi, ViInt32 step, ViReal64 *LCTestTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCTestTime == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCTestTime");
        			
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_TEST_TIME, 0, LCTestTime));   
											
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCTimeDwell                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCTimeDwell (ViSession vi,
                                          ViInt32 step, ViReal64 *LCTimeDwell)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCTimeDwell == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCTimeDwell");
        			
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_TIME_DWELL, 0, LCTimeDwell));   
											
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCPowerModeOfStep                                                  
 * Purpose: This function gets the power mode of specify step of LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCPowerModeOfStep (ViSession vi, ViInt32 step,
                                               ViChar setLCPowerMode[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	ViString	cPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (setLCPowerMode == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for setLCPowerMode");
        				
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode)); 
		checkErr( Ivi_GetViInt32EntryFromValue (iPowerMode, &attrLcPowerModeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cPowerMode, VI_NULL));
		strcpy  ( setLCPowerMode, cPowerMode);
	}
											
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}	   

/*******************************************************************************
 * Function: C19032P_GetLCPowerHiLimitOfStep                                                  
 * Purpose: This function gets the power limit of high and low of the LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCPowerLimitOfStep (ViSession vi, ViInt32 step,
                                                ViInt32 setLCPowerMode,
                                                ViReal64 *LCLimitHigh_ofPower,
                                                ViReal64 *LCLimitLow_ofPower)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	ViString	cPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCLimitHigh_ofPower == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 4, "Null address for LCLimitHigh_ofPower");
	
    if (LCLimitLow_ofPower == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER,5, "Null address for LCLimitLow_ofPower");
        								        									
	
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_LC));   
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));	
		if (iMode!=C19032P_VAL_MODE_LC)
	   	return VI_ERROR_INVALID_MODE_COMMAND;
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
	
	if (setLCPowerMode>=C19032P_VAL_LC_POWER_SIMULATE)
		return VI_ERROR_NOT_MATCH_POWMODE;		
		
	if ((iMode==C19032P_VAL_MODE_LC) && (iPowerMode==setLCPowerMode))
	{
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_LIMIT_HIGH, 0, LCLimitHigh_ofPower));  											
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_LIMIT_LOW, 0, LCLimitLow_ofPower));  											
	}
	else
		return VI_ERROR_INVALID_MODE_COMMAND;	
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLcSimulateVolt                                                  
 * Purpose: This function gets the target voltage of simulate of the LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLcSimulateVolt (ViSession vi, ViInt32 step,
                                            ViReal64 *targetVoltage)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (targetVoltage == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for targetVoltage");
		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));											
		if (iPowerMode==C19032P_VAL_LC_POWER_SIMULATE) 
			checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT, 0, targetVoltage));  											
		else
			return VI_ERROR_INVALID_MODE_COMMAND;
	}		
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCPowSouCurrHi_Lo                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCPowSouCurrHi_Lo (ViSession vi,
                                                ViInt32 step,
                                                ViReal64 *LCPowSouCurrentHigh,
                                                ViReal64 *LCPowSouCurrentLow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCPowSouCurrentHigh == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCPowSouCurrentHigh");
    if (LCPowSouCurrentLow == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCPowSouCurrentLow");
        
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_CURRENT_HIGH, 0, LCPowSouCurrentHigh));											
 		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_CURRENT_LOW, 0, LCPowSouCurrentLow));
	}		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCPowSouVAHi_Lo                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCPowSouVAHi_Lo (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 *LCPowSouVAHigh,
                                              ViReal64 *LCPowSouVALow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCPowSouVAHigh == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCPowSouVAHigh");
    if (LCPowSouVALow == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCPowSouVALow");
        
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_VA_HIGH, 0, LCPowSouVAHigh));											
 		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_VA_LOW, 0, LCPowSouVALow));
	}		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCPowSouPowerHi_Lo                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCPowSouPowerHi_Lo (ViSession vi,
                                                 ViInt32 step,
                                                 ViReal64 *LCPowSouPowerHigh,
                                                 ViReal64 *LCPowSouPowerLow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCPowSouPowerHigh == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCPowSouPowerHigh");
    if (LCPowSouPowerLow == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCPowSouPowerLow");
        
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_POWER_HIGH, 0, LCPowSouPowerHigh));											
 		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_POWER_LOW, 0, LCPowSouPowerLow));
	}		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_GetLCPowerSouPFACLow                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCPowerSouPFACLow (ViSession vi,
                                                ViInt32 step,
                                                ViReal64 *LCPowerSouPFACLow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCPowerSouPFACLow == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCPowerSouPFACLow");

	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViReal64 (vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_PFACTOR, 0, LCPowerSouPFACLow));											
 
	}		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCUPM                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCUPM (ViSession vi, ViInt32 step,
                                    ViBoolean *LCUPM)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3];
	ViInt32		iMode, iPowerMode;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
    if (LCUPM == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCUPM");

	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{	
		checkErr( Ivi_GetAttributeViBoolean (vi, ChanId, C19032P_ATTR_LC_UPM, 0, LCUPM));											
 
	}		

Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetLCChanState                                                  
 * Purpose: This function gets the channel status of LC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetLCChanState (ViSession vi, ViInt32 step, ViChar LCChannelStatus[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[3], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		iMode, retCnt;
	ViSession	io = Ivi_IOSession (vi);
	
	checkErr( C19032P_LockSession( vi, VI_NULL )); 
	
    if (LCChannelStatus == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for LCChannelStatus");
        								
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));		
	if (iMode!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:LC:CHAN:LOW?\n", step);
		viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		strncpy   ( LCChannelStatus, cResult, retCnt);
	}
		
Error:
	C19032P_UnlockSession( vi, VI_NULL );        
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPAModeParameter                                                  
 * Purpose:  This function sets parameters in PA mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPAModeParameter (ViSession vi, ViChar message[], ViInt32 underTest, ViInt32 step)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount;
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
				
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{
		Fmt(ChanId, "%s<%d", step); 
		if (step==nStepCount+1)
		 	checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1));											
		
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_PA));
		viCheckParm( Ivi_SetAttributeViString( vi, ChanId, C19032P_ATTR_PAUSE_MESSAGE_STRING, 0, message), 2, "PAUSE string");
		viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_PAUSE_SIGNAL_STATE, 0, underTest), 3, "PAUSE under test ON/OFF");
	}												
	else
		return VI_ERROR_STEP_COUNT;

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPAMessage                                                 
 * Purpose:  This function sets message in PA mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPAMessage (ViSession vi, ViInt32 step, ViChar message[])
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
				
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{
		Fmt(ChanId, "%s<%d", step); 
		if (step==nStepCount+1)
		 	viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1), 2, "Step count");
		
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_PA));
		viCheckParm( Ivi_SetAttributeViString( vi, ChanId, C19032P_ATTR_PAUSE_MESSAGE_STRING, 0, message), 3, "PAUSE string");
	}												
	else
		return VI_ERROR_STEP_COUNT;
		
 	checkErr(C19032P_Check_Parameter_Error(vi)); 
Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPAUnderTest                                                 
 * Purpose:  This function sets under test on/off in PA mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPAUnderTest (ViSession vi,
                                          ViInt32 step,
                                          ViBoolean setSignal_ofPause)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_PA)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
		viCheckParm( Ivi_SetAttributeViBoolean ( vi, ChanId, C19032P_ATTR_PAUSE_SIGNAL_STATE, 0, setSignal_ofPause), 3, "PAUSE under test ON/OFF");

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetPATestTime                                                  
 * Purpose:  This function sets test time in PA mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetPATestTime (ViSession vi, ViInt32 step, ViReal64 testTime)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=2;
	ViInt32		nModel;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
//		return VI_ERROR_NOT_SUPPORT_VERSION;
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_PA)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, testTime, index, &CoercedVal));
//		testTime = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_PAUSE_TEST_TIME, 0, testTime), 3, "PAUSE test time");
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_GetPauseString                                                  
 * Purpose: This function gets the signal state of PAUSE MODE.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_GetPauseString (ViSession vi, ViInt32 step, ViChar getPauseString[])
{
	ViStatus    error = VI_SUCCESS;
    ViChar		ChanId[3];
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    if (getPauseString == VI_NULL)
        viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for getPauseString");
        
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr(ModeCheck(vi,step,C19032P_VAL_MODE_PA));    	
	checkErr( Ivi_GetAttributeViString(vi, ChanId, C19032P_ATTR_PAUSE_MESSAGE_STRING, 0, AUTO_BUFFER_SIZE, getPauseString));  

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}



/*******************************************************************************
 * Function: C19032P_SetOSCModeParameter                                                  
 * Purpose:  This function sets parameters in OSC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetOSCModeParameter (ViSession vi, ViReal64 openCheck, ViReal64 shortCheck, ViInt32 step)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount;
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
				
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{
		Fmt(ChanId, "%s<%d", step); 
		if (step==nStepCount+1)
		 	checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1));											
		
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_OSC));
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_OSC_OPEN_CHECK, 0, openCheck), 2, "OSC open check");
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_OSC_SHORT_CHECK, 0, shortCheck), 3, "OSC short check");
	}												
	else
		return VI_ERROR_STEP_COUNT;

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetOSCOpenCheck                                                  
 * Purpose:  This function sets open check in OSC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetOSCOpenCheck (ViSession vi, ViInt32 step, ViReal64 openCheck)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nStepCount, CoercedVal, index=4;
	
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, &nStepCount));
				
	if (step>=1 && step<=100 && step<=nStepCount+1)
	{
		Fmt(ChanId, "%s<%d", step); 
		if (step==nStepCount+1)
		 	viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_STEP_COUNT, 0, nStepCount+1), 2, "Step count");
		
		checkErr   ( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, C19032P_VAL_MODE_OSC));
//		checkErr   ( C19032P_CheckFloat( vi, openCheck, index, &CoercedVal));
//		openCheck =( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_OSC_OPEN_CHECK, 0, openCheck), 3, "OSC open check");
	}												
	else
		return VI_ERROR_STEP_COUNT;

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetOSCShortCheck
 * Purpose:  This function sets short check in OSC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetOSCShortCheck (ViSession vi, ViInt32 step, ViReal64 shortCheck)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, CoercedVal, index=4;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_OSC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr    ( C19032P_CheckFloat( vi, shortCheck, index, &CoercedVal));
//		shortCheck =( ViReal64) CoercedVal/pow(10, index);
		viCheckParm ( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_OSC_SHORT_CHECK, 0, shortCheck), 3, "OSC short check");
	}
Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetOSCShortCheck
 * Purpose:  This function sets short check in OSC mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBChanScanState (ViSession vi,
                                              ViInt32 step,
                                              ViChar GBChannelHighScanState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4],  cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL ;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);

	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
 
	if (iMode != C19032P_VAL_MODE_GB)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:GB:CHAN:HIGH %s\n", step, GBChannelHighScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));

		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetGBChanState                                                  
 * Purpose:  This function sets the channel state of GB mode.
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetGBChanState (ViSession vi, ViInt32 step, ViInt32 scanerNumber,
                                         ViInt32 ChState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4], Scanner[4], cGbState[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCount, onoff[8]={0,0,0,0,0,0,0,0}, i, iScanType, iTemp[8]={0,0,0,0,0,0,0,0};
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);
	Fmt     ( Scanner, "%s<%d", scanerNumber);
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
	checkErr( Ivi_GetAttributeViInt32 (vi, Scanner, C19032P_ATTR_OPTION_SCAN_TYPE, 0, &iScanType));
	if (scanerNumber<1 || scanerNumber>8)
		return VI_ERROR_SCAN_NUM;
	if (iScanType==0)
		return VI_ERROR_SCANNER_BOX_NULL;
	if (iMode != C19032P_VAL_MODE_GB)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:GB:CHAN:HIGH (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		switch (iScanType)
		{
			case C19032P_VAL_GB_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = iGB_FloatingChState[0][i];
				break;
			case C19032P_VAL_6000_01:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_01ChState[0][i];
				break;
			case C19032P_VAL_6000_02:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_02ChState[0][i];
				break;
			case C19032P_VAL_6000_03:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_03ChState[0][i];
				break;
			case C19032P_VAL_6000_04:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04ChState[0][i];
				break;
			case C19032P_VAL_6000_06:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06ChState[0][i];
				break;
			case C19032P_VAL_9030A: 
				for (i=0;i<8;i++)
					iTemp[i] = i9030AChState[0][i];
				break;
			case C19032P_VAL_6000_04_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04FloatingChState[0][i];
				break;
			case C19032P_VAL_6000_06_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06FloatingChState[0][i];
				break;
			case C19032P_VAL_6000_11:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_11ChState[0][i];
				break;
		}
	
		for (i=0;i<8;i++)
		{
			onoff[i] = (iTemp[i]>=0)? 1 : 0;
			if (onoff[i])
			{
				switch (iTemp[i])
				{
					case C19032P_VAL_X:
						if (ChState[i]!=0)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_LX:
						if (ChState[i]!=0 && ChState[i]!=1)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HX:
						if (ChState[i]!=0 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HLX:
						if (ChState[i]!=0 && ChState[i]!=1 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
				}	  
			}
		}
	   
		nCount = 0;
		memset( cGbState, 0, sizeof(cGbState));
		nCount += sprintf(cGbState + nCount, "(@%d(", scanerNumber);

		for (i=0;i<8;i++)
		{
			if (onoff[i] && ChState[i]==2)
				nCount += sprintf (cGbState + nCount, "%d,", i+1);
		//	sprintf ( ChanId, "%d", scanerNumber*100+i+1);
		//	checkErr( Ivi_AddToChannelTable (vi, ChanId));
		//	checkErr( Ivi_ApplyDefaultSetup (vi));
		//	checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_GB_CHANNEL_STATE, IVI_VAL_SET_CACHE_ONLY, (onoff[i])? ChState[i] : -1));
		}
		
		if (nCount==4)	
			nCount += sprintf(cGbState + (nCount-1), "(0))");  		//(nCount-1) to avoid ","
		else
			nCount += sprintf(cGbState + (nCount-1), "))");  		//(nCount-1) to avoid ","  

		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:GB:CHAN:HIGH %s\n", step, cGbState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}	
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetACChanState                                                  
 * Purpose:  This function sets the channel state of the step on specify scaner.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetACChanState (ViSession vi, ViInt32 step, ViInt32 scanerNumber,
                                         ViInt32 ChState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4], Scanner[4], cAcStateH[BUFFER_SIZE], cAcStateL[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL, onoff[8]={0,0,0,0,0,0,0,0}, i, iScanType, iTemp[8]={0,0,0,0,0,0,0,0};
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);
	Fmt     ( Scanner, "%s<%d", scanerNumber);
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
	checkErr( Ivi_GetAttributeViInt32 (vi, Scanner, C19032P_ATTR_OPTION_SCAN_TYPE, 0, &iScanType));
	if (scanerNumber<1 || scanerNumber>8)
		return VI_ERROR_SCAN_NUM;
	if (iScanType==0)
		return VI_ERROR_SCANNER_BOX_NULL;
	if (iMode != C19032P_VAL_MODE_AC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:AC:CHAN:HIGH (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:AC:CHAN:LOW (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
/*		
		
		switch (iScanType)
		{
			case C19032P_VAL_GB_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = iGB_FloatingChState[1][i];
				break;
			case C19032P_VAL_6000_01:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_01ChState[1][i];
				break;
			case C19032P_VAL_6000_02:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_02ChState[1][i];
				break;
			case C19032P_VAL_6000_03:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_03ChState[1][i];
				break;
			case C19032P_VAL_6000_04:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04ChState[1][i];
				break;
			case C19032P_VAL_6000_06:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06ChState[1][i];
				break;
			case C19032P_VAL_9030A: 
				for (i=0;i<8;i++)
					iTemp[i] = i9030AChState[1][i];
				break;
			case C19032P_VAL_6000_04_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04FloatingChState[1][i];
				break;
			case C19032P_VAL_6000_06_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06FloatingChState[1][i];
				break;
			case C19032P_VAL_6000_11:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_11ChState[1][i];
				break;
		}
	
		for (i=0;i<8;i++)
		{
			onoff[i] = (iTemp[i]>=0)? 1 : 0;
			if (onoff[i])
			{
				switch (iTemp[i])
				{
					case C19032P_VAL_X:
						if (ChState[i]!=0)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_LX:
						if (ChState[i]!=0 && ChState[i]!=1)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HX:
						if (ChState[i]!=0 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HLX:
						if (ChState[i]!=0 && ChState[i]!=1 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
				}	  
			}
		}
	
		nCountH = nCountL = 0;
		memset( cAcStateH, 0, sizeof(cAcStateH));
		memset( cAcStateL, 0, sizeof(cAcStateL));
		nCountH += sprintf(cAcStateH + nCountH, "(@%d(", scanerNumber);
		nCountL += sprintf(cAcStateL + nCountL, "(@%d(", scanerNumber);
 */
 
 
		for (i=0;i<8;i++)
		{
			if (onoff[i] && ChState[i]==1)
				nCountL += sprintf (cAcStateL + nCountL, "%d,", i+1);
			if (onoff[i] && ChState[i]==2)
				nCountH += sprintf (cAcStateH + nCountH, "%d,", i+1);
				
		//	sprintf ( ChanId, "%d", scanerNumber*100+i+1);
		//	checkErr( Ivi_AddToChannelTable (vi, ChanId));
		//	checkErr( Ivi_ApplyDefaultSetup (vi));
		//	checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_AC_CHANNEL_STATE, IVI_VAL_SET_CACHE_ONLY, (onoff[i])? ChState[i] : -1));
		//	MessagePopup("", ChanId);
		}
		
		if (nCountH==4)	
			nCountH += sprintf(cAcStateH + (nCountH-1), "(0))");  	    //(nCountH-1) to avoid ","
		else
			nCountH += sprintf(cAcStateH + (nCountH-1), "))");  		//(nCountH-1) to avoid ","  

		if (nCountL==4)	
			nCountL += sprintf(cAcStateL + (nCountL-1), "(0))");  	    //(nCountL-1) to avoid ","
		else
			nCountL += sprintf(cAcStateL + (nCountL-1), "))");  		//(nCountL-1) to avoid ","  

		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:AC:CHAN:HIGH %s\n", step, cAcStateH);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:AC:CHAN:LOW %s\n", step, cAcStateL);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}						
/*******************************************************************************
 * Function: C19032P_SetDCChanScanState                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCChanScanState (ViSession vi,
                                              ViInt32 step,
                                              ViChar DCChannelHighScanState[],
                                              ViChar DCChannelLowScanState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4],  cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL ;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);

	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
 
	if (iMode != C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DC:CHAN:HIGH %s\n", step, DCChannelHighScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DC:CHAN:LOW %s\n", step, DCChannelLowScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetDCChanState                                                  
 * Purpose:  This function sets the channel state of the step of WDC mode on specify scaner.     
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetDCChanState (ViSession vi, ViInt32 step, ViInt32 scanerNumber,
                                         ViInt32 ChState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4], Scanner[4], cDcStateH[BUFFER_SIZE], cDcStateL[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL, onoff[8]={0,0,0,0,0,0,0,0}, i, iScanType, iTemp[8]={0,0,0,0,0,0,0,0};
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%d", step);
	Fmt     ( Scanner, "%d", scanerNumber);
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
	checkErr( Ivi_GetAttributeViInt32 (vi, Scanner, C19032P_ATTR_OPTION_SCAN_TYPE, 0, &iScanType));
	if (scanerNumber<1 || scanerNumber>8)
		return VI_ERROR_SCAN_NUM;
	if (iScanType==0)
		return VI_ERROR_SCANNER_BOX_NULL;
	if (iMode != C19032P_VAL_MODE_DC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DC:CHAN:HIGH (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DC:CHAN:LOW (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		switch (iScanType)
		{
			case C19032P_VAL_GB_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = iGB_FloatingChState[2][i];
				break;
			case C19032P_VAL_6000_01:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_01ChState[2][i];
				break;
			case C19032P_VAL_6000_02:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_02ChState[2][i];
				break;
			case C19032P_VAL_6000_03:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_03ChState[2][i];
				break;
			case C19032P_VAL_6000_04:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04ChState[2][i];
				break;
			case C19032P_VAL_6000_06:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06ChState[2][i];
				break;
			case C19032P_VAL_9030A: 
				for (i=0;i<8;i++)
					iTemp[i] = i9030AChState[2][i];
				break;
			case C19032P_VAL_6000_04_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04FloatingChState[2][i];
				break;
			case C19032P_VAL_6000_06_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06FloatingChState[2][i];
				break;
			case C19032P_VAL_6000_11:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_11ChState[2][i];
				break;
		}
	
		for (i=0;i<8;i++)
		{
			onoff[i] = (iTemp[i]>=0)? 1 : 0;
			if (onoff[i])
			{
				switch (iTemp[i])
				{
					case C19032P_VAL_X:
						if (ChState[i]!=0)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_LX:
						if (ChState[i]!=0 && ChState[i]!=1)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HX:
						if (ChState[i]!=0 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HLX:
						if (ChState[i]!=0 && ChState[i]!=1 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
				}	  
			}
		}
	
		nCountH = nCountL = 0;
		memset( cDcStateH, 0, sizeof(cDcStateH));
		memset( cDcStateL, 0, sizeof(cDcStateL));
		nCountH += sprintf(cDcStateH + nCountH, "(@%d(", scanerNumber);
		nCountL += sprintf(cDcStateL + nCountL, "(@%d(", scanerNumber);

		for (i=0;i<8;i++)
		{
			if (onoff[i] && ChState[i]==1)
				nCountL += sprintf (cDcStateL + nCountL, "%d,", i+1);
			if (onoff[i] && ChState[i]==2)
				nCountH += sprintf (cDcStateH + nCountH, "%d,", i+1);
				
		//	sprintf ( ChanId, "%d", scanerNumber*100+i+1);
		//	checkErr( Ivi_AddToChannelTable (vi, ChanId));
		//	checkErr( Ivi_ApplyDefaultSetup (vi));
		//	checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_DC_CHANNEL_STATE, IVI_VAL_SET_CACHE_ONLY, (onoff[i])? ChState[i] : -1));
		}
		
		if (nCountH==4)	
			nCountH += sprintf(cDcStateH + (nCountH-1), "(0))");  	    //(nCountH-1) to avoid ","
		else
			nCountH += sprintf(cDcStateH + (nCountH-1), "))");  		//(nCountH-1) to avoid ","  

		if (nCountL==4)	
			nCountL += sprintf(cDcStateL + (nCountL-1), "(0))");  	    //(nCountL-1) to avoid ","
		else
			nCountL += sprintf(cDcStateL + (nCountL-1), "))");  		//(nCountL-1) to avoid ","  


		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DC:CHAN:HIGH %s\n", step, cDcStateH);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DC:CHAN:LOW %s\n", step, cDcStateL);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetIRCurrHighRange                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRCurrHighRange (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 IRCurrHighRange)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=2;
	ViInt32		nModel;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)		
//		return VI_ERROR_NOT_SUPPORT_VERSION;
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, IRCurrHighRange, index, &CoercedVal));
//		IRCurrHighRange = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_CURR_HIGH_RANGE, 0, IRCurrHighRange), 3, "IR Curr high Range");
	}

Error:
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRCurrLowRange                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRCurrLowRange (ViSession vi,
                                             ViInt32 step,
                                             ViReal64 IRCurrLowRange)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nFWVersion, iStepType, CoercedVal, index=2;
	ViInt32		nModel;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)		
//		return VI_ERROR_NOT_SUPPORT_VERSION;
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
//		checkErr   ( C19032P_CheckFloat( vi, IRCurrLowRange, index, &CoercedVal));
//		IRCurrLowRange = ( ViReal64) CoercedVal/pow(10, index);
		viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_IR_CURRENT_LOW_RANGE, 0, IRCurrLowRange), 3, "IR Curr Low Range");
	}

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetIRRangeAuto                                                  
 * Purpose:  This function Sets the IR channel state of the IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRRangeAuto (ViSession vi,
                                          ViInt32 step, ViInt32 rangeAuto)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		nFWVersion, iStepType;
	ViInt32		nModel;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
//	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
//		return VI_ERROR_NOT_SUPPORT_VERSION;
		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
	if (iStepType!=C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		
			viCheckParm( Ivi_SetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_IR_CURR_RANGE_AUTO, 0, rangeAuto), 2, "IR current range auto state");

	}

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetIRChanState                                                  
 * Purpose:  This function Sets the IR channel state of the IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRChanScanState (ViSession vi,
                                              ViInt32 step,
                                              ViChar IRChannelHighScanState[],
                                              ViChar IRChannelLowScanState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4],  cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL ;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);

	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
 
	if (iMode != C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:IR:CHAN:HIGH %s\n", step, IRChannelHighScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:IR:CHAN:LOW %s\n", step, IRChannelLowScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetIRChanState                                                  
 * Purpose:  This function Sets the IR channel state of the IR mode.    
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetIRChanState (ViSession vi, ViInt32 step, ViInt32 scanerNumber,
                                         ViInt32 ChState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4], Scanner[4], cIrStateH[BUFFER_SIZE], cIrStateL[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL, onoff[8]={0,0,0,0,0,0,0,0}, i, iScanType, iTemp[8]={0,0,0,0,0,0,0,0};
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);
	Fmt     ( Scanner, "%s<%d", scanerNumber);
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
	checkErr( Ivi_GetAttributeViInt32 (vi, Scanner, C19032P_ATTR_OPTION_SCAN_TYPE, 0, &iScanType));
	if (scanerNumber<1 || scanerNumber>8)
		return VI_ERROR_SCAN_NUM;
	if (iScanType==0)
		return VI_ERROR_SCANNER_BOX_NULL;
	if (iMode != C19032P_VAL_MODE_IR)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:IR:CHAN:HIGH (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:IR:CHAN:LOW (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		switch (iScanType)
		{
			case C19032P_VAL_GB_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = iGB_FloatingChState[3][i];
				break;
			case C19032P_VAL_6000_01:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_01ChState[3][i];
				break;
			case C19032P_VAL_6000_02:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_02ChState[3][i];
				break;
			case C19032P_VAL_6000_03:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_03ChState[3][i];
				break;
			case C19032P_VAL_6000_04:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04ChState[3][i];
				break;
			case C19032P_VAL_6000_06:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06ChState[3][i];
				break;
			case C19032P_VAL_9030A: 
				for (i=0;i<8;i++)
					iTemp[i] = i9030AChState[3][i];
				break;
			case C19032P_VAL_6000_04_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04FloatingChState[3][i];
				break;
			case C19032P_VAL_6000_06_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06FloatingChState[3][i];
				break;
			case C19032P_VAL_6000_11:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_11ChState[3][i];
				break;
		}
	
		for (i=0;i<8;i++)
		{
			onoff[i] = (iTemp[i]>=0)? 1 : 0;
			if (onoff[i])
			{
				switch (iTemp[i])
				{
					case C19032P_VAL_X:
						if (ChState[i]!=0)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_LX:
						if (ChState[i]!=0 && ChState[i]!=1)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HX:
						if (ChState[i]!=0 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HLX:
						if (ChState[i]!=0 && ChState[i]!=1 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
				}	  
			}
		}
	
		nCountH = nCountL = 0;
		memset( cIrStateH, 0, sizeof(cIrStateH));
		memset( cIrStateL, 0, sizeof(cIrStateL));
		nCountH += sprintf(cIrStateH + nCountH, "(@%d(", scanerNumber);
		nCountL += sprintf(cIrStateL + nCountL, "(@%d(", scanerNumber);

		for (i=0;i<8;i++)
		{
			if (onoff[i] && ChState[i]==1)
				nCountL += sprintf (cIrStateL + nCountL, "%d,", i+1);
			if (onoff[i] && ChState[i]==2)
				nCountH += sprintf (cIrStateH + nCountH, "%d,", i+1);
				
		//	sprintf ( ChanId, "%d", scanerNumber*100+i+1);
		//	checkErr( Ivi_AddToChannelTable (vi, ChanId));
		//	checkErr( Ivi_ApplyDefaultSetup (vi));
		//	checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_IR_CHANNEL_STATE, IVI_VAL_SET_CACHE_ONLY, (onoff[i])? ChState[i] : -1));
		}
		
		if (nCountH==4)	
			nCountH += sprintf(cIrStateH + (nCountH-1), "(0))");  	    //(nCountH-1) to avoid ","
		else
			nCountH += sprintf(cIrStateH + (nCountH-1), "))");  		//(nCountH-1) to avoid ","  

		if (nCountL==4)	
			nCountL += sprintf(cIrStateL + (nCountL-1), "(0))");  	    //(nCountL-1) to avoid ","
		else
			nCountL += sprintf(cIrStateL + (nCountL-1), "))");  		//(nCountL-1) to avoid ","  


		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:IR:CHAN:HIGH %s\n", step, cIrStateH);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:IR:CHAN:LOW %s\n", step, cIrStateL);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}				
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCPowSouCurrentHigh                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCPowSouCurrHi_Lo (ViSession vi,
                                                ViInt32 step,
                                                ViReal64 LCPowSouCurrentHigh,
                                                ViReal64 LCPowSouCurrentLow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2;
 	ViReal64 	fLowLimit;
 	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	 	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
  /*
		checkErr    ( C19032P_CheckFloat( vi, LCPowSouCurrentHigh, index, &CoercedVal));
		LCPowSouCurrentHigh =( ViReal64) CoercedVal/pow(10, index);
		
 		checkErr    ( C19032P_CheckFloat( vi, LCPowSouCurrentLow, index, &CoercedVal));
		LCPowSouCurrentLow =( ViReal64) CoercedVal/pow(10, index);
*/	
		checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
		checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));


		if (LCPowSouCurrentHigh<LCPowSouCurrentLow)	
		   return VI_ERROR_INVALID_LIMIT_VALUE;  
		

		if (iPowerMode==C19032P_VAL_LC_POWER_SOURCE && iUseSource==1)
		{
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_CURRENT_HIGH, 0, LCPowSouCurrentHigh), 3, "LC Power Source Current High");
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_CURRENT_LOW, 0, LCPowSouCurrentLow), 4, "LC Power Source Current Low");	
		}
		else
			return VI_ERROR_NOT_MATCH_POWMODE;
	}

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCPowSouVAHi_Lo                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCPowSouVAHi_Lo (ViSession vi,
                                              ViInt32 step,
                                              ViReal64 LCPowSouVAHigh,
                                              ViReal64 LCPowSouVALow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2;
 	ViReal64 	fLowLimit;

	checkErr(C19032P_GetError_IfTrigStart (vi));   	 	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
   /*
		checkErr    ( C19032P_CheckFloat( vi, LCPowSouVAHigh, index, &CoercedVal));
		LCPowSouVAHigh =( ViReal64) CoercedVal/pow(10, index);
		
 		checkErr    ( C19032P_CheckFloat( vi, LCPowSouVALow, index, &CoercedVal));
		LCPowSouVALow =( ViReal64) CoercedVal/pow(10, index);
*/
		checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
		checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));

		if (LCPowSouVAHigh<LCPowSouVALow)	
		   return VI_ERROR_INVALID_LIMIT_VALUE;  
		

		if (iPowerMode==C19032P_VAL_LC_POWER_SOURCE && iUseSource==1)

		{
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_VA_HIGH, 0, LCPowSouVAHigh), 3, "LC Power Source VA High");
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_VA_LOW, 0, LCPowSouVALow), 4, "LC Power Source VA Low");	
		}
		else
			return VI_ERROR_NOT_MATCH_POWMODE;
	}

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCPowSouPowerHi_Lo                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCPowSouPowerHi_Lo (ViSession vi,
                                                 ViInt32 step,
                                                 ViReal64 LCPowSouPowerHigh,
                                                 ViReal64 LCPowSouPowerLow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2;
 	ViReal64 	fLowLimit;

	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
   /*
		checkErr    ( C19032P_CheckFloat( vi, LCPowSouPowerHigh, index, &CoercedVal));
		LCPowSouPowerHigh =( ViReal64) CoercedVal/pow(10, index);
		
 		checkErr    ( C19032P_CheckFloat( vi, LCPowSouPowerLow, index, &CoercedVal));
		LCPowSouPowerLow =( ViReal64) CoercedVal/pow(10, index);
*/
		checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
		checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));

		if (LCPowSouPowerHigh<LCPowSouPowerLow)	
		   return VI_ERROR_INVALID_LIMIT_VALUE;  
		

		if (iPowerMode==C19032P_VAL_LC_POWER_SOURCE && iUseSource==1)

		{
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_POWER_HIGH, 0, LCPowSouPowerHigh), 3, "LC Power Source Power High");
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_POWER_LOW, 0, LCPowSouPowerLow), 4, "LC Power Source Power Low");	
		}
		else
			return VI_ERROR_NOT_MATCH_POWMODE;
	}

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCPowerSouPFAC                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCPowerSouPFACLow (ViSession vi,
                                                ViInt32 step,
                                                ViReal64 LCPowerSouPFACLow)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2;
 	ViReal64 	fLowLimit;

	checkErr(C19032P_GetError_IfTrigStart (vi));   	 	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));

//	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{

//		checkErr    ( C19032P_CheckFloat( vi, LCPowerSouPFACLow, index, &CoercedVal));
//		LCPowerSouPFACLow =( ViReal64) CoercedVal/pow(10, index);


	//	if (iPowerMode==C19032P_VAL_LC_POWER_SOURCE && iUseSource==1)
	
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode)); 	
   	if (iPowerMode==C19032P_VAL_LC_POWER_SOURCE) 
		{
			viCheckParm( Ivi_SetAttributeViReal64( vi, ChanId, C19032P_ATTR_LC_POWER_SOUR_PFACTOR, 0, LCPowerSouPFACLow), 3, "LC Power Source Power LCPowerSouPFAC Low");

		}
		else
			return VI_ERROR_NOT_MATCH_POWMODE;
	}

Error:
	return error;
}
/*******************************************************************************
 * Function: C19032P_SetLCUPM                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetLCUPM (ViSession vi, ViInt32 step,
                                    ViBoolean setLCUPM)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		ChanId[4];
	ViInt32		iStepType, iPowerMode, iUseSource, CoercedVal, index=2;
 	ViReal64 	fLowLimit;
 	
	checkErr(C19032P_GetError_IfTrigStart (vi));   	 	
	checkErr( CheckStepRange(vi,step));        
	Fmt     ( ChanId, "%s<%d", step); 
	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iStepType));
//	checkErr( Ivi_GetAttributeViInt32 ( vi, ChanId, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
//	checkErr( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_OPTION_USE_SOURCE, 0, &iUseSource));
	if (iStepType!=C19032P_VAL_MODE_LC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{

		viCheckParm( Ivi_SetAttributeViBoolean( vi, ChanId, C19032P_ATTR_LC_UPM, 0, setLCUPM), 3, "LC UPM");

	}

Error:
	return error;
}


 /*******************************************************************************
 * Function: C19032P_SetOSCChanScanState                                                  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetOSCChanScanState (ViSession vi,
                                               ViInt32 step,
                                               ViChar OSCChannelHighScanState[],
                                               ViChar OSCChannelLowScanState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4],  cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL ;
	
	checkErr(C19032P_GetError_IfTrigStart (vi));   		
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%s<%d", step);

	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
 
	if (iMode != C19032P_VAL_MODE_OSC)
		return VI_ERROR_INVALID_MODE_COMMAND;
	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:OSC:CHAN:HIGH %s\n", step, OSCChannelHighScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:OSC:CHAN:LOW %s\n", step, OSCChannelLowScanState);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}									
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*******************************************************************************
 * Function: C19032P_SetOSCChanState                                                  
 * Purpose: This function Sets the channel status of the OSC mode.  
 *******************************************************************************/
ViStatus _VI_FUNC C19032P_SetOSCChanState (ViSession vi, ViInt32 step, ViInt32 scanerNumber,
                                          ViInt32 ChState[])
{
	ViStatus	error = VI_SUCCESS;
	ViSession	io = Ivi_IOSession(vi); 
	ViChar		ChanId[4], Scanner[4], cOscStateH[BUFFER_SIZE], cOscStateL[BUFFER_SIZE], cBuf[BUFFER_SIZE];
	ViInt32		iMode, retCnt, nCountH, nCountL, onoff[8]={0,0,0,0,0,0,0,0}, i, iScanType, iTemp[8]={0,0,0,0,0,0,0,0};
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	checkErr( CheckStepRange(vi, step));	
	
	Fmt     ( ChanId, "%d", step);
	Fmt     ( Scanner, "%d", scanerNumber);
	
	checkErr( Ivi_GetAttributeViInt32 (vi, ChanId, C19032P_ATTR_STEP_MODE_TYPE, 0, &iMode));
	if (iMode != C19032P_VAL_MODE_OSC)
		return VI_ERROR_INVALID_MODE_COMMAND;	
	checkErr( Ivi_GetAttributeViInt32 (vi, Scanner, C19032P_ATTR_OPTION_SCAN_TYPE, 0, &iScanType));
	if (scanerNumber<1 || scanerNumber>8)
		return VI_ERROR_SCAN_NUM;
	if (iScanType==0)
		return VI_ERROR_SCANNER_BOX_NULL;

	else
	{
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:OSC:CHAN:HIGH (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:OSC:CHAN:LOW (@%d(0))\n", step, scanerNumber);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		
		
		
		switch (iScanType)
		{
			case C19032P_VAL_GB_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = iGB_FloatingChState[5][i];
				break;
			case C19032P_VAL_6000_01:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_01ChState[5][i];
				break;
			case C19032P_VAL_6000_02:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_02ChState[5][i];
				break;
			case C19032P_VAL_6000_03:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_03ChState[5][i];
				break;
			case C19032P_VAL_6000_04:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04ChState[5][i];
				break;
			case C19032P_VAL_6000_06:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06ChState[5][i];
				break;
			case C19032P_VAL_9030A: 
				for (i=0;i<8;i++)
					iTemp[i] = i9030AChState[5][i];
				break;
			case C19032P_VAL_6000_04_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_04FloatingChState[5][i];
				break;
			case C19032P_VAL_6000_06_FLOATING:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_06FloatingChState[5][i];
				break;
			case C19032P_VAL_6000_11:
				for (i=0;i<8;i++)
					iTemp[i] = i6000_11ChState[5][i];
				break;
		}
	
		for (i=0;i<8;i++)
		{
			onoff[i] = (iTemp[i]>=0)? 1 : 0;
			if (onoff[i])
			{
				switch (iTemp[i])
				{
					case C19032P_VAL_X:
						if (ChState[i]!=0)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_LX:
						if (ChState[i]!=0 && ChState[i]!=1)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HX:
						if (ChState[i]!=0 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
					case C19032P_VAL_HLX:
						if (ChState[i]!=0 && ChState[i]!=1 && ChState[i]!=2)
							return VI_ERROR_DATA_OUT_OF_RANGE;
						break;
				}	  
			}
		}

		nCountH = nCountL = 0;
		memset( cOscStateH, 0, sizeof(cOscStateH));
		memset( cOscStateL, 0, sizeof(cOscStateL));
		nCountH += sprintf(cOscStateH + nCountH, "(@%d(", scanerNumber);
		nCountL += sprintf(cOscStateL + nCountL, "(@%d(", scanerNumber);

		for (i=0;i<8;i++)
		{
			if (onoff[i] && ChState[i]==1)
				nCountL += sprintf (cOscStateL + nCountL, "%d,", i+1);
			if (onoff[i] && ChState[i]==2)
				nCountH += sprintf (cOscStateH + nCountH, "%d,", i+1);
			
		//	sprintf ( ChanId, "%d", scanerNumber*100+i+1);
		//	checkErr( Ivi_AddToChannelTable (vi, ChanId));
		//	checkErr( Ivi_ApplyDefaultSetup (vi));
		//	checkErr( Ivi_SetAttributeViInt32 (vi, ChanId, C19032P_ATTR_OSC_CHANNEL_STATE, IVI_VAL_SET_CACHE_ONLY, (onoff[i])? ChState[i] : -1));
		}
		
		if (nCountH==4)	
			nCountH += sprintf(cOscStateH + (nCountH-1), "(0))");  		//(nCountH-1) to avoid ","
		else
			nCountH += sprintf(cOscStateH + (nCountH-1), "))");  		//(nCountH-1) to avoid ","  
		if (nCountL==4)	
			nCountL += sprintf(cOscStateL + (nCountL-1), "(0))");  		//(nCountL-1) to avoid ","
		else
			nCountL += sprintf(cOscStateL + (nCountL-1), "))");  		//(nCountL-1) to avoid ","  

		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:OSC:CHAN:HIGH %s\n", step, cOscStateH);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt));
		Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:OSC:CHAN:LOW %s\n", step, cOscStateL);
		viCheckErr( viWrite ( io, cBuf, NumFmtdBytes(), &retCnt)); 
		checkErr  ( C19032P_Check_Parameter_Error(vi));
	}			
	
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetPresetIEC                                       
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetPresetIEC (ViSession vi,
                                        ViInt32 *IEC601)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (IEC601==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for IEC601" );	
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_PRESET_IEC_STATE, 0, IEC601));   
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}
/*****************************************************************************
 * Function: C19032P_GetPresetIEC                                       
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetPresetRjudgement (ViSession vi,
                                               ViInt32 *rampJudgement)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));
	
	if (rampJudgement==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for rampJudgement" );	
	
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_PRESET_RAMP_JUDGEMENT, 0, rampJudgement));   
    
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_GetPresetGFI                                       
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetPresetGFI (ViSession vi,
                                        ViChar presetGFI[])
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr( C19032P_LockSession( vi, VI_NULL ));

	if (presetGFI==VI_NULL)
		viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for presetGFI" );	
						
	checkErr( Ivi_GetAttributeViString (vi, VI_NULL, C19032P_ATTR_PRESET_GFI, 0, 
	AUTO_BUFFER_SIZE, presetGFI));   
    														
Error:
	C19032P_UnlockSession( vi, VI_NULL );
	return error;
}

/*****************************************************************************
 * Function: C19032P_SetAttribute<type> Functions                                    
 * Purpose:  These functions enable the instrument driver user to set 
 *           attribute values directly.  There are typesafe versions for 
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.                                         
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_SetAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                ViAttr attributeId, ViInt32 value)
{                                                                                                           
    return Ivi_SetAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                    value);
}                                                                                                           
ViStatus _VI_FUNC C19032P_SetAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViReal64 value)
{                                                                                                           
    return Ivi_SetAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}                                                                                                           
ViStatus _VI_FUNC C19032P_SetAttributeViString (ViSession vi, ViConstString channelName, 
                                                 ViAttr attributeId, ViConstString value) 
{   
    return Ivi_SetAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                     value);
}   
ViStatus _VI_FUNC C19032P_SetAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViBoolean value)
{                                                                                                           
    return Ivi_SetAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           
ViStatus _VI_FUNC C19032P_SetAttributeViSession (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViSession value)
{                                                                                                           
    return Ivi_SetAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}                                                                                                           

/*****************************************************************************
 * Function: C19032P_CheckAttribute<type> Functions                                  
 * Purpose:  These functions enable the instrument driver user to check  
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the C19032P_ATTR_RANGE_CHECK 
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32, 
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.                         
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_CheckAttributeViInt32 (ViSession vi, ViConstString channelName, 
                                                  ViAttr attributeId, ViInt32 value)
{                                                                                                           
    return Ivi_CheckAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                      value);
}
ViStatus _VI_FUNC C19032P_CheckAttributeViReal64 (ViSession vi, ViConstString channelName, 
                                                   ViAttr attributeId, ViReal64 value)
{                                                                                                           
    return Ivi_CheckAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                       value);
}                                                                                                           
ViStatus _VI_FUNC C19032P_CheckAttributeViString (ViSession vi, ViConstString channelName, 
                                                   ViAttr attributeId, ViConstString value)  
{   
    return Ivi_CheckAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                       value);
}   
ViStatus _VI_FUNC C19032P_CheckAttributeViBoolean (ViSession vi, ViConstString channelName, 
                                                    ViAttr attributeId, ViBoolean value)
{                                                                                                           
    return Ivi_CheckAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                        value);
}                                                                                                           
ViStatus _VI_FUNC C19032P_CheckAttributeViSession (ViSession vi, ViConstString channelName, 
                                                    ViAttr attributeId, ViSession value)
{                                                                                                           
    return Ivi_CheckAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                                        value);
}                                                                                                           

/*****************************************************************************
 * Function: C19032P_GetNextCoercionRecord                        
 * Purpose:  This function enables the instrument driver user to obtain
 *           the coercion information associated with the IVI session.                                                              
 *           This function retrieves and clears the oldest instance in which 
 *           the instrument driver coerced a value the instrument driver user
 *           specified to another value.                     
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetNextCoercionRecord (ViSession vi,
                                                  ViInt32 bufferSize,
                                                  ViChar  record[])
{
    return Ivi_GetNextCoercionString (vi, bufferSize, record);
}

/*****************************************************************************
 * Function: C19032P_LockSession and C19032P_UnlockSession Functions                        
 * Purpose:  These functions enable the instrument driver user to lock the 
 *           session around a sequence of driver calls during which other
 *           execution threads must not disturb the instrument state.
 *                                                                          
 *           NOTE:  The callerHasLock parameter must be a local variable 
 *           initialized to VI_FALSE and passed by reference, or you can pass 
 *           VI_NULL.                     
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_LockSession (ViSession vi, ViBoolean *callerHasLock)  
{                                              
   return 0;//Ivi_LockSession(vi,callerHasLock);   
 // return Ivi_LockSession(vi,callerHasLock); 
}                                              
ViStatus _VI_FUNC C19032P_UnlockSession (ViSession vi, ViBoolean *callerHasLock) 
{ 

   return 0;//Ivi_UnlockSession(vi,callerHasLock);  
  //   return Ivi_UnlockSession(vi,callerHasLock); 
}   

/*****************************************************************************
 * Function: C19032P_GetErrorInfo and C19032P_ClearErrorInfo Functions                       
 * Purpose:  These functions enable the instrument driver user to  
 *           get or clear the error information the driver associates with the
 *           IVI session.                                                        
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_GetErrorInfo (ViSession vi, ViStatus *primaryError, 
                                         ViStatus *secondaryError, ViChar errorElaboration[255])  
{                                                                                                           
    return Ivi_GetErrorInfo(vi, primaryError, secondaryError, errorElaboration);                                
}                                                                                                           
ViStatus _VI_FUNC C19032P_ClearErrorInfo (ViSession vi)                                                        
{                                                                                                           
    return Ivi_ClearErrorInfo (vi);                                                                             
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions                      
 * Purpose:  These functions enable the instrument driver user to  
 *           write and read commands directly to and from the instrument.            
 *                                                                           
 *           Note:  These functions bypass the IVI attribute state caching.  
 *                  WriteInstrData invalidates the cached values for all 
 *                  attributes.
 *****************************************************************************/
ViStatus _VI_FUNC C19032P_WriteInstrData (ViSession vi, ViConstString writeBuffer)   
{   
    return Ivi_WriteInstrData (vi, writeBuffer);    
}   
ViStatus _VI_FUNC C19032P_ReadInstrData (ViSession vi, ViInt32 numBytes, 
                                          ViChar rdBuf[], ViInt32 *bytesRead)  
{   
    return Ivi_ReadInstrData (vi, numBytes, rdBuf, bytesRead);   
} 

/*****************************************************************************
 *-------------------- Utility Functions (Not Exported) ---------------------*
 *****************************************************************************/
 
/*****************************************************************************
 * Function: C19032P_DefaultInstrSetup                                               
 * Purpose:  This function sends a default setup to the instrument.  The    
 *           C19032P_reset function calls this function.  The 
 *           C19032P_IviInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter.  This function is 
 *           useful for configuring settings that other instrument driver 
 *           functions require.    
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus C19032P_DefaultInstrSetup (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
	ViSession   io = Ivi_IOSession(vi); /* call only when locked */
	ViChar		cBuf[BUFFER_SIZE], ChanId[4];
	ViInt32		iScanNum, iFloating, i, retCnt;

    /* Invalidate all attributes */
    checkErr( Ivi_InvalidateAllAttributes (vi));
    
    if (!Ivi_Simulating(vi))
    {
	    checkErr  ( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
	    Fmt       ( cBuf, "%s<*CLS;*ESE 1;*SRE 32\n");
	    viCheckErr( viWrite(io, cBuf, NumFmtdBytes(), &retCnt));
	//	viCheckErr( viPrintf(io, "*CLS;*ESE 1;*SRE 32\n"));
    }
Error:
    return error;
}

/*****************************************************************************
 * Function: ReadToFile and WriteFromFile Functions                          
 * Purpose:  Functions for instrument driver developers to read/write        
 *           instrument data to/from a file.                                 
 *****************************************************************************/
static ViStatus C19032P_ReadToFile (ViSession vi, ViConstString filename, 
                                     ViInt32 maxBytesToRead, ViInt32 fileAction, 
                                     ViInt32 *totalBytesWritten)  
{   
    return Ivi_ReadToFile (vi, filename, maxBytesToRead, fileAction, totalBytesWritten);  
}   
static ViStatus C19032P_WriteFromFile (ViSession vi, ViConstString filename, 
                                        ViInt32 maxBytesToWrite, ViInt32 byteOffset, 
                                        ViInt32 *totalBytesWritten) 
{   
    return Ivi_WriteFromFile (vi, filename, maxBytesToWrite, byteOffset, totalBytesWritten); 
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/


/*****************************************************************************
 *----------------- Attribute Range Tables and Callbacks --------------------*
 *****************************************************************************/

/*- C19032P_ATTR_ID_QUERY_RESPONSE -*/ 
static ViStatus _VI_FUNC C19032PAttrIdQueryResponse_ReadCallback (ViSession vi, 
                                                                 ViSession io,
                                                                 ViConstString channelName, 
                                                                 ViAttr attributeId,
                                                                 const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rdBuffer[BUFFER_SIZE];
    ViUInt32    retCnt;
    
    viCheckErr( viWrite (io, "*IDN?\n", 6, &retCnt));
	viCheckErr( viRead  (io, rdBuffer, BUFFER_SIZE-1, &retCnt));
    rdBuffer[retCnt] =0;

    checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, rdBuffer));
    
Error:
    return error;
}
    
/*- C19032P_ATTR_SPECIFIC_DRIVER_REVISION -*/	
static ViStatus _VI_FUNC C19032PAttrDriverRevision_ReadCallback (ViSession vi, 
                                                                  ViSession io,
                                                                  ViConstString channelName, 
                                                                  ViAttr attributeId,
                                                                  const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      driverRevision[256];
    
    
    sprintf (driverRevision, 
             "Driver: C19032P %.2f, Compiler: %s %3.2f, "
             "Components: IVIEngine %.2f, VISA-Spec %.2f",
             C19032P_MAJOR_VERSION + C19032P_MINOR_VERSION/1000.0, 
             IVI_COMPILER_NAME, IVI_COMPILER_VER_NUM, 
             IVI_ENGINE_MAJOR_VERSION + IVI_ENGINE_MINOR_VERSION/1000.0, 
             Ivi_ConvertVISAVer(VI_SPEC_VERSION));

    checkErr( Ivi_SetValInStringCallback (vi, attributeId, driverRevision));    
Error:
    return error;
} 

/*- C19032P_ATTR_INSTRUMENT_FIRMWARE_REVISION -*/   
static ViStatus _VI_FUNC C19032PAttrFirmwareRevision_ReadCallback (ViSession vi, 
                                                                    ViSession io,
                                                                    ViConstString channelName, 
                                                                    ViAttr attributeId,
                                                                    const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      instrRev[BUFFER_SIZE];
    
    viCheckErr( viPrintf(io, "*IDN?\n"));
	viCheckErr( viScanf (io, "%*[^,],%*[^,],%*[^,],%256[^\n]", instrRev));
    checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, instrRev));
    
Error:
    return error;
}

/*- C19032P_ATTR_INSTRUMENT_MANUFACTURER -*/   
static ViStatus _VI_FUNC C19032PAttrInstrumentManufacturer_ReadCallback (ViSession vi, 
                                                                          ViSession io,
                                                                          ViConstString channelName, 
                                                                          ViAttr attributeId,
                                                                          const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rdBuffer[BUFFER_SIZE];
    ViChar      idQ[BUFFER_SIZE];
    
    checkErr( Ivi_GetAttributeViString (vi, "", C19032P_ATTR_ID_QUERY_RESPONSE, 0, AUTO_BUFFER_SIZE, idQ));
    sscanf  ( idQ, "%256[^,]", rdBuffer);
    checkErr( Ivi_SetValInStringCallback (vi, attributeId, rdBuffer));
    
Error:
    return error;
}

/*- C19032P_ATTR_INSTRUMENT_MODEL -*/ 
static ViStatus _VI_FUNC C19032PAttrInstrumentModel_ReadCallback (ViSession vi, 
                                                                   ViSession io,
                                                                   ViConstString channelName, 
                                                                   ViAttr attributeId,
                                                                   const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rdBuffer[BUFFER_SIZE];
    ViChar      idQ[BUFFER_SIZE];
    
    checkErr( Ivi_GetAttributeViString (vi, "", C19032P_ATTR_ID_QUERY_RESPONSE, 0, BUFFER_SIZE, idQ));
    sscanf  ( idQ, "%*[^,],%256[^,]", rdBuffer);
    checkErr( Ivi_SetValInStringCallback (vi, attributeId, rdBuffer));
    
Error:
    return error;
}
   
/* C19032P_ATTR_AC_ARC_LIMIT */
static ViStatus _VI_FUNC C19032PAttrAcArcLimit_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:LIM:ARC:LEV?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcArcLimit_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:AC:LIM:ARC:LEV %lf\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_AC_ARC_FILTER */ 
static ViStatus _VI_FUNC C19032PAttrAcArcFilter_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt, iTempVal;
	ViReal64	fTempVal;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:LIM:ARC:FILT?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	fTempVal = atof(cResult);
	iTempVal = RoundRealToNearestInteger (fTempVal);
	Fmt       ( cBuf, "%s<%d", iTempVal);
	checkErr  ( Ivi_GetViInt32EntryFromString (cBuf, &ChrArcFilterRangeTable, value, VI_NULL, VI_NULL, VI_NULL, VI_NULL));

Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcArcFilter_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	ViString	cmd;
	
	checkErr  ( Ivi_GetViInt32EntryFromValue (value, &ChrArcFilterRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cmd, VI_NULL));
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:LIM:ARC:FILT %s\n", channelName, cmd);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_AC_FALL_TIME */
static ViStatus _VI_FUNC C19032PAttrAcFallTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:AC:TIME:FALL %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_AC_LIMIT_HIGH */
static ViStatus _VI_FUNC C19032PAttrAcLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:LIM:HIGH?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:AC:LIM:HIGH %.8f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

/* C19032P_ATTR_AC_LIMIT_LOW */
static ViStatus _VI_FUNC C19032PAttrAcLimitLow_RangeTableCallback (ViSession vi,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal, fMinVal;
		ViInt32				nModel;   
	
	checkErr( Ivi_GetAttributeViInt32( vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	
	if ( nModel == C19032P_VAL_MODEL_19032P)
	{
//		checkErr( Ivi_GetViReal64EntryFromValue( 1, &ChrAcLimitHighRangeTable,&fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL,VI_NULL));  
		fMaxVal=0.04;
		fMinVal=0.000001;
		
	}	
	else
	{
//		checkErr( Ivi_GetViReal64EntryFromValue( 0, &ChrAcLimitHighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL,VI_NULL));  	
	
		fMaxVal=0.1;
		fMinVal=0.000001;	
	}
	
	
		
	
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));	
	


Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:LIM:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:AC:LIM:LOW %.8f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

/* C19032P_ATTR_AC_RAMP_TIME */
static ViStatus _VI_FUNC C19032PAttrAcRampTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:TIME:RAMP?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);

Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcRampTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:AC:TIME:RAMP %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_AC_TEST_TIME */
static ViStatus _VI_FUNC C19032PAttrAcTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:TIME:TEST?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:AC:TIME:TEST %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

/* C19032P_ATTR_AC_VOLTAGE */
static ViStatus _VI_FUNC C19032PAttrAcVoltage_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:LEV?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcVoltage_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:AC:LEV %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

/* C19032P_ATTR_AUTO_RESULT_STATE */ 
static ViStatus _VI_FUNC C19032PAttrAutoResultState_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:RES:AREP?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAutoResultState_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:RES:AREP %s\n", (value)? "ON" : "OFF");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_DC_ARC_FILTER */
static ViStatus _VI_FUNC C19032PAttrDcArcFilter_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt, iTempVal;
	ViReal64	fTempVal;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:LIM:ARC:FILT?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	fTempVal = atof(cResult);
	iTempVal = RoundRealToNearestInteger (fTempVal);
	Fmt       ( cBuf, "%s<%d", iTempVal);
	checkErr  ( Ivi_GetViInt32EntryFromString (cBuf, &ChrArcFilterRangeTable, value, VI_NULL, VI_NULL, VI_NULL, VI_NULL));

	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcArcFilter_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	ViString	cmd;
	
	checkErr  ( Ivi_GetViInt32EntryFromValue (value, &ChrArcFilterRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cmd, VI_NULL));
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:LIM:ARC:FILT %s\n", channelName, cmd);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error; 
}

/* C19032P_ATTR_DC_ARC_LIMIT */
static ViStatus _VI_FUNC C19032PAttrDcArcLimit_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:LIM:ARC:LEV?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcArcLimit_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:DC:LIM:ARC:LEV %.3f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_DC_DWELL_TIME */
static ViStatus _VI_FUNC C19032PAttrDcDwellTime_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:DC:TIME:DWEL %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_DC_FALL_TIME */
static ViStatus _VI_FUNC C19032PAttrDcFallTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:DC:TIME:FALL %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_DC_LIMIT_HIGH */
static ViStatus _VI_FUNC C19032PAttrDcLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:LIM:HIGH?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	

 // 	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:LIM:HIGH %.10f\n", channelName,value);
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:DC:LIM:HIGH %.10f\n", channelName, value);    	
	viCheckErr( viWrite (io, cBuf,nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_DC_LIMIT_LOW */

static ViStatus _VI_FUNC C19032PAttrDcLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:LIM:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:DC:LIM:LOW %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_DC_RAMP_TIME */
static ViStatus _VI_FUNC C19032PAttrDcRampTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:TIME:RAMP?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcRampTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:DC:TIME:RAMP %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error; 
}

/* C19032P_ATTR_DC_TEST_TIME */
static ViStatus _VI_FUNC C19032PAttrDcTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:TIME:TEST?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);

Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:DC:TIME:TEST %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_DC_VOLTAGE */
static ViStatus _VI_FUNC C19032PAttrDcVoltage_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:LEV?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcVoltage_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:DC:LEV %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_DELETE_STEP */
static ViStatus _VI_FUNC C19032PAttrDeleteStep_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%d:DEL\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	checkErr  ( C19032P_Check_Parameter_Error(vi));   	 
	
Error:
	return error;
}

/* C19032P_ATTR_GB_CURRENT */
static ViStatus _VI_FUNC C19032PAttrGbCurrent_RangeTableCallback (ViSession vi,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal, fMinVal;
	ViInt32				iRate;
	ViInt32				nModel;
	
	checkErr( Ivi_GetAttributeViInt32(vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	
	if ( nModel == C19032P_VAL_MODEL_19032P)
		checkErr( Ivi_GetViReal64EntryFromCmdValue (1, &ChrGbCurrentRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));     
	else
	{
		checkErr( Ivi_GetAttributeViInt32(vi, VI_NULL, C19032P_ATTR_OPTION_GB_RATE, 0, &iRate));
		checkErr( Ivi_GetViReal64EntryFromCmdValue (iRate, &ChrGbCurrentRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));
	}	

	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));

	/*
		NOTE:  Insert code here to select the correct range table
		or to modify a dynamic range table.  Set the tblPtr
		local variable to the address of the range table.
	*/

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbCurrent_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:GB:LEV?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
 	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbCurrent_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:GB:LEV %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	checkErr  ( C19032P_Check_Parameter_Error(vi));  	 
Error:
	return error;
}

/* C19032P_ATTR_GB_LIMIT_HIGH */
static ViStatus _VI_FUNC C19032PAttrGbLimitHigh_RangeTableCallback (ViSession vi,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fCurrent, fTempVal, fMaxVal, fMinVal;
	ViInt32				iRate;

	checkErr( Ivi_GetAttributeViReal64(vi, channelName, C19032P_ATTR_GB_CURRENT, 0, &fCurrent));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_OPTION_GB_RATE, 0, &iRate));
	
	fTempVal = 0.21 * pow(30, 2) / iRate;
	fTempVal/= fCurrent;
	fMaxVal = (fTempVal>0.51)? 0.51:fTempVal;
	fMinVal = 0.0001;
	
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));
	/*
		NOTE:  Insert code here to select the correct range table
		or to modify a dynamic range table.  Set the tblPtr
		local variable to the address of the range table.
	*/

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;  
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:GB:LIM:HIGH?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;														   
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:GB:LIM:HIGH %lf\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	checkErr  ( C19032P_Check_Parameter_Error(vi));   	
	
Error:
	return error;
}

/* C19032P_ATTR_GB_LIMIT_LOW */ 
static ViStatus _VI_FUNC C19032PAttrGbLimitLow_RangeTableCallback (ViSession vi,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal, fMinVal;

	checkErr( Ivi_GetAttributeViReal64(vi, channelName, C19032P_ATTR_GB_LIMIT_HIGH, 0, &fMaxVal));
	fMinVal = 0;
	
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));

	/*
		NOTE:  Insert code here to select the correct range table
		or to modify a dynamic range table.  Set the tblPtr
		local variable to the address of the range table.
	*/

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:GB:LIM:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:GB:LIM:LOW %lf\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	checkErr  ( C19032P_Check_Parameter_Error(vi));   
Error:
	return error;
}

/* C19032P_ATTR_GB_TEST_TIME */
static ViStatus _VI_FUNC C19032PAttrGbTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:GB:TIME:TEST?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:GB:TIME:TEST %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_GB_TWINPORT_STATE */
static ViStatus _VI_FUNC C19032PAttrGbTwinportState_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:GB:TPOR?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbTwinportState_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:GB:TPOR %d\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_IR_CURR_RANGE_AUTO */
static ViStatus _VI_FUNC C19032PAttrIrCurrRangeAuto_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:RANG:AUTO %s", channelName, (value)? "ON":"OFF" );
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_IR_CURR_RANGE_UPPER */
static ViStatus _VI_FUNC C19032PAttrIrCurrRangeUpper_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:IR:RANGE:UPPER %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_IR_FALL_TIME */
static ViStatus _VI_FUNC C19032PAttrIrFallTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:IR:TIME:FALL %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_IR_LIMIT_HIGH */
static ViStatus _VI_FUNC C19032PAttrIrLimitHigh_RangeTableCallback (ViSession vi,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal, fMinVal;

	checkErr( Ivi_GetAttributeViReal64(vi, channelName, C19032P_ATTR_IR_LIMIT_LOW, 0, &fMinVal));
	fMaxVal = 50000e+6;
	
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, 0, 0, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 1, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));

	/*
		NOTE:  Insert code here to select the correct range table
		or to modify a dynamic range table.  Set the tblPtr
		local variable to the address of the range table.
	*/

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:LIM:HIGH?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:IR:LIM:HIGH %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_IR_LIMIT_LOW */
static ViStatus _VI_FUNC C19032PAttrIrLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:LIM:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:IR:LIM:LOW %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_IR_RAMP_TIME */ 
static ViStatus _VI_FUNC C19032PAttrIrRampTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:TIME:RAMP?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrRampTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:IR:TIME:RAMP %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_IR_TEST_TIME */
static ViStatus _VI_FUNC C19032PAttrIrTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:TIME:TEST?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:IR:TIME:TEST %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_IR_VOLTAGE */
static ViStatus _VI_FUNC C19032PAttrIrVoltage_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP %s:IR:LEV?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrVoltage_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:IR:LEV %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_LC_AC_HIGH_LIMIT */
static ViStatus _VI_FUNC C19032PAttrLcAcHighLimit_RangeTableCallback (ViSession vi,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal, fMinVal;
	ViInt32				iDevice;
	ViChar     			getLCDisplay[20]; 
	
	checkErr( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_DEVICE_TYPE, 0, &iDevice));
	
	if (iDevice==5)
	{
	
		checkErr( Ivi_GetAttributeViString( vi,channelName, C19032P_ATTR_LC_DISPLAY,
											 IVI_VAL_DIRECT_USER_CALL,-1,getLCDisplay) );	
	
		if (!strncmp(getLCDisplay, "RMS", 7)) 
		 {
		 fMaxVal=0.05;
		 fMinVal=0.0000001;
		 
		 }
		 else
		 {
		 fMaxVal=0.07;
		 fMinVal=0.0000001;
		
		 }
	}
	else
		checkErr( Ivi_GetViReal64EntryFromCmdValue (iDevice, &ChrLcLimitHighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));	
	
	
	
	checkErr( Ivi_GetViReal64EntryFromCmdValue (iDevice, &ChrLcLimitHighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));

	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));

	/*
		NOTE:  Insert code here to select the correct range table
		or to modify a dynamic range table.  Set the tblPtr
		local variable to the address of the range table.
	*/

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcAcHighLimit_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:LAC %.8f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_LC_DC_HIGH_LIMIT */
static ViStatus _VI_FUNC C19032PAttrLcDcHighLimit_RangeTableCallback (ViSession vi,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal, fMinVal;
	
	checkErr( Ivi_GetAttributeViReal64 (vi, channelName, C19032P_ATTR_LC_LIMIT_HIGH, 0, &fMaxVal));	
	fMinVal = 0;
	
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));

	/*
		NOTE:  Insert code here to select the correct range table
		or to modify a dynamic range table.  Set the tblPtr
		local variable to the address of the range table.
	*/

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcDcHighLimit_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:LDC %.8f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_LC_DEVICE_TYPE */
static ViStatus _VI_FUNC C19032PAttrLcDeviceType_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:DEV?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
    checkErr  ( Ivi_GetViInt32EntryFromString (cResult, &attrLcDeviceTypeRangeTable, value, VI_NULL, VI_NULL, VI_NULL, VI_NULL));
    
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcDeviceType_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViString	cDevice;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	checkErr  ( Ivi_GetViInt32EntryFromValue (value, &attrLcDeviceTypeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cDevice, VI_NULL));
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:DEV %s\n", channelName, cDevice);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_LC_GR_SWITCH_STATE */
static ViStatus _VI_FUNC C19032PAttrLcGrSwitchState_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:GSWI?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcGrSwitchState_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:GSWI %d\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_LC_LIMIT_HIGH */
static ViStatus _VI_FUNC C19032PAttrLcLimitHigh_RangeTableCallback (ViSession vi,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iDevice;
	ViReal64			fMaxVal, fMinVal;
	ViChar     			getLCDisplay[20];


	checkErr( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_DEVICE_TYPE, 0, &iDevice));
	
	if (iDevice==5)
	{
	
		checkErr( Ivi_GetAttributeViString( vi,channelName, C19032P_ATTR_LC_DISPLAY,
											 IVI_VAL_DIRECT_USER_CALL,-1,getLCDisplay) );	
	
		if (!strncmp(getLCDisplay, "RMS", 7)) 
		 {
		 fMaxVal=0.05;
		 fMinVal=0.0000001;
		 
		 }
		 else
		 {
		 fMaxVal=0.07;
		 fMinVal=0.0000001;
		
		 }
	}
	else
		checkErr( Ivi_GetViReal64EntryFromCmdValue (iDevice, &ChrLcLimitHighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));
	
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));
	 

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcLimitHigh_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:LIM:HIGH?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcLimitHigh_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:LIM:HIGH %.8f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_LC_LIMIT_LOW */
static ViStatus _VI_FUNC C19032PAttrLcLimitLow_RangeTableCallback (ViSession vi,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iDevice;
	ViReal64			fMaxVal, fMinVal;
	ViChar     			getLCDisplay[20];
	
	
	
	checkErr( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_DEVICE_TYPE, 0, &iDevice));
	
	if (iDevice==5)
	{
	
		checkErr( Ivi_GetAttributeViString( vi,channelName, C19032P_ATTR_LC_DISPLAY,
											 IVI_VAL_DIRECT_USER_CALL,-1,getLCDisplay) );	
	
		if (!strncmp(getLCDisplay, "RMS", 7)) 
		 {
		 fMaxVal=0.05;
		 fMinVal=0.0000001;
		 
		 }
		 else
		 {
		 fMaxVal=0.07;
		 fMinVal=0.0000001;
		
		 }
	}
	else
		checkErr( Ivi_GetViReal64EntryFromCmdValue (iDevice, &ChrLcLimitHighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));
	
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcLimitLow_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:LIM:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcLimitLow_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS; 
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:LIM:LOW %.8f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_LC_LINE_INPUT */ 
static ViStatus _VI_FUNC C19032PAttrLcLineInput_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:LINE?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
    checkErr  ( Ivi_GetViInt32EntryFromString (cResult, &attrLcLineInputRangeTable, value, VI_NULL, VI_NULL, VI_NULL, VI_NULL));
								   
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcLineInput_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViString	cLineInput;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	checkErr  ( Ivi_GetViInt32EntryFromValue (value, &attrLcLineInputRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cLineInput, VI_NULL));
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:LINE %s\n", channelName, cLineInput);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_LC_METER_METHOD */
static ViStatus _VI_FUNC C19032PAttrLcMeterMethod_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:METE?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	checkErr  ( Ivi_GetViInt32EntryFromString (cResult, &attrLcMeterMethodRangeTable, value, VI_NULL, VI_NULL, VI_NULL, VI_NULL));
 								
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcMeterMethod_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViString	cMeter;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	checkErr  ( Ivi_GetViInt32EntryFromValue (value, &attrLcMeterMethodRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cMeter, VI_NULL));
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:METE %s\n", channelName, cMeter);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_LC_POWER_LIMIT_HIGH */
static ViStatus _VI_FUNC C19032PAttrLcPowerLimitHigh_RangeTableCallback (ViSession vi,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iPowerMode;
	ViSession			io = Ivi_IOSession (vi);
	ViChar				cScanner[10];
	ViReal64			fMaxVal, fMinVal;
	
 	checkErr  ( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
											
 	switch (iPowerMode)
 	{
 		case C19032P_VAL_LC_POWER_VOLTAGE:
			tblPtr = &ChrLcPowerVoltLimitRangeTable;
			checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));			  // add by angel 2006/06/13
			break;
		case C19032P_VAL_LC_POWER_CURRENT:
			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				tblPtr = &ChrLcPowerCurrLimit0708RangeTable;
			else
				tblPtr = &ChrLcPowerCurrLimit040506RangeTable; 	
				
			
			checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));
			break;
 		case C19032P_VAL_LC_POWER_VA:
			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				tblPtr = &ChrLcPowerVALimit0708RangeTable;
				
			else
				tblPtr = &ChrLcPowerVALimit040506RangeTable;   		


			checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));
			break;
	}
	
Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitHigh_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViString	cPowerMode;
	ViInt32		iPowerMode, retCnt;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	
	checkErr  ( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
	checkErr  ( Ivi_GetViInt32EntryFromValue(iPowerMode, &attrLcPowerModeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cPowerMode, VI_NULL));
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:POW:%s:LIM:HIGH?\n", channelName, cPowerMode);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitHigh_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViString	cPowerMode;    
	ViInt32		iPowerMode, retCnt, nCount;
	ViChar		cBuf[BUFFER_SIZE];
	
	checkErr  ( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
	checkErr  ( Ivi_GetViInt32EntryFromValue(iPowerMode, &attrLcPowerModeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cPowerMode, VI_NULL));
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:POW:%s:LIM:HIGH %.3f\n", channelName, cPowerMode, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	checkErr  ( C19032P_Check_Parameter_Error(vi)); 
Error:
	return error;
}

/* C19032P_ATTR_LC_POWER_LIMIT_LOW */
static ViStatus _VI_FUNC C19032PAttrLcPowerLimitLow_RangeTableCallback (ViSession vi,
                                                                       ViConstString channelName,
                                                                       ViAttr attributeId,
                                                                       IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iPowerMode;
	ViSession			io = Ivi_IOSession (vi);
	ViChar				cScanner[10];
	ViReal64			fMaxVal, fMinVal;
	
 	checkErr  ( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
											
 	switch (iPowerMode)
 	{
 		case C19032P_VAL_LC_POWER_VOLTAGE:
			tblPtr = &ChrLcPowerVoltLimitRangeTable;
			checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));			  // add by angel 2006/06/13
			break;
		case C19032P_VAL_LC_POWER_CURRENT:
			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				tblPtr = &ChrLcPowerCurrLimit0708RangeTable;
			else
				tblPtr = &ChrLcPowerCurrLimit040506RangeTable; 	
				
			
			checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));
			break;
 		case C19032P_VAL_LC_POWER_VA:
			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				tblPtr = &ChrLcPowerVALimit0708RangeTable;
				
			else
				tblPtr = &ChrLcPowerVALimit040506RangeTable;   		


			checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));
			break;
	}
	
Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitLow_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViString	cPowerMode;
	ViInt32		iPowerMode, retCnt;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	
	checkErr  ( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
	checkErr  ( Ivi_GetViInt32EntryFromValue(iPowerMode, &attrLcPowerModeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cPowerMode, VI_NULL));
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:POW:%s:LIM:LOW?\n", channelName, cPowerMode);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerLimitLow_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViString	cPowerMode;    
	ViInt32		iPowerMode, retCnt, nCount;
	ViChar		cBuf[BUFFER_SIZE];
	
	checkErr  ( Ivi_GetAttributeViInt32 (vi, channelName, C19032P_ATTR_LC_POWER_MODE, 0, &iPowerMode));
	checkErr  ( Ivi_GetViInt32EntryFromValue(iPowerMode, &attrLcPowerModeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cPowerMode, VI_NULL));
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:POW:%s:LIM:LOW %.3f\n", channelName, cPowerMode, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	checkErr  ( C19032P_Check_Parameter_Error(vi)); 
	
Error:
	return error;
}

/* C19032P_ATTR_LC_POWER_MODE */
static ViStatus _VI_FUNC C19032PAttrLcPowerMode_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:POW:MODE?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
   
	if (!CompareBytes( cResult, 1, "VOLT", 1, 2, 0))
		*value = 0;
	if (!CompareBytes( cResult, 1, "CURR", 1, 2, 0))
		*value = 1;
	if (!CompareBytes( cResult, 1, "VA", 1, 1, 0))
		*value = 2;
	if (!CompareBytes( cResult, 1, "SIMU", 1, 2, 0))
		*value = 3;
	if (!CompareBytes( cResult, 1, "SOUR", 1, 2, 0))
		*value = 4;
		
Error:
	return error;

}

static ViStatus _VI_FUNC C19032PAttrLcPowerMode_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViString	cPowerMode;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	checkErr  ( Ivi_GetViInt32EntryFromValue (value, &attrLcPowerModeRangeTable, VI_NULL, VI_NULL, VI_NULL, VI_NULL, &cPowerMode, VI_NULL));
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:POW:MODE %s\n", channelName, cPowerMode);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	checkErr  ( C19032P_Check_Parameter_Error(vi));
Error:
	return error;
}

/* C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT */
static ViStatus _VI_FUNC C19032PAttrLcPowerSimuTargetVolt_ReadCallback (ViSession vi,
                                                                       ViSession io,
                                                                       ViConstString channelName,
                                                                       ViAttr attributeId,
                                                                       ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:POWER:SIMU:TVOL?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSimuTargetVolt_WriteCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:POWER:SIMU:TVOL %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_LC_POWER_SOUR_TRAGET_FREQ */ 
static ViStatus _VI_FUNC C19032PAttrLcPowerSourTargetFreq_WriteCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:POWER:SOUR:TFR %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_LC_POWER_SOUR_TRAGET_VOLT */
static ViStatus _VI_FUNC C19032PAttrLcPowerSourTargetVolt_WriteCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:POWER:SOUR:TVOL %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_LC_TEST_TIME */ 
static ViStatus _VI_FUNC C19032PAttrLcTestTime_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:TIME:TEST?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcTestTime_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:TIME:TEST %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_LLO_STATE */
static ViStatus _VI_FUNC C19032PAttrLloState_WriteCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViBoolean value)
{
	ViStatus	error = VI_SUCCESS;
	short 		ibaddress[2] = {23,-1};  
	ViBoolean	LLO_State;
	
	if (value)
		SetRWLS(0, ibaddress);   
	else 
		EnableLocal(0, ibaddress);      
						
Error:
	return error;
}

/* C19032P_ATTR_MEM_DEL */
static ViStatus _VI_FUNC C19032PAttrMemDel_WriteCallback (ViSession vi,
                                                         ViSession io,
                                                         ViConstString channelName,
                                                         ViAttr attributeId,
                                                         ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<MEM:DEL:LOCA %d\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	checkErr  ( C19032P_Check_Parameter_Error(vi));
	
Error:
	return error;
}

/* C19032P_ATTR_MEM_NAME_DEFINE */
static ViStatus _VI_FUNC C19032PAttrMemNameDefine_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViConstString value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<MEM:STAT:DEF %s,%s\n", value, channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
//	checkErr  ( C19032P_Check_Parameter_Error(vi));   
	
Error:
	return error;
}

/* C19032P_ATTR_OFFSET_STATE */ 	   
static ViStatus _VI_FUNC C19032PAttrOffsetState_WriteCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STAR:OFFS %s\n", (value)? "GET" : "OFF");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_OPTION_GB_FLOATING */
static ViStatus _VI_FUNC C19032PAttrOptionGbFloating_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		nFWVersion, retCnt;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		nModel;

	Fmt       ( cBuf, "%s<SYST:OPT:SCAN:GBFL?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

/* C19032P_ATTR_OPTION_GB_RATE */
static ViStatus _VI_FUNC C19032PAttrOptionGbRate_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		nFWVersion, retCnt;
	ViInt32		nModel;

	Fmt       ( cBuf, "%s<SYST:OPT:GBC:RATE?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	checkErr  ( Ivi_GetViInt32EntryFromString (cResult, &attrOptionGbRateRangeTable, value, VI_NULL, VI_NULL, VI_NULL, VI_NULL));

Error:
	return error;
}

/* C19032P_ATTR_OPTION_SCAN_CARD */
static ViStatus _VI_FUNC C19032PAttrOptionScanCard_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                const ViConstString cacheValue)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		nFWVersion, retCnt;
	ViInt32		nModel;

	Fmt       ( cBuf, "%s<SYSTem:OPTion:SCAN:INSide:TYPE?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, cResult));


Error:
	return error;
}

/* C19032P_ATTR_OPTION_SCAN_NUM */
static ViStatus _VI_FUNC C19032PAttrOptionScanNum_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		nFWVersion, retCnt;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		nModel;
	
/*	
	checkErr  ( Ivi_GetAttributeViInt32 ( vi, VI_NULL, C19032P_ATTR_FWVERSION, 0, &nFWVersion));
	checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	if (nFWVersion<520 && nModel != C19032P_VAL_MODEL_19032P)
		return VI_ERROR_NOT_SUPPORT_VERSION;
*/		
		
	Fmt       ( cBuf, "%s<SYST:OPT:SCAN:NUMB?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

/* C19032P_ATTR_OPTION_USE_SOURCE */ 
static ViStatus _VI_FUNC C19032PAttrOptionUseSource_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViInt32		nFWVersion, retCnt;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		nModel;	

	Fmt       ( cBuf, "%s<SYST:OPT:SOUR:AC?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

/* C19032P_ATTR_OSC_OPEN_CHECK */
static ViStatus _VI_FUNC C19032PAttrOscOpenCheck_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:OSC:LIM:OPEN %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_OSC_SHRT_CHECK */
static ViStatus _VI_FUNC C19032PAttrOscShortCheck_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 value){
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:OSC:LIM:SHORT %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PAUSE_MESSAGE_STRING */
static ViStatus _VI_FUNC C19032PAttrPauseMessageString_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    const ViConstString cacheValue)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:PAUSE:MESS?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, cResult));
 			  
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPauseMessageString_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViConstString value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:PAUSE:MESS %s\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_PAUSE_SIGNAL_STATE */
static ViStatus _VI_FUNC C19032PAttrPauseSignalState_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViBoolean *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:PAUSE:UTSI?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPauseSignalState_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViBoolean value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:PAUSE:UTSI %d\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PAUSE_TEST_TIME */
static ViStatus _VI_FUNC C19032PAttrPauseTestTime_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:PAUSE:TIME:TEST %.2f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_PRESET_AC_FREQUENCY */ 
static ViStatus _VI_FUNC C19032PAttrPresetAcFrequency_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	ViReal64	fTempVal;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:AC:FREQ?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	fTempVal = atof(cResult);  
	*value = RoundRealToNearestInteger(fTempVal);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetAcFrequency_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;							
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:AC:FREQ %d\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PRESET_AUTO_RANGE */
static ViStatus _VI_FUNC C19032PAttrPresetAutoRange_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:WRAN:AUTO?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetAutoRange_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:WRAN:AUTO %d\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PRESET_GB_FREQUENCY */
static ViStatus _VI_FUNC C19032PAttrPresetGbFrequency_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;   
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:GB:FREQ?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetGbFrequency_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:GB:FREQ %.2f\n", value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PRESET_GB_VOLTAGE */
static ViStatus _VI_FUNC C19032PAttrPresetGbVoltage_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:GB:VOLT?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetGbVoltage_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:GB:VOLT %.2f\n", value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PRESET_IEC_STATE */
static ViStatus _VI_FUNC C19032PAttrPresetIecState_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:IEC?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetIecState_WriteCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:IEC %d\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PRESET_LOT_NUMBER */ 
static ViStatus _VI_FUNC C19032PAttrPresetLotNumber_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 const ViConstString cacheValue)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE]; 
    ViUInt32	retCnt;
    
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:NUM:LOT?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
    viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
    cResult[retCnt] =0;
    checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, cResult));
 	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetLotNumber_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViConstString value)
{
	ViStatus	error = VI_SUCCESS;  
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:NUM:LOT %s\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
 	checkErr(C19032P_Check_Parameter_Error(vi));   
Error:
	return error;	
}

/* C19032P_ATTR_PRESET_PART_NUMBER */
static ViStatus _VI_FUNC C19032PAttrPresetPartNumber_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  const ViConstString cacheValue)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE]; 
    ViUInt32	retCnt;
    
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:NUM:PART?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
    checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, cResult));

Error:
	return error;

}

static ViStatus _VI_FUNC C19032PAttrPresetPartNumber_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViConstString value)
{
	ViStatus	error = VI_SUCCESS;  
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:NUM:PART %s\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
  	checkErr(C19032P_Check_Parameter_Error(vi));   
Error:
	return error;	
}

/* C19032P_ATTR_PRESET_PASS_HOLD_TIME */
static ViStatus _VI_FUNC C19032PAttrPresetPassHoldTime_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:TIME:PASS?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetPassHoldTime_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:TIME:PASS %.2f\n", value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PRESET_RAMP_JUDGEMENT */	  
static ViStatus _VI_FUNC C19032PAttrPresetRampJudgement_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:RJUD %d\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_PRESET_SERIAL_NUMBER */ 
static ViStatus _VI_FUNC C19032PAttrPresetSerialNumber_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    const ViConstString cacheValue)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:NUM:SERI?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
    checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, cResult));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetSerialNumber_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViConstString value)
{
	ViStatus	error = VI_SUCCESS;  
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:NUM:SERI %s\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
  	checkErr(C19032P_Check_Parameter_Error(vi));   
Error:
	return error;	
}

/* C19032P_ATTR_PRESET_SOFT_AGC_STATE */
static ViStatus _VI_FUNC C19032PAttrPresetSoftAgcState_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:AGC?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetSoftAgcState_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:AGC %d\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PRESET_START_WAIT */
static ViStatus _VI_FUNC C19032PAttrPresetStartWait_ReadCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:TIME:AST?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetStartWait_WriteCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:TIME:AST %.2f\n", value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_PRESET_STEP_HOLD_TIME */ 
static ViStatus _VI_FUNC C19032PAttrPresetStepHoldTime_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:TIME:STEP?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value =  (!CompareBytes(cResult, 1, "KEY", 1, 2, 0))? 0.00 : atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetStepHoldTime_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	if (value==0)
	 	nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:TIME:STEP KEY\n");  
	else
		nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:TIME:STEP %.2f\n", value);  
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_QUERY_MEM_INDEX_NUM */
static ViStatus _VI_FUNC C19032PAttrQueryMemIndexNum_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		memName[BUFFER_SIZE], cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	checkErr  ( Ivi_GetAttributeViString( vi, VI_NULL, C19032P_ATTR_QUERY_MEM_INDEX, 0, AUTO_BUFFER_SIZE, memName));
	Fmt       ( cBuf, "%s<MEM:SEAR:NAME:IND? \"%s\"\n", memName);		   // modify by angel 2006/04/14
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
/*	if (*value!=0)
	{
		Fmt       ( cBuf, "%s<MEM:STAT:DEF? %s\n", memName);
		viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
		viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
		cResult[retCnt] = 0;
		*value = atoi(cResult);
	}*/
	
Error:
	return error;
}

/* C19032P_ATTR_REMOTE_START_KEY */ 
static ViStatus _VI_FUNC C19032PAttrRemoteStartKey_WriteCallback (ViSession vi,
                                                                 ViSession io,
                                                                 ViConstString channelName,
                                                                 ViAttr attributeId,
                                                                 ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<TRIG:SOUR:EXT:STAT %d\n", value);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}

/* C19032P_ATTR_RUN_COMPLETE_STATUS */
static ViStatus _VI_FUNC C19032PAttrRunCompleteStatus_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:RES:COMP?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

/* C19032P_ATTR_STANDARD_CAPACITANCE */
static ViStatus _VI_FUNC C19032PAttrStandardCapacitance_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STAR:CSTANDARD\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));

Error:
	return error;
}

/* C19032P_ATTR_STEP_MODE_TYPE */
static ViStatus _VI_FUNC C19032PAttrStepModeType_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:MODE?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	checkErr  ( Ivi_GetViInt32EntryFromString (cResult, &attrStepModeTypeRangeTable, value, VI_NULL, VI_NULL, VI_NULL, VI_NULL));
   
Error:
	return error;
}

/* C19032P_ATTR_STEP_SETTING */
static ViStatus _VI_FUNC C19032PAttrStepSetting_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             const ViConstString cacheValue)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:SET?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
 	checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, cResult));  

Error:
	return error;
}

/* C19032P_ATTR_SYS_VERSION */
static ViStatus _VI_FUNC C19032PAttrSysVersion_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            const ViConstString cacheValue)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SYST:VERS?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
   	checkErr  ( Ivi_SetValInStringCallback (vi, attributeId, cResult)); 	

Error:
	return error;
}

/* C19032P_ATTR_TEST_STATE */
static ViStatus _VI_FUNC C19032PAttrTestState_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViBoolean value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:%s\n", (value)? "STAR" : "STOP");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}
	  
	  
//CK add 20090616

//CK add 20090616 
static ViStatus _VI_FUNC C19032PAttrAcLimitHigh_RangeTableCallback (ViSession vi,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal, fMinVal;
	ViInt32				nModel;
	
	checkErr( Ivi_GetAttributeViInt32( vi, VI_NULL, C19032P_ATTR_MODEL, 0, &nModel));
	
	if ( nModel == C19032P_VAL_MODEL_19032P)
		checkErr( Ivi_GetViReal64EntryFromCmdValue( 1, &ChrAcLimitHighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));  
	else
		checkErr( Ivi_GetViReal64EntryFromCmdValue( 0, &ChrAcLimitHighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));  		
	
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, fMinVal, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));

	/*
		NOTE:  Insert code here to select the correct range table
		or to modify a dynamic range table.  Set the tblPtr
		local variable to the address of the range table.
	*/

Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrQueryMemMaxCapacity_ReadCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<:MEM:NST?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult); 	

Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrFuncMode_ReadCallback (ViSession vi,
                                                           ViSession io,
                                                           ViConstString channelName,
                                                           ViAttr attributeId,
                                                           ViBoolean *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<:FUNCs?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult); 	

Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrFuncMode_WriteCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViBoolean value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<FUNC %s\n", (value)? "BRE" : "GEN");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAutoReportSaved_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViBoolean *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ASAV?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult); 	

Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAutoReportSaved_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViBoolean value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:RES:ASAV %s\n", (value)? "ON" : "OFF");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcLimitReal_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:AC:LIM:REAL?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcLimitReal_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:AC:LIM:REAL %.8f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbCurrentOffset_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:GB:CURR:OFFS?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrGbCurrentOffset_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:GB:CURR:OFFS %.4f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcFrequency_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:AC:FREQ?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcFrequency_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViInt32 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:AC:FREQ %d\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcFallTime_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:AC:TIME:FALL?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcCurrentOffset_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:AC:CURR:OFFS?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrAcCurrentOffset_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:AC:CURR:OFFS %.6f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}


static ViStatus _VI_FUNC C19032PAttrDcCurrentOffset_RangeTableCallback (ViSession vi,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal=0, fMinVal=0,HighCurrentLimit=0;
	ViInt32				nModel;
	
	
	checkErr( Ivi_GetAttributeViReal64 (vi, channelName, C19032P_ATTR_DC_LIMIT_HIGH, 0, &HighCurrentLimit)); 
	
 /*
	if ( HighCurrentLimit>=0.000000001 && HighCurrentLimit<=0.000002999 )
																																
		checkErr( Ivi_GetViReal64EntryFromIndex ( 0, &attrDcCurrentOffset_Lowest_RangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL,VI_NULL)); 
	else if( HighCurrentLimit>0.000002999 && HighCurrentLimit<=0.00002999)
		checkErr( Ivi_GetViReal64EntryFromIndex ( 0, &attrDcCurrentOffset_Lower_RangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));     	
	else if( HighCurrentLimit>0.00002999 && HighCurrentLimit<=0.0002999)  		
 		checkErr( Ivi_GetViReal64EntryFromIndex ( 0, &attrDcCurrentOffset_LowRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));   
 		
 		
	else  if(HighCurrentLimit>0.0002999 && HighCurrentLimit<=0.002999)
		checkErr( Ivi_GetViReal64EntryFromIndex ( 0, &attrDcCurrentOffset_MidRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));       	
	else // (HighCurrentLimit>0.0029999 && HighCurrentLimit<=0.02999)
		checkErr( Ivi_GetViReal64EntryFromIndex ( 0, &attrDcCurrentOffset_HighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL));          
*/

 	if ( HighCurrentLimit>=0.000000001 && HighCurrentLimit<=0.000002999 )
	 fMaxVal=0.000002999;	
	else if( HighCurrentLimit>0.000002999 && HighCurrentLimit<=0.00002999)
	 fMaxVal=0.00002999;	     	
	else if( HighCurrentLimit>0.00002999 && HighCurrentLimit<=0.0002999)  		
	 fMaxVal=0.0002999;	     
	else  if(HighCurrentLimit>0.0002999 && HighCurrentLimit<=0.002999)
	 fMaxVal=0.002999;	      	
	else
	 fMaxVal=0.025;   

		
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, 0, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));


Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcCurrentOffset_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:DC:CURR:OFFS?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcCurrentOffset_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
							 
	nCount = sprintf ( cBuf, "SAFE:STEP%s:DC:CURR:OFFS %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcDwellTime_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:TIME:DWEL?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrDcFallTime_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:DC:TIME:FALL?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrFallTime_ReadCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:TIME:FALL?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrCurrentLowRange_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:IR:RANG:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrCurrentLowRange_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
							 
	nCount = sprintf ( cBuf, "SAFE:STEP%s:IR:RANG:LOW %.9f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrCurrRangeUpper_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:RANGE:UPPER?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrCurrHighRange_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:RANGE:UPPER?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrCurrHighRange_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
							 
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:IR:RANGE:UPPER %.9f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrIrCurrRangeAuto_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:IR:RANG:AUTO?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPauseTestTime_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:PAUSE:TIME:TEST?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetGfi_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            const ViConstString cacheValue)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:PRES:GFI?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	checkErr( Ivi_SetValInStringCallback (vi, attributeId, cResult));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetGfi_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViConstString value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
							 
	nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:GFI %s\n", value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
 	checkErr(C19032P_Check_Parameter_Error(vi));    	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetScreen_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViBoolean value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
   if (value==1)
	   {
		nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:SCRE ON\n");	   
	   }
	else 	
		nCount = sprintf ( cBuf, "SOUR:SAFE:PRES:SCRE OFF\n");						 

	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrOscOpenCheck_ReadCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:OSC:LIM:OPEN?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrOscShortCheck_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:OSC:LIM:SHOR?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrOscChannelState_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:OSC:LIM:SHOR?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrOscCrange_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt,rangeSelect;
	
	viCheckErr( Ivi_GetAttributeViInt32( vi, VI_NULL, C19032P_ATTR_OSC_RANGE_SELECT, 0, &rangeSelect));	
							
	if (rangeSelect==0)
		Fmt( cBuf, "%s<SAFE:STEP%s:OSC:CRAN?MAX\n", channelName);		
	
	if (rangeSelect==1)
		Fmt( cBuf, "%s<SAFE:STEP%s:OSC:CRAN?MIN\n", channelName);	
	if (rangeSelect==2)
		Fmt( cBuf, "%s<SAFE:STEP%s:OSC:CRAN?NOW\n", channelName);	

	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrOscCstandard_WriteCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViConstString value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
							 
	nCount = sprintf ( cBuf, "%s", value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrOscCurrentOffset_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViConstString value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
							 
	nCount = sprintf ( cBuf, "%s", value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcDisplay_ReadCallback (ViSession vi,
                                                            ViSession io,
                                                            ViConstString channelName,
                                                            ViAttr attributeId,
                                                            const ViConstString cacheValue)
{

	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SOUR:SAFE:STEP%s:LC:DISP?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
 	checkErr( Ivi_SetValInStringCallback (vi, attributeId, cResult)); 
	
Error:
	return error;

}

static ViStatus _VI_FUNC C19032PAttrLcDisplay_WriteCallback (ViSession vi,
                                                             ViSession io,
                                                             ViConstString channelName,
                                                             ViAttr attributeId,
                                                             ViConstString value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
							 
	nCount = sprintf ( cBuf, "SOUR:SAFE:STEP%s:LC:DISP %s\n",channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcTimeDwell_ReadCallback (ViSession vi,
                                                              ViSession io,
                                                              ViConstString channelName,
                                                              ViAttr attributeId,
                                                              ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:TIME:DWEL?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcTimeDwell_WriteCallback (ViSession vi,
                                                               ViSession io,
                                                               ViConstString channelName,
                                                               ViAttr attributeId,
                                                               ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:TIME:DWEL %.1f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentHigh_ReadCallback (ViSession vi,
                                                                         ViSession io,
                                                                         ViConstString channelName,
                                                                         ViAttr attributeId,
                                                                         ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:POW:SOUR:CURR?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentHigh_WriteCallback (ViSession vi,
                                                                          ViSession io,
                                                                          ViConstString channelName,
                                                                          ViAttr attributeId,
                                                                          ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:POW:SOUR:CURR %.3f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}


static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentHigh_RangeTableCallback (ViSession vi,
                                                                               ViConstString channelName,
                                                                               ViAttr attributeId,
                                                                               IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iPowerMode;
	ViSession			io = Ivi_IOSession (vi);
	ViChar				cScanner[10];
	ViReal64			fMaxVal, fMinVal;
	
 	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));  
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
											

			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouCurrLimit0708RangeTable));  
			else
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouCurrLimit040506RangeTable));	

	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));				
				
Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentLow_RangeTableCallback (ViSession vi,
                                                                              ViConstString channelName,
                                                                              ViAttr attributeId,
                                                                              IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iPowerMode;
	ViSession			io = Ivi_IOSession (vi);
	ViChar				cScanner[10];
	ViReal64			fMaxVal, fMinVal;
	
 	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));  
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
											

			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouCurrLimit0708RangeTable));  
			else
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouCurrLimit040506RangeTable));	
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));				
Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentLow_ReadCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:POW:SOUR:CURR:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourCurrentLow_WriteCallback (ViSession vi,
                                                                         ViSession io,
                                                                         ViConstString channelName,
                                                                         ViAttr attributeId,
                                                                         ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:POW:SOUR:CURR:LOW %.3f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourTargetFreq_ReadCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:POW:SOUR:TFR??\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaHigh_RangeTableCallback (ViSession vi,
                                                                          ViConstString channelName,
                                                                          ViAttr attributeId,
                                                                          IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iPowerMode;
	ViSession			io = Ivi_IOSession (vi);
	ViChar				cScanner[10];
	ViReal64			fMaxVal, fMinVal;
	
  	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr)); 
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
											

			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouVALimit0708RangeTable));  
			else
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouVALimit040506RangeTable));	
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));				
Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaHigh_ReadCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:POW:SOUR:VA:HIGH??\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaHigh_WriteCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:POW:SOUR:VA:HIGH %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaLow_RangeTableCallback (ViSession vi,
                                                                         ViConstString channelName,
                                                                         ViAttr attributeId,
                                                                         IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iPowerMode;
	ViSession			io = Ivi_IOSession (vi);
	ViChar				cScanner[10];
	ViReal64			fMaxVal, fMinVal;
	
  	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr)); 
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
											

			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouVALimit0708RangeTable));  
			else
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouVALimit040506RangeTable));	
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));				
Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaLow_ReadCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:POW:SOUR:VA:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourVaLow_WriteCallback (ViSession vi,
                                                                    ViSession io,
                                                                    ViConstString channelName,
                                                                    ViAttr attributeId,
                                                                    ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:POW:SOUR:VA:LOW %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerHigh_RangeTableCallback (ViSession vi,
                                                                             ViConstString channelName,
                                                                             ViAttr attributeId,
                                                                             IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iPowerMode;
	ViSession			io = Ivi_IOSession (vi);
	ViChar				cScanner[10];
	ViReal64			fMaxVal, fMinVal;
	
  	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr)); 
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
											

			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouVALimit0708RangeTable)); 
			else
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouVALimit040506RangeTable));	
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));				
Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerHigh_ReadCallback (ViSession vi,
                                                                       ViSession io,
                                                                       ViConstString channelName,
                                                                       ViAttr attributeId,
                                                                       ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:POW:SOUR:POW:HIGH?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerHigh_WriteCallback (ViSession vi,
                                                                        ViSession io,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:POW:SOUR:POW:HIGH %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerLow_RangeTableCallback (ViSession vi,
                                                                            ViConstString channelName,
                                                                            ViAttr attributeId,
                                                                            IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViInt32				iPowerMode;
	ViSession			io = Ivi_IOSession (vi);
	ViChar				cScanner[10];
	ViReal64			fMaxVal, fMinVal;
	
 	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr)); 
 	checkErr  ( Ivi_GetAttributeViString(vi, VI_NULL, C19032P_ATTR_OPTION_SCAN_CARD, 0, 9, cScanner));
											

			if (!strncmp (cScanner, "6000-07", 7) || !strncmp (cScanner, "6000-08", 7))
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouVALimit0708RangeTable));	  
			else
				checkErr(Ivi_SetStoredRangeTablePtr (vi, attributeId, &AttrLcPowSouVALimit040506RangeTable));	
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));				
Error:
	*rangeTablePtr = tblPtr;
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerLow_ReadCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:POW:SOUR:POW:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPowerLow_WriteCallback (ViSession vi,
                                                                       ViSession io,
                                                                       ViConstString channelName,
                                                                       ViAttr attributeId,
                                                                       ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:POW:SOUR:POW:LOW %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPfactor_ReadCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:POW:SOUR:PFAC:LOW?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcPowerSourPfactor_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:POW:SOUR:PFAC:LOW %.3f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcUpm_ReadCallback (ViSession vi,
                                                        ViSession io,
                                                        ViConstString channelName,
                                                        ViAttr attributeId,
                                                        ViBoolean *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:UPM?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcUpm_WriteCallback (ViSession vi,
                                                         ViSession io,
                                                         ViConstString channelName,
                                                         ViAttr attributeId,
                                                         ViBoolean value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	
	if(value)
		nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:UPM ON\n", channelName);
	else
		nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:UPM OFF\n", channelName); 
		
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffset_ReadCallback (ViSession vi,
                                                                  ViSession io,
                                                                  ViConstString channelName,
                                                                  ViAttr attributeId,
                                                                  ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:CURR:OFFS?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffset_WriteCallback (ViSession vi,
                                                                   ViSession io,
                                                                   ViConstString channelName,
                                                                   ViAttr attributeId,
                                                                   ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:CURR:OFFS %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffsetLac_ReadCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:CURR:OFFS:LAC?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffsetLac_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:CURR:OFFS:LAC %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffsetLdc_ReadCallback (ViSession vi,
                                                                     ViSession io,
                                                                     ViConstString channelName,
                                                                     ViAttr attributeId,
                                                                     ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:CURR:OFFS:LDC?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcCurrentOffsetLdc_WriteCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViReal64 value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE];
	ViInt32		retCnt, nCount;
	
	nCount = sprintf ( cBuf, "SAFE:STEP%s:LC:CURR:OFFS:LDC %.10f\n", channelName, value);
	viCheckErr( viWrite (io, cBuf, nCount, &retCnt));
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcAcHighLimit_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:LAC?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrLcDcHighLimit_ReadCallback (ViSession vi,
                                                                ViSession io,
                                                                ViConstString channelName,
                                                                ViAttr attributeId,
                                                                ViReal64 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:STEP%s:LC:LDC?\n", channelName);
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atof(cResult);
	
Error:
	return error;
}

static ViStatus _VI_FUNC C19032PAttrPresetRampJudgement_ReadCallback (ViSession vi,
                                                                      ViSession io,
                                                                      ViConstString channelName,
                                                                      ViAttr attributeId,
                                                                      ViInt32 *value)
{
	ViStatus	error = VI_SUCCESS;
	ViChar		cBuf[BUFFER_SIZE], cResult[BUFFER_SIZE];
	ViInt32		retCnt;
	
	Fmt       ( cBuf, "%s<SAFE:PRES:RJUD?\n");
	viCheckErr( viWrite (io, cBuf, NumFmtdBytes(), &retCnt));
	viCheckErr( viRead  (io, cResult, BUFFER_SIZE-1, &retCnt));
	cResult[retCnt] = 0;
	*value = atoi(cResult);
	
Error:
	return error;
}


static ViStatus _VI_FUNC C19032PAttrAcCurrentOffset_RangeTableCallback (ViSession vi,
                                                                        ViConstString channelName,
                                                                        ViAttr attributeId,
                                                                        IviRangeTablePtr *rangeTablePtr)
{
	ViStatus			error = VI_SUCCESS;
	IviRangeTablePtr	tblPtr = VI_NULL;
	ViReal64			fMaxVal, fMinVal,WACHighCurrentLimit;
	ViInt32				nModel,Result,Result2;
	
	
	checkErr( Ivi_GetAttributeViReal64 (vi, channelName, C19032P_ATTR_AC_LIMIT_HIGH, 0, &WACHighCurrentLimit)); 
	

	if ( WACHighCurrentLimit<= 0.002999 && WACHighCurrentLimit>=0.001)
	fMaxVal=0.002999;//	checkErr( Ivi_GetViReal64EntryFromValue ( 0, &attrAcCurrentOffset_LowRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL,VI_NULL)); 
	else if(WACHighCurrentLimit >0.02999)	    
		fMaxVal=0.1;//	checkErr( Ivi_GetViReal64EntryFromValue ( 0, &attrAcCurrentOffset_HighRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL, VI_NULL,VI_NULL)); 
	else  
		fMaxVal=0.02999;//	checkErr( Ivi_GetViReal64EntryFromValue ( 0, &attrAcCurrentOffset_MidRangeTable, &fMinVal, &fMaxVal, VI_NULL, VI_NULL,VI_NULL, VI_NULL));  
  
	checkErr( Ivi_GetStoredRangeTablePtr (vi, attributeId, &tblPtr));  
	checkErr( Ivi_SetRangeTableEntry (tblPtr, 0, 0, fMaxVal, 0.0, VI_NULL, VI_NULL));
	checkErr( Ivi_SetStoredRangeTablePtr (vi, attributeId, tblPtr));


Error:
	*rangeTablePtr = tblPtr;
	return error;
}

/*****************************************************************************
 * Function: C19032P_InitAttributes                                                  
 * Purpose:  This function adds attributes to the IVI session, initializes   
 *           instrument attributes, and sets attribute invalidation          
 *           dependencies.                                                   
 *****************************************************************************/
static ViStatus C19032P_InitAttributes (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     flags = 0;
    
    IviRangeTablePtr    attrGbCurrentRangeTable=VI_NULL,
    					attrGbLimitHighRangeTable=VI_NULL,
    					attrGbLimitLowRangeTable=VI_NULL,
    					attrAcLimitLowRangeTable=VI_NULL,
    					attrDcLimitLowRangeTable=VI_NULL,
    					attrIrLimitHighRangeTable=VI_NULL,
    					attrLcLimitHighRangeTable=VI_NULL,
    					attrLcLimitLowRangeTable=VI_NULL,
    					attrLcPowerLimitHighRangeTable=VI_NULL,
    					attrLcPowerLimitLowRangeTable=VI_NULL,
    					attrLcAcDcHighRangeTable=VI_NULL,
    					attrAcLimitHighRangeTable=VI_NULL,//ck add
    					attrDcLimitHighRangeTable=VI_NULL;//ck add

    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrGbCurrentRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrGbCurrentRangeTable, 0, 1, 30, 0.0, VI_NULL, 0));
  
    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrGbLimitHighRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrGbLimitHighRangeTable, 0, 0.0001, 0.51, 0.0, VI_NULL, 0));
    				
    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrGbLimitLowRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrGbLimitLowRangeTable, 0, 0.0, 0.51, 0.0, VI_NULL, 0));

    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrAcLimitLowRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrAcLimitLowRangeTable, 0, 0.0, 0.04, 0.0, VI_NULL, 0));
	
    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrDcLimitLowRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrDcLimitLowRangeTable, 0, 0.0, 0.012, 0.0, VI_NULL, 0));
	
    checkErr( Ivi_RangeTableNew (vi, 3, 1, VI_TRUE, VI_TRUE, &attrIrLimitHighRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrIrLimitHighRangeTable, 0, 0.0, 50000e+6, 0.0, VI_NULL, 0));

    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrLcLimitHighRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrLcLimitHighRangeTable, 0, 0.0001e-3, 6e-3, 0.0, VI_NULL, 0));
	
    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrLcLimitLowRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrLcLimitLowRangeTable, 0, 0.0, 6e-3, 0.0, VI_NULL, 0));
 
    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrLcPowerLimitHighRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrLcPowerLimitHighRangeTable, 0, 0.0, 300, 0.0, VI_NULL, 0));
	
    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrLcPowerLimitLowRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrLcPowerLimitLowRangeTable, 0, 0.0, 300, 0.0, VI_NULL, 0));
	
    checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrLcAcDcHighRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrLcAcDcHighRangeTable, 0, 0.0, 0.0, 0.0, VI_NULL, 0));
	
	//CK add
	checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrAcLimitHighRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrAcLimitHighRangeTable, 0, 0.0, 0.0, 0.0, VI_NULL, 0));
	
	checkErr( Ivi_RangeTableNew (vi, 2, 1, VI_TRUE, VI_TRUE, &attrDcLimitHighRangeTable));	
	checkErr( Ivi_SetRangeTableEntry (attrDcLimitHighRangeTable, 0, 0.0, 0.0, 0.0, VI_NULL, 0));
	
        /*- Initialize instrument attributes --------------------------------*/            

    checkErr( Ivi_SetAttributeViInt32 (vi, "", C19032P_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION,
                                       0, C19032P_MAJOR_VERSION));
    checkErr( Ivi_SetAttributeViInt32 (vi, "", C19032P_ATTR_SPECIFIC_DRIVER_MINOR_VERSION,
                                       0, C19032P_MINOR_VERSION));
    checkErr( Ivi_SetAttrReadCallbackViString (vi, C19032P_ATTR_SPECIFIC_DRIVER_REVISION,
                                               C19032PAttrDriverRevision_ReadCallback));
    checkErr( Ivi_SetAttributeViInt32 (vi, "", C19032P_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION,
                                       0, C19032P_CLASS_SPEC_MAJOR_VERSION));
    checkErr( Ivi_SetAttributeViInt32 (vi, "", C19032P_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION,
                                       0, C19032P_CLASS_SPEC_MINOR_VERSION));
    checkErr( Ivi_SetAttributeViString (vi, "", C19032P_ATTR_IO_SESSION_TYPE,
                                        0, C19032P_IO_SESSION_TYPE));
    checkErr( Ivi_SetAttributeViString (vi, "", C19032P_ATTR_SUPPORTED_INSTRUMENT_MODELS,
                                        0, C19032P_SUPPORTED_INSTRUMENT_MODELS));


    checkErr( Ivi_SetAttrReadCallbackViString (vi, C19032P_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                               C19032PAttrFirmwareRevision_ReadCallback));
	checkErr( Ivi_GetAttributeFlags (vi, C19032P_ATTR_INSTRUMENT_MANUFACTURER, &flags));
	checkErr( Ivi_SetAttributeFlags (vi, C19032P_ATTR_INSTRUMENT_MANUFACTURER, 
	                                 flags | IVI_VAL_USE_CALLBACKS_FOR_SIMULATION));
	checkErr( Ivi_SetAttrReadCallbackViString (vi, C19032P_ATTR_INSTRUMENT_MANUFACTURER,
	                                           C19032PAttrInstrumentManufacturer_ReadCallback));

	checkErr( Ivi_GetAttributeFlags (vi, C19032P_ATTR_INSTRUMENT_MODEL, &flags));
	checkErr( Ivi_SetAttributeFlags (vi, C19032P_ATTR_INSTRUMENT_MODEL, 
	                                 flags | IVI_VAL_USE_CALLBACKS_FOR_SIMULATION));
	checkErr( Ivi_SetAttrReadCallbackViString (vi, C19032P_ATTR_INSTRUMENT_MODEL,
	                                           C19032PAttrInstrumentModel_ReadCallback));

    checkErr( Ivi_SetAttributeViString (vi, "", C19032P_ATTR_SPECIFIC_DRIVER_VENDOR,
                                        0, C19032P_DRIVER_VENDOR));
    checkErr( Ivi_SetAttributeViString (vi, "", C19032P_ATTR_SPECIFIC_DRIVER_DESCRIPTION,
                                        0, C19032P_DRIVER_DESCRIPTION));
 
 
	checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, IVI_ATTR_SUPPORTS_WR_BUF_OPER_MODE, 
	                                     0, VI_FALSE));


	checkErr( Ivi_SetAttributeViString (vi, "", C19032P_ATTR_GROUP_CAPABILITIES, 0,
	                                    "None"));
    
        /*- Add instrument-specific attributes ------------------------------*/            
	
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_ID_QUERY_RESPONSE,
	                                    "C19032P_ATTR_ID_QUERY_RESPONSE",
	                                    "Chroma,19032,0,1.07",
	                                    IVI_VAL_NOT_USER_WRITABLE,
	                                    C19032PAttrIdQueryResponse_ReadCallback,
	                                    VI_NULL));
	                                           
    checkErr( Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OPC_TIMEOUT,
                                       "C19032P_ATTR_OPC_TIMEOUT",
                                       5000, IVI_VAL_HIDDEN | IVI_VAL_DONT_CHECK_STATUS,
                                       VI_NULL, VI_NULL, VI_NULL)); 
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_MEM_DEL,
	                                   "C19032P_ATTR_MEM_DEL", 0,
	                                   IVI_VAL_NEVER_CACHE, VI_NULL,
	                                   C19032PAttrMemDel_WriteCallback,
	                                   &ChrMemIndexRangeTable));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_MEM_NAME_DEFINE,
	                                    "C19032P_ATTR_MEM_NAME_DEFINE", 0,
	                                    IVI_VAL_MULTI_CHANNEL | IVI_VAL_NEVER_CACHE,
	                                    VI_NULL,
	                                    C19032PAttrMemNameDefine_WriteCallback));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_SYS_VERSION,
	                                    "C19032P_ATTR_SYS_VERSION", 0, 0,
	                                    C19032PAttrSysVersion_ReadCallback, VI_NULL));
	checkErr (Ivi_AddAttributeViBoolean (vi, C19032P_ATTR_TEST_STATE,
	                                     "C19032P_ATTR_TEST_STATE", 0, 0, VI_NULL,
	                                     C19032PAttrTestState_WriteCallback));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OFFSET_STATE,
	                                   "C19032P_ATTR_OFFSET_STATE", 0, 0, VI_NULL,
	                                   C19032PAttrOffsetState_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_RUN_COMPLETE_STATUS,
	                                   "C19032P_ATTR_RUN_COMPLETE_STATUS", 0,
	                                   IVI_VAL_NEVER_CACHE,
	                                   C19032PAttrRunCompleteStatus_ReadCallback,
	                                   VI_NULL, VI_NULL));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_AUTO_RESULT_STATE,
	                                   "C19032P_ATTR_AUTO_RESULT_STATE", 0, 0,
	                                   C19032PAttrAutoResultState_ReadCallback,
	                                   C19032PAttrAutoResultState_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_DELETE_STEP,
	                                   "C19032P_ATTR_DELETE_STEP", 0,
	                                   IVI_VAL_NEVER_CACHE, VI_NULL,
	                                   C19032PAttrDeleteStep_WriteCallback,
	                                   &attrDeleteStepRangeTable));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_STEP_SETTING,
	                                    "C19032P_ATTR_STEP_SETTING", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrStepSetting_ReadCallback, VI_NULL));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_GB_CURRENT,
	                                    "C19032P_ATTR_GB_CURRENT", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrGbCurrent_ReadCallback,
	                                    C19032PAttrGbCurrent_WriteCallback, 
	                                    attrGbCurrentRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_GB_CURRENT,
	                                         C19032PAttrGbCurrent_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_GB_LIMIT_HIGH,
	                                    "C19032P_ATTR_GB_LIMIT_HIGH", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrGbLimitHigh_ReadCallback,
	                                    C19032PAttrGbLimitHigh_WriteCallback, 
	                                    attrGbLimitHighRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_GB_LIMIT_HIGH,
	                                         C19032PAttrGbLimitHigh_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_GB_LIMIT_LOW,
	                                    "C19032P_ATTR_GB_LIMIT_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrGbLimitLow_ReadCallback,
	                                    C19032PAttrGbLimitLow_WriteCallback,
	                                    attrGbLimitLowRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_GB_LIMIT_LOW,
	                                         C19032PAttrGbLimitLow_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_GB_TEST_TIME,
	                                    "C19032P_ATTR_GB_TEST_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrGbTestTime_ReadCallback,
	                                    C19032PAttrGbTestTime_WriteCallback,
	                                    &ChrTestTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_GB_TWINPORT_STATE,
	                                   "C19032P_ATTR_GB_TWINPORT_STATE", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrGbTwinportState_ReadCallback,
	                                   C19032PAttrGbTwinportState_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_GB_CHANNEL_STATE,
	                                   "C19032P_ATTR_GB_CHANNEL_STATE", -1,
	                                   IVI_VAL_MULTI_CHANNEL, VI_NULL, VI_NULL,
	                                   VI_NULL));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_AC_CHANNEL_STATE,
	                                   "C19032P_ATTR_AC_CHANNEL_STATE", -1,
	                                   IVI_VAL_MULTI_CHANNEL, VI_NULL, VI_NULL,
	                                   VI_NULL));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                   "C19032P_ATTR_STEP_MODE_TYPE", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrStepModeType_ReadCallback, VI_NULL,
	                                   &attrStepModeTypeRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_VOLTAGE,
	                                    "C19032P_ATTR_AC_VOLTAGE", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcVoltage_ReadCallback,
	                                    C19032PAttrAcVoltage_WriteCallback,
	                                    &attrAcVoltageRangeTable, 0));
	checkErr (Ivi_AddAttributeViBoolean (vi, C19032P_ATTR_LLO_STATE,
	                                     "C19032P_ATTR_LLO_STATE", 0,
	                                     IVI_VAL_NEVER_CACHE, VI_NULL,
	                                     C19032PAttrLloState_WriteCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_LIMIT_HIGH,
	                                    "C19032P_ATTR_AC_LIMIT_HIGH", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcLimitHigh_ReadCallback,
	                                    C19032PAttrAcLimitHigh_WriteCallback,
	                                    &ChrAcLimitHighRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_AC_LIMIT_HIGH,
	                                         C19032PAttrAcLimitHigh_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_LIMIT_LOW,
	                                    "C19032P_ATTR_AC_LIMIT_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcLimitLow_ReadCallback,
	                                    C19032PAttrAcLimitLow_WriteCallback,
	                                    &ChrAcLimitLowRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_AC_LIMIT_LOW,
	                                         C19032PAttrAcLimitLow_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_ARC_LIMIT,
	                                    "C19032P_ATTR_AC_ARC_LIMIT", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcArcLimit_ReadCallback,
	                                    C19032PAttrAcArcLimit_WriteCallback,
	                                    &attrAcArcLimitRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_AC_ARC_FILTER,
	                                   "C19032P_ATTR_AC_ARC_FILTER", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrAcArcFilter_ReadCallback,
	                                   C19032PAttrAcArcFilter_WriteCallback,
	                                   &ChrArcFilterRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_RAMP_TIME,
	                                    "C19032P_ATTR_AC_RAMP_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcRampTime_ReadCallback,
	                                    C19032PAttrAcRampTime_WriteCallback,
	                                    &ChrRampFallTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_TEST_TIME,
	                                    "C19032P_ATTR_AC_TEST_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcTestTime_ReadCallback,
	                                    C19032PAttrAcTestTime_WriteCallback,
	                                    &ChrTestTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_VOLTAGE,
	                                    "C19032P_ATTR_DC_VOLTAGE", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcVoltage_ReadCallback,
	                                    C19032PAttrDcVoltage_WriteCallback,
	                                    &attrDcVoltageRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_LIMIT_HIGH,
	                                    "C19032P_ATTR_DC_LIMIT_HIGH", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcLimitHigh_ReadCallback,
	                                    C19032PAttrDcLimitHigh_WriteCallback,
	                                    &attrDCModeDcLimitHighRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_LIMIT_LOW,
	                                    "C19032P_ATTR_DC_LIMIT_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcLimitLow_ReadCallback,
	                                    C19032PAttrDcLimitLow_WriteCallback,
	                                    &attrDCModeDcLimitLowRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_ARC_LIMIT,
	                                    "C19032P_ATTR_DC_ARC_LIMIT", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcArcLimit_ReadCallback,
	                                    C19032PAttrDcArcLimit_WriteCallback,
	                                    &attrDcArcLimitRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_DC_ARC_FILTER,
	                                   "C19032P_ATTR_DC_ARC_FILTER", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrDcArcFilter_ReadCallback,
	                                   C19032PAttrDcArcFilter_WriteCallback,
	                                   &ChrArcFilterRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_RAMP_TIME,
	                                    "C19032P_ATTR_DC_RAMP_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcRampTime_ReadCallback,
	                                    C19032PAttrDcRampTime_WriteCallback,
	                                    &ChrRampFallTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_TEST_TIME,
	                                    "C19032P_ATTR_DC_TEST_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcTestTime_ReadCallback,
	                                    C19032PAttrDcTestTime_WriteCallback,
	                                    &ChrTestTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_DC_CHANNEL_STATE,
	                                   "C19032P_ATTR_DC_CHANNEL_STATE", -1,
	                                   IVI_VAL_MULTI_CHANNEL, VI_NULL, VI_NULL,
	                                   VI_NULL));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_VOLTAGE,
	                                    "C19032P_ATTR_IR_VOLTAGE", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrIrVoltage_ReadCallback,
	                                    C19032PAttrIrVoltage_WriteCallback,
	                                    &attrIrVoltageRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_LIMIT_HIGH,
	                                    "C19032P_ATTR_IR_LIMIT_HIGH", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrIrLimitHigh_ReadCallback,
	                                    C19032PAttrIrLimitHigh_WriteCallback,
	                                    attrIrLimitHighRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_IR_LIMIT_HIGH,
	                                         C19032PAttrIrLimitHigh_RangeTableCallback));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_STEP_COUNT,
	                                   "C19032P_ATTR_STEP_COUNT", 0, 0, VI_NULL,
	                                   VI_NULL, &attrStepCountRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_LIMIT_LOW,
	                                    "C19032P_ATTR_IR_LIMIT_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrIrLimitLow_ReadCallback,
	                                    C19032PAttrIrLimitLow_WriteCallback,
	                                    &attrIrLimitLowRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_RAMP_TIME,
	                                    "C19032P_ATTR_IR_RAMP_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrIrRampTime_ReadCallback,
	                                    C19032PAttrIrRampTime_WriteCallback,
	                                    &ChrRampFallTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_TEST_TIME,
	                                    "C19032P_ATTR_IR_TEST_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrIrTestTime_ReadCallback,
	                                    C19032PAttrIrTestTime_WriteCallback,
	                                    &ChrTestTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_IR_CHANNEL_STATE,
	                                   "C19032P_ATTR_IR_CHANNEL_STATE", -1,
	                                   IVI_VAL_MULTI_CHANNEL, VI_NULL, VI_NULL,
	                                   VI_NULL));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_LC_CHANNEL_STATE,
	                                   "C19032P_ATTR_LC_CHANNEL_STATE", -1,
	                                   IVI_VAL_MULTI_CHANNEL, VI_NULL, VI_NULL,
	                                   VI_NULL));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_LIMIT_LOW,
	                                    "C19032P_ATTR_LC_LIMIT_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcLimitLow_ReadCallback,
	                                    C19032PAttrLcLimitLow_WriteCallback, 
	                                    attrLcLimitLowRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_LC_LIMIT_LOW,
	                                         C19032PAttrLcLimitLow_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_TEST_TIME,
	                                    "C19032P_ATTR_LC_TEST_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcTestTime_ReadCallback,
	                                    C19032PAttrLcTestTime_WriteCallback,
	                                    &ChrTestTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_LC_POWER_MODE,
	                                   "C19032P_ATTR_LC_POWER_MODE", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrLcPowerMode_ReadCallback,
	                                   C19032PAttrLcPowerMode_WriteCallback,
	                                   &attrLcPowerModeRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_LC_DEVICE_TYPE,
	                                   "C19032P_ATTR_LC_DEVICE_TYPE", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrLcDeviceType_ReadCallback,
	                                   C19032PAttrLcDeviceType_WriteCallback,
	                                   &attrLcDeviceTypeRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_LC_LINE_INPUT,
	                                   "C19032P_ATTR_LC_LINE_INPUT", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrLcLineInput_ReadCallback,
	                                   C19032PAttrLcLineInput_WriteCallback,
	                                   &attrLcLineInputRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_LIMIT_HIGH,
	                                    "C19032P_ATTR_LC_LIMIT_HIGH", 0.0001,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcLimitHigh_ReadCallback,
	                                    C19032PAttrLcLimitHigh_WriteCallback,
	                                    attrLcLimitHighRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_LC_LIMIT_HIGH,
	                                         C19032PAttrLcLimitHigh_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_LIMIT_HIGH,
	                                    "C19032P_ATTR_LC_POWER_LIMIT_HIGH", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerLimitHigh_ReadCallback,
	                                    C19032PAttrLcPowerLimitHigh_WriteCallback,
	                                    attrLcPowerLimitHighRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_LC_POWER_LIMIT_HIGH,
	                                         C19032PAttrLcPowerLimitHigh_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_LIMIT_LOW,
	                                    "C19032P_ATTR_LC_POWER_LIMIT_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerLimitLow_ReadCallback,
	                                    C19032PAttrLcPowerLimitLow_WriteCallback,
	                                    attrLcPowerLimitLowRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_LC_POWER_LIMIT_LOW,
	                                         C19032PAttrLcPowerLimitLow_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT,
	                                    "C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSimuTargetVolt_ReadCallback,
	                                    C19032PAttrLcPowerSimuTargetVolt_WriteCallback,
	                                    &attrLcPowerSimuTargetVoltRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_PRESET_PASS_HOLD_TIME,
	                                    "C19032P_ATTR_PRESET_PASS_HOLD_TIME", 0, 0,
	                                    C19032PAttrPresetPassHoldTime_ReadCallback,
	                                    C19032PAttrPresetPassHoldTime_WriteCallback,
	                                    &attrPresetPassHoldTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_PRESET_STEP_HOLD_TIME,
	                                    "C19032P_ATTR_PRESET_STEP_HOLD_TIME", 0, 0,
	                                    C19032PAttrPresetStepHoldTime_ReadCallback,
	                                    C19032PAttrPresetStepHoldTime_WriteCallback,
	                                    &attrPresetStepHoldTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_PRESET_GB_VOLTAGE,
	                                    "C19032P_ATTR_PRESET_GB_VOLTAGE", 0, 0,
	                                    C19032PAttrPresetGbVoltage_ReadCallback,
	                                    C19032PAttrPresetGbVoltage_WriteCallback,
	                                    &attrPresetGbVoltageRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_PRESET_AC_FREQUENCY,
	                                   "C19032P_ATTR_PRESET_AC_FREQUENCY", 0, 0,
	                                   C19032PAttrPresetAcFrequency_ReadCallback,
	                                   C19032PAttrPresetAcFrequency_WriteCallback,
	                                   &attrPresetAcFrequencyRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_PRESET_AUTO_RANGE,
	                                   "C19032P_ATTR_PRESET_AUTO_RANGE", 0, 0,
	                                   C19032PAttrPresetAutoRange_ReadCallback,
	                                   C19032PAttrPresetAutoRange_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_PRESET_SOFT_AGC_STATE,
	                                   "C19032P_ATTR_PRESET_SOFT_AGC_STATE", 0, 0,
	                                   C19032PAttrPresetSoftAgcState_ReadCallback,
	                                   C19032PAttrPresetSoftAgcState_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_PRESET_PART_NUMBER,
	                                    "C19032P_ATTR_PRESET_PART_NUMBER", 0, 0,
	                                    C19032PAttrPresetPartNumber_ReadCallback,
	                                    C19032PAttrPresetPartNumber_WriteCallback));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_PRESET_LOT_NUMBER,
	                                    "C19032P_ATTR_PRESET_LOT_NUMBER", 0, 0,
	                                    C19032PAttrPresetLotNumber_ReadCallback,
	                                    C19032PAttrPresetLotNumber_WriteCallback));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_PRESET_SERIAL_NUMBER,
	                                    "C19032P_ATTR_PRESET_SERIAL_NUMBER", 0, 0,
	                                    C19032PAttrPresetSerialNumber_ReadCallback,
	                                    C19032PAttrPresetSerialNumber_WriteCallback));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_LC_METER_METHOD,
	                                   "C19032P_ATTR_LC_METER_METHOD", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrLcMeterMethod_ReadCallback,
	                                   C19032PAttrLcMeterMethod_WriteCallback,
	                                   &attrLcMeterMethodRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_LC_GR_SWITCH_STATE,
	                                   "C19032P_ATTR_LC_GR_SWITCH_STATE", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrLcGrSwitchState_ReadCallback,
	                                   C19032PAttrLcGrSwitchState_WriteCallback,
	                                   &attrLcGrSwitchStateRangeTable));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_QUERY_MEM_INDEX,
	                                    "C19032P_ATTR_QUERY_MEM_INDEX", 0, 0, VI_NULL,
	                                    VI_NULL));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_QUERY_MEM_INDEX_NUM,
	                                   "C19032P_ATTR_QUERY_MEM_INDEX_NUM", 0,
	                                   IVI_VAL_NEVER_CACHE,
	                                   C19032PAttrQueryMemIndexNum_ReadCallback,
	                                   VI_NULL, VI_NULL));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_PAUSE_MESSAGE_STRING,
	                                    "C19032P_ATTR_PAUSE_MESSAGE_STRING", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrPauseMessageString_ReadCallback,
	                                    C19032PAttrPauseMessageString_WriteCallback));
	checkErr (Ivi_AddAttributeViBoolean (vi, C19032P_ATTR_PAUSE_SIGNAL_STATE,
	                                     "C19032P_ATTR_PAUSE_SIGNAL_STATE", 0,
	                                     IVI_VAL_MULTI_CHANNEL,
	                                     C19032PAttrPauseSignalState_ReadCallback,
	                                     C19032PAttrPauseSignalState_WriteCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_PRESET_GB_FREQUENCY,
	                                    "C19032P_ATTR_PRESET_GB_FREQUENCY", 0, 0,
	                                    C19032PAttrPresetGbFrequency_ReadCallback,
	                                    C19032PAttrPresetGbFrequency_WriteCallback,
	                                    &attrPresetGbFrequencyRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_PRESET_IEC_STATE,
	                                   "C19032P_ATTR_PRESET_IEC_STATE", 0, 0,
	                                   C19032PAttrPresetIecState_ReadCallback,
	                                   C19032PAttrPresetIecState_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_PRESET_START_WAIT,
	                                    "C19032P_ATTR_PRESET_START_WAIT", 0, 0,
	                                    C19032PAttrPresetStartWait_ReadCallback,
	                                    C19032PAttrPresetStartWait_WriteCallback,
	                                    &attrPresetStartWaitRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_DWELL_TIME,
	                                    "C19032P_ATTR_DC_DWELL_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcDwellTime_ReadCallback,
	                                    C19032PAttrDcDwellTime_WriteCallback,
	                                    &attrDcDwellTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_OSC_OPEN_CHECK,
	                                    "C19032P_ATTR_OSC_OPEN_CHECK", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrOscOpenCheck_ReadCallback,
	                                    C19032PAttrOscOpenCheck_WriteCallback,
	                                    &attrOscOpenCheckRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_OSC_SHORT_CHECK,
	                                    "C19032P_ATTR_OSC_SHORT_CHECK", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrOscShortCheck_ReadCallback,
	                                    C19032PAttrOscShortCheck_WriteCallback,
	                                    &attrOscShortCheckRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OPTION_SCAN_TYPE,
	                                   "C19032P_ATTR_OPTION_SCAN_TYPE", 0,
	                                   IVI_VAL_MULTI_CHANNEL, VI_NULL, VI_NULL,
	                                   &attrOptionScanTypeRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OSC_CHANNEL_STATE,
	                                   "C19032P_ATTR_OSC_CHANNEL_STATE", -1,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrOscChannelState_ReadCallback,
	                                   VI_NULL, VI_NULL));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_STANDARD_CAPACITANCE,
	                                   "C19032P_ATTR_STANDARD_CAPACITANCE", 0,
	                                   IVI_VAL_NEVER_CACHE, VI_NULL,
	                                   C19032PAttrStandardCapacitance_WriteCallback,
	                                   VI_NULL));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OPTION_GB_RATE,
	                                   "C19032P_ATTR_OPTION_GB_RATE", 0, 0,
	                                   C19032PAttrOptionGbRate_ReadCallback, VI_NULL,
	                                   &attrOptionGbRateRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_FWVERSION,
	                                   "C19032P_ATTR_FWVERSION", 0, 0, VI_NULL,
	                                   VI_NULL, VI_NULL));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_OPTION_SCAN_CARD,
	                                    "C19032P_ATTR_OPTION_SCAN_CARD", 0, 0,
	                                    C19032PAttrOptionScanCard_ReadCallback,
	                                    VI_NULL));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OPTION_GB_FLOATING,
	                                   "C19032P_ATTR_OPTION_GB_FLOATING", 0, 0,
	                                   C19032PAttrOptionGbFloating_ReadCallback,
	                                   VI_NULL, &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OPTION_SCAN_NUM,
	                                   "C19032P_ATTR_OPTION_SCAN_NUM", 0, 0,
	                                   C19032PAttrOptionScanNum_ReadCallback, VI_NULL,
	                                   &attrOptionScanNumRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_PAUSE_TEST_TIME,
	                                    "C19032P_ATTR_PAUSE_TEST_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrPauseTestTime_ReadCallback,
	                                    C19032PAttrPauseTestTime_WriteCallback,
	                                    &ChrTestTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_TARGET_VOLT,
	                                    "C19032P_ATTR_LC_POWER_SOUR_TARGET_VOLT", 0,
	                                    IVI_VAL_MULTI_CHANNEL, VI_NULL,
	                                    C19032PAttrLcPowerSourTargetVolt_WriteCallback,
	                                    &attrLcPowerSimuTargetVoltRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_TARGET_FREQ,
	                                    "C19032P_ATTR_LC_POWER_SOUR_TARGET_FREQ", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSourTargetFreq_ReadCallback,
	                                    C19032PAttrLcPowerSourTargetFreq_WriteCallback,
	                                    &attrLcPowerSourTargetFreqRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OPTION_USE_SOURCE,
	                                   "C19032P_ATTR_OPTION_USE_SOURCE", 0, 0,
	                                   C19032PAttrOptionUseSource_ReadCallback,
	                                   VI_NULL, &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_FALL_TIME,
	                                    "C19032P_ATTR_AC_FALL_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcFallTime_ReadCallback,
	                                    C19032PAttrAcFallTime_WriteCallback,
	                                    &ChrRampFallTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_FALL_TIME,
	                                    "C19032P_ATTR_DC_FALL_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcFallTime_ReadCallback,
	                                    C19032PAttrDcFallTime_WriteCallback,
	                                    &ChrRampFallTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_FALL_TIME,
	                                    "C19032P_ATTR_IR_FALL_TIME", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrIrFallTime_ReadCallback,
	                                    C19032PAttrIrFallTime_WriteCallback,
	                                    &ChrRampFallTimeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_CURR_RANGE_UPPER,
	                                    "C19032P_ATTR_IR_CURR_RANGE_UPPER", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrIrCurrRangeUpper_ReadCallback,
	                                    C19032PAttrIrCurrRangeUpper_WriteCallback,
	                                    &attrIrCurrRangeUpperRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_AC_HIGH_LIMIT,
	                                    "C19032P_ATTR_LC_AC_HIGH_LIMIT", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcAcHighLimit_ReadCallback,
	                                    C19032PAttrLcAcHighLimit_WriteCallback,
	                                    attrLcAcDcHighRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_LC_AC_HIGH_LIMIT,
	                                         C19032PAttrLcAcHighLimit_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_DC_HIGH_LIMIT,
	                                    "C19032P_ATTR_LC_DC_HIGH_LIMIT", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcDcHighLimit_ReadCallback,
	                                    C19032PAttrLcDcHighLimit_WriteCallback,
	                                    attrLcAcDcHighRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_PRESET_RAMP_JUDGEMENT,
	                                   "C19032P_ATTR_PRESET_RAMP_JUDGEMENT", 0, 0,
	                                   C19032PAttrPresetRampJudgement_ReadCallback,
	                                   C19032PAttrPresetRampJudgement_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_REMOTE_START_KEY,
	                                   "C19032P_ATTR_REMOTE_START_KEY", 0, 0, VI_NULL,
	                                   C19032PAttrRemoteStartKey_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_IR_CURR_RANGE_AUTO,
	                                   "C19032P_ATTR_IR_CURR_RANGE_AUTO", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrIrCurrRangeAuto_ReadCallback,
	                                   C19032PAttrIrCurrRangeAuto_WriteCallback,
	                                   &ChrOnOffRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_MODEL, "C19032P_ATTR_MODEL",
	                                   C19032P_VAL_MODEL_19032P, 0, VI_NULL, VI_NULL,
	                                   &attrModelRangeTable));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_QUERY_MEM_MAX_CAPACITY,
	                                   "C19032P_ATTR_QUERY_MEM_MAX_CAPACITY", 0,
	                                   IVI_VAL_NEVER_CACHE,
	                                   C19032PAttrQueryMemMaxCapacity_ReadCallback,
	                                   VI_NULL, VI_NULL));
	checkErr (Ivi_AddAttributeViBoolean (vi, C19032P_ATTR_FUNC_MODE,
	                                     "C19032P_ATTR_FUNC_MODE", 0, 0,
	                                     C19032PAttrFuncMode_ReadCallback,
	                                     C19032PAttrFuncMode_WriteCallback));
	checkErr (Ivi_AddAttributeViBoolean (vi, C19032P_ATTR_AUTO_REPORT_SAVED,
	                                     "C19032P_ATTR_AUTO_REPORT_SAVED", 0, 0,
	                                     C19032PAttrAutoReportSaved_ReadCallback,
	                                     C19032PAttrAutoReportSaved_WriteCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_LIMIT_REAL,
	                                    "C19032P_ATTR_AC_LIMIT_REAL", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcLimitReal_ReadCallback,
	                                    C19032PAttrAcLimitReal_WriteCallback,
	                                    &attrAcLimitRealRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_GB_CURRENT_OFFSET,
	                                    "C19032P_ATTR_GB_CURRENT_OFFSET", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrGbCurrentOffset_ReadCallback,
	                                    C19032PAttrGbCurrentOffset_WriteCallback,
	                                    &attrGbCurrentOffsetRangeTable, 0));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_AC_FREQUENCY,
	                                   "C19032P_ATTR_AC_FREQUENCY", 0,
	                                   IVI_VAL_MULTI_CHANNEL,
	                                   C19032PAttrAcFrequency_ReadCallback,
	                                   C19032PAttrAcFrequency_WriteCallback,
	                                   &attrAcFrequencyRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_AC_CURRENT_OFFSET,
	                                    "C19032P_ATTR_AC_CURRENT_OFFSET", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrAcCurrentOffset_ReadCallback,
	                                    C19032PAttrAcCurrentOffset_WriteCallback,
	                                    &attrAcCurrentOffset_LowRangeTable, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_AC_CURRENT_OFFSET,
	                                         C19032PAttrAcCurrentOffset_RangeTableCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_DC_CURRENT_OFFSET,
	                                    "C19032P_ATTR_DC_CURRENT_OFFSET", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrDcCurrentOffset_ReadCallback,
	                                    C19032PAttrDcCurrentOffset_WriteCallback,
	                                    &attrDcCurrentOffset_LowRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_CURRENT_LOW_RANGE,
	                                    "C19032P_ATTR_IR_CURRENT_LOW_RANGE", 0,
	                                    IVI_VAL_MULTI_CHANNEL | IVI_VAL_NEVER_CACHE,
	                                    C19032PAttrIrCurrentLowRange_ReadCallback,
	                                    C19032PAttrIrCurrentLowRange_WriteCallback,
	                                    &attrIrCurrentLowRangeRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_IR_CURR_HIGH_RANGE,
	                                    "C19032P_ATTR_IR_CURR_HIGH_RANGE", 0,
	                                    IVI_VAL_MULTI_CHANNEL | IVI_VAL_NEVER_CACHE,
	                                    C19032PAttrIrCurrHighRange_ReadCallback,
	                                    C19032PAttrIrCurrHighRange_WriteCallback,
	                                    &attrIrCurrHighRangeRangeTable, 0));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_PRESET_GFI,
	                                    "C19032P_ATTR_PRESET_GFI", 0, 0,
	                                    C19032PAttrPresetGfi_ReadCallback,
	                                    C19032PAttrPresetGfi_WriteCallback));
	checkErr (Ivi_AddAttributeViBoolean (vi, C19032P_ATTR_PRESET_SCREEN,
	                                     "C19032P_ATTR_PRESET_SCREEN", 0, 0, VI_NULL,
	                                     C19032PAttrPresetScreen_WriteCallback));
	checkErr (Ivi_AddAttributeViInt32 (vi, C19032P_ATTR_OSC_RANGE_SELECT,
	                                   "C19032P_ATTR_OSC_RANGE_SELECT", 0, 0,
	                                   VI_NULL, VI_NULL,
	                                   &attrOscRangeSelectRangeTable));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_OSC_CRANGE,
	                                    "C19032P_ATTR_OSC_CRANGE", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrOscCrange_ReadCallback, VI_NULL,
	                                    VI_NULL, 0));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_OSC_CSTANDARD,
	                                    "C19032P_ATTR_OSC_CSTANDARD", 0, 0, VI_NULL,
	                                    C19032PAttrOscCstandard_WriteCallback));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_OSC_CURRENT_OFFSET,
	                                    "C19032P_ATTR_OSC_CURRENT_OFFSET", 0, 0,
	                                    VI_NULL,
	                                    C19032PAttrOscCurrentOffset_WriteCallback));
	checkErr (Ivi_AddAttributeViString (vi, C19032P_ATTR_LC_DISPLAY,
	                                    "C19032P_ATTR_LC_DISPLAY", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcDisplay_ReadCallback,
	                                    C19032PAttrLcDisplay_WriteCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_TIME_DWELL,
	                                    "C19032P_ATTR_LC_TIME_DWELL", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcTimeDwell_ReadCallback,
	                                    C19032PAttrLcTimeDwell_WriteCallback,
	                                    &attrLcTimeDwellRangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_CURRENT_HIGH,
	                                    "C19032P_ATTR_LC_POWER_SOUR_CURRENT_HIGH", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSourCurrentHigh_ReadCallback,
	                                    C19032PAttrLcPowerSourCurrentHigh_WriteCallback,
	                                    &AttrLcPowSouCurrLimit0708RangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_CURRENT_LOW,
	                                    "C19032P_ATTR_LC_POWER_SOUR_CURRENT_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSourCurrentLow_ReadCallback,
	                                    C19032PAttrLcPowerSourCurrentLow_WriteCallback,
	                                    &AttrLcPowSouCurrLimit0708RangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_VA_HIGH,
	                                    "C19032P_ATTR_LC_POWER_SOUR_VA_HIGH", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSourVaHigh_ReadCallback,
	                                    C19032PAttrLcPowerSourVaHigh_WriteCallback,
	                                    &AttrLcPowSouVALimit0708RangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_VA_LOW,
	                                    "C19032P_ATTR_LC_POWER_SOUR_VA_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSourVaLow_ReadCallback,
	                                    C19032PAttrLcPowerSourVaLow_WriteCallback,
	                                    &AttrLcPowSouVALimit0708RangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_POWER_HIGH,
	                                    "C19032P_ATTR_LC_POWER_SOUR_POWER_HIGH", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSourPowerHigh_ReadCallback,
	                                    C19032PAttrLcPowerSourPowerHigh_WriteCallback,
	                                    &AttrLcPowSouVALimit0708RangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_POWER_LOW,
	                                    "C19032P_ATTR_LC_POWER_SOUR_POWER_LOW", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSourPowerLow_ReadCallback,
	                                    C19032PAttrLcPowerSourPowerLow_WriteCallback,
	                                    &AttrLcPowSouVALimit0708RangeTable, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_POWER_SOUR_PFACTOR,
	                                    "C19032P_ATTR_LC_POWER_SOUR_PFACTOR", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcPowerSourPfactor_ReadCallback,
	                                    C19032PAttrLcPowerSourPfactor_WriteCallback,
	                                    &attrLcPowerSourPfactorRangeTable, 0));
	checkErr (Ivi_AddAttributeViBoolean (vi, C19032P_ATTR_LC_UPM,
	                                     "C19032P_ATTR_LC_UPM", 0,
	                                     IVI_VAL_MULTI_CHANNEL,
	                                     C19032PAttrLcUpm_ReadCallback,
	                                     C19032PAttrLcUpm_WriteCallback));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_CURRENT_OFFSET,
	                                    "C19032P_ATTR_LC_CURRENT_OFFSET", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcCurrentOffset_ReadCallback,
	                                    C19032PAttrLcCurrentOffset_WriteCallback,
	                                    VI_NULL, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_CURRENT_OFFSET_LAC,
	                                    "C19032P_ATTR_LC_CURRENT_OFFSET_LAC", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcCurrentOffsetLac_ReadCallback,
	                                    C19032PAttrLcCurrentOffsetLac_WriteCallback,
	                                    VI_NULL, 0));
	checkErr (Ivi_AddAttributeViReal64 (vi, C19032P_ATTR_LC_CURRENT_OFFSET_LDC,
	                                    "C19032P_ATTR_LC_CURRENT_OFFSET_LDC", 0,
	                                    IVI_VAL_MULTI_CHANNEL,
	                                    C19032PAttrLcCurrentOffsetLdc_ReadCallback,
	                                    C19032PAttrLcCurrentOffsetLdc_WriteCallback,
	                                    VI_NULL, 0));
	checkErr (Ivi_SetAttrRangeTableCallback (vi,
	                                         C19032P_ATTR_LC_POWER_SOUR_POWER_LOW,
	                                         C19032PAttrLcPowerSourPowerLow_RangeTableCallback));
	checkErr (Ivi_SetAttrRangeTableCallback (vi,
	                                         C19032P_ATTR_LC_POWER_SOUR_POWER_HIGH,
	                                         C19032PAttrLcPowerSourPowerHigh_RangeTableCallback));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_LC_POWER_SOUR_VA_LOW,
	                                         C19032PAttrLcPowerSourVaLow_RangeTableCallback));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_LC_POWER_SOUR_VA_HIGH,
	                                         C19032PAttrLcPowerSourVaHigh_RangeTableCallback));
	checkErr (Ivi_SetAttrRangeTableCallback (vi,
	                                         C19032P_ATTR_LC_POWER_SOUR_CURRENT_LOW,
	                                         C19032PAttrLcPowerSourCurrentLow_RangeTableCallback));
	checkErr (Ivi_SetAttrRangeTableCallback (vi,
	                                         C19032P_ATTR_LC_POWER_SOUR_CURRENT_HIGH,
	                                         C19032PAttrLcPowerSourCurrentHigh_RangeTableCallback));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_DC_CURRENT_OFFSET,
	                                         C19032PAttrDcCurrentOffset_RangeTableCallback));
	checkErr (Ivi_SetAttrRangeTableCallback (vi, C19032P_ATTR_LC_DC_HIGH_LIMIT,
	                                         C19032PAttrLcDcHighLimit_RangeTableCallback));
                                       
        /*- Setup attribute invalidations -----------------------------------*/            

 
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_GB_CURRENT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_GB_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_STEP_MODE_TYPE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_ARC_LIMIT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_ARC_FILTER, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_RAMP_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_ARC_LIMIT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_ARC_FILTER, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_RAMP_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_STEP_COUNT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_RAMP_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_POWER_MODE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_DEVICE_TYPE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_POWER_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_POWER_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_METER_METHOD, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_QUERY_MEM_INDEX,
	                                        C19032P_ATTR_QUERY_MEM_INDEX_NUM, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AUTO_RESULT_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_STEP_SETTING, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_GB_CURRENT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_GB_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_ARC_LIMIT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_ARC_FILTER, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_ARC_LIMIT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_ARC_FILTER, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_RAMP_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_POWER_MODE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_DEVICE_TYPE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_POWER_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_POWER_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_METER_METHOD, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_PAUSE_MESSAGE_STRING, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_PAUSE_SIGNAL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_RAMP_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_RAMP_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_GB_TWINPORT_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_GB_TWINPORT_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_GB_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_GB_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_GB_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_GB_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_VOLTAGE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_VOLTAGE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_VOLTAGE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_VOLTAGE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_VOLTAGE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_VOLTAGE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_LINE_INPUT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_LINE_INPUT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_GR_SWITCH_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_GR_SWITCH_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_GB_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_GB_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_PAUSE_MESSAGE_STRING, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_PAUSE_SIGNAL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_DWELL_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_OSC_OPEN_CHECK, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_OSC_SHORT_CHECK, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_OSC_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_DWELL_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_OSC_OPEN_CHECK, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_OSC_SHORT_CHECK, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_OSC_CHANNEL_STATE, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_LC_POWER_MODE,
	                                        C19032P_ATTR_LC_POWER_SIMU_TARGET_VOLT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_LC_POWER_MODE,
	                                        C19032P_ATTR_LC_POWER_LIMIT_HIGH, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_LC_POWER_MODE,
	                                        C19032P_ATTR_LC_POWER_LIMIT_LOW, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_PAUSE_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_POWER_SOUR_TARGET_VOLT,
	                                        VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_POWER_SOUR_TARGET_FREQ,
	                                        VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_AC_FALL_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_DC_FALL_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_FALL_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_CURR_RANGE_UPPER, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_AC_HIGH_LIMIT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_LC_DC_HIGH_LIMIT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_DELETE_STEP,
	                                        C19032P_ATTR_IR_CURR_RANGE_AUTO, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_PAUSE_TEST_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_POWER_SOUR_TARGET_VOLT,
	                                        VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_POWER_SOUR_TARGET_FREQ,
	                                        VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_AC_FALL_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_DC_FALL_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_FALL_TIME, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_CURR_RANGE_UPPER, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_AC_HIGH_LIMIT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_LC_DC_HIGH_LIMIT, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_STEP_MODE_TYPE,
	                                        C19032P_ATTR_IR_CURR_RANGE_AUTO, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_IR_CURR_RANGE_UPPER,
	                                        C19032P_ATTR_IR_CURR_RANGE_AUTO, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_IR_CURR_RANGE_AUTO,
	                                        C19032P_ATTR_IR_CURR_RANGE_UPPER, VI_TRUE));
	checkErr (Ivi_AddAttributeInvalidation (vi, C19032P_ATTR_OSC_RANGE_SELECT,
	                                        C19032P_ATTR_OSC_CRANGE, VI_TRUE));
Error:
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
  
